/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file  xos_ams.c
 * @brief The implementatio of AMS client
 *
 * \todo The codes of client is building.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "xos.h"
#include "xos_ams.h"
#include "xos_utils.h"
#include "xos_dbaux.h"
#include "xos_protocol.h"
#include "xos_policy_rule.h"
#include "xos_cache.h"

#define MAXBUF		128


#if DEBUG_INFO
#define AMS_DEBUG(args...)      DEBUG(args)
#else
#define AMS_DEBUG(args...)      ((void)(0))
#endif

int amsclient_usermapping_internal(int sockfd, char *mapped_dn, char *mapped_vo,
			  char *mapped_role, char *mapped_subgroup,
			  AMS_GPASSWD * gpwd, AMS_GGROUPS * primarygrp)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;
//	int uadd = 0, gadd = 0;

	if(sockfd <= 0)  return -1;
/*
	if(mapped_vo && mapped_dn && mapped_role)
		uadd = ams_cache_gpwd_search(mapped_dn,mapped_vo,mapped_role,gpwd);
	if(!uadd && !mapped_subgroup) return 0;

	if(mapped_vo && mapped_subgroup)
		gadd = ams_cache_ggrp_search(mapped_vo,mapped_subgroup,primarygrp);
	if(!gadd && !uadd) return 0;
*/

	if (amsclient_packetmsg  /* opt == 1 */
	    (message, PTL_OT_USERMAPPING, mapped_dn, mapped_vo, mapped_role,
	     mapped_subgroup)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, gpwd, primarygrp, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}
/*	
	if(!ret) {
		if(uadd) ams_cache_gpwd_insert(gpwd);
		if(gadd) ams_cache_ggrp_insert(primarygrp);
	}
*/
	return ret;
	
				
}

int amsclient_usermapping(char *mapped_dn, char *mapped_vo,
			  char *mapped_role, char *mapped_subgroup,
			  AMS_GPASSWD * gpwd, AMS_GGROUPS * primarygrp)
{
	int sockfd = -1, ret = -1;
//	int uadd = 0, gadd = 0;
	
	if (!mapped_vo && (mapped_dn || mapped_subgroup || mapped_role))
		return -1;
/*
	if(mapped_vo && mapped_dn && mapped_role)
		uadd = ams_cache_gpwd_search(mapped_dn,mapped_vo,mapped_role,gpwd);
	if(!uadd && !mapped_subgroup) return 0;

	if(mapped_vo && mapped_subgroup)
		gadd = ams_cache_ggrp_search(mapped_vo,mapped_subgroup,primarygrp);
	if(!gadd && !uadd) return 0;

	AMS_DEBUG("uadd = '%d', gadd = '%d'\n",uadd,gadd);
*/
	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_usermapping_internal(sockfd, mapped_dn, mapped_vo,
			  mapped_role, mapped_subgroup,
			  gpwd, primarygrp);
	
	amsclient_connect_close(sockfd);
/*	
	if(!ret) {
		if(uadd) ams_cache_gpwd_insert(gpwd);
		if(gadd) ams_cache_ggrp_insert(primarygrp);
	}
*/
	return ret;
	
}


#if 0
int amsclient_usermapping(char *mapped_dn, char *mapped_vo,
			  char *mapped_role, char *mapped_subgroup,
			  AMS_GPASSWD * gpwd, AMS_GGROUPS * primarygrp)
{
	ConfData confdata;
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	initialize_confdata(&confdata);
	if (xos_read_config_file(PAM_CONF, &confdata, 0) != 1) {
		AMS_DEBUG
		    ("AMS_Client:Can not read configuration file '%s'",
		     PAM_CONF);
		return -1;
	}

	if (amsclient_packetmsg
	    (message, 1, mapped_dn, mapped_vo, mapped_role,
	     mapped_subgroup)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	if (amsclient_connect
	    (confdata.ams_server, confdata.ams_port, message,
	     &message_len)) {
		AMS_DEBUG("AMS_Client:Can not connect AMS server");
		return -1;
	}

	if (amsclient_unpacketmsg(message, gpwd, primarygrp, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	return ret;
}
#endif


int amsclient_mappinginfo_internal(int sockfd, char *mapped_dn, char *mapped_vo,
			  char *mapped_subgroup, AMS_GPASSWD * gpwd,
			  AMS_GGROUPS * primarygrp)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;
	int uadd = 0, gadd = 0;

	if(sockfd <= 0) return -1;
	
	if (!mapped_vo && (mapped_dn || mapped_subgroup))
		return -1;
		
	memset(message, 0, PACKAGE_MAXLEN);
	
	if(mapped_vo && mapped_dn && mapped_subgroup)
		uadd = ams_cache_gpwd_search(mapped_dn,mapped_vo,mapped_subgroup,gpwd);
	if(!uadd && !mapped_subgroup) return 0;

	if(mapped_vo && mapped_subgroup)
		gadd = ams_cache_ggrp_search(mapped_vo,mapped_subgroup,primarygrp);
	if(!gadd && !uadd) return 0;


	/* prepare package */
	if (mapped_dn && mapped_vo && mapped_subgroup) {
		if (amsclient_packetmsg
		    (message, PTL_OT_GETBYACTGRP, mapped_dn, mapped_vo, mapped_subgroup,
		     NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else if (mapped_vo && mapped_subgroup) {
		if (amsclient_packetmsg
		    (message, PTL_OT_GETGRP, mapped_dn, mapped_vo, mapped_subgroup,
		     NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	}
	
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	/* unpack the packet */
	if (amsclient_unpacketmsg(message, gpwd, primarygrp, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}	
	
	if(!ret) {
		if(uadd) ams_cache_gpwd_insert(gpwd);
		if(gadd) ams_cache_ggrp_insert(primarygrp);
	}
	return ret;
}


int amsclient_mappinginfo(char *mapped_dn, char *mapped_vo,
			  char *mapped_subgroup, AMS_GPASSWD * gpwd,
			  AMS_GGROUPS * primarygrp)
{
	int sockfd = -1, ret = -1;
	int uadd = 0, gadd = 0;

	if (!mapped_vo && (mapped_dn || mapped_subgroup))
		return -1;

	if(mapped_vo && mapped_dn && mapped_subgroup)
		uadd = ams_cache_gpwd_search(mapped_dn,mapped_vo,mapped_subgroup,gpwd);
	if(!uadd && !mapped_subgroup) return 0;

	if(mapped_vo && mapped_subgroup)
		gadd = ams_cache_ggrp_search(mapped_vo,mapped_subgroup,primarygrp);
	if(!gadd && !uadd) return 0;


	AMS_DEBUG("uadd = '%d', gadd = '%d'",uadd,gadd);

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_mappinginfo_internal(sockfd, mapped_dn,mapped_vo,mapped_subgroup,gpwd,primarygrp);
	
	amsclient_connect_close(sockfd);

	if(!ret) {
		if(uadd) ams_cache_gpwd_insert(gpwd);
		if(gadd) ams_cache_ggrp_insert(primarygrp);
	}


	return ret;
		

}


int amsclient_clearmapping_internal(int sockfd, char *mapped_dn, char *mapped_vo,
			   char *mapped_subgroup)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	if (!mapped_vo && (mapped_dn || mapped_subgroup))
		return -1;
		
	memset(message, 0, PACKAGE_MAXLEN);
	
	/* clear all the items in database */
	if (!mapped_vo && !mapped_dn && !mapped_subgroup) {
		if (amsclient_packetmsg  /* opt == 4 */
		    (message, PTL_OT_CLEARACTGRP, NULL, NULL, NULL, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	}

	/* clear account */
	if (mapped_vo && mapped_dn && mapped_subgroup) {
		if (amsclient_packetmsg  /* opt == 2 */
		    (message, PTL_OT_CLEARACCT, mapped_dn, mapped_vo, mapped_subgroup,
		     NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	}
	/* clear group */
	else if (mapped_vo && mapped_subgroup) {
		if (amsclient_packetmsg /* opt == 3 */
		    (message, PTL_OT_CLEARGRP, NULL, mapped_vo, NULL, mapped_subgroup)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	}
	
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}
	
	if(!ret) {
			ams_cache_whole_erase(mapped_dn,mapped_vo,mapped_subgroup,0,0);
	}

	return ret;			
}


int amsclient_clearmapping(char *mapped_dn, char *mapped_vo,
			   char *mapped_subgroup)
{
	int sockfd = -1, ret = -1;

	if (!mapped_vo && (mapped_dn || mapped_subgroup))
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_clearmapping_internal(sockfd, mapped_dn, mapped_vo,
			   mapped_subgroup);
	
	amsclient_connect_close(sockfd);

	if(!ret) {
			ams_cache_whole_erase(mapped_dn,mapped_vo,mapped_subgroup,0,0);
	}
	return ret;

}

int amsclient_clearuidgid_internal(int sockfd, int clear_uid, int clear_gid)
{
	char message[PACKAGE_MAXLEN];
	char buf_uid[MAXBUF], buf_gid[MAXBUF];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	
	/*TODO: pre-determine whether uid/gid is in given boundary */

	memset(message, 0, PACKAGE_MAXLEN);
	memset(buf_uid, 0, MAXBUF);
	memset(buf_gid, 0, MAXBUF);
	
	if (clear_uid > 0 && clear_gid > 0) {
		sprintf(buf_uid, "%d", clear_uid);
		sprintf(buf_gid, "%d", clear_gid);

		if (amsclient_packetmsg  /* opt == 7 */
		    (message, PTL_OT_CLEARUIDGID, buf_uid, buf_gid, NULL, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else if (clear_uid > 0) {
		sprintf(buf_uid, "%d", clear_uid);

		if (amsclient_packetmsg  /* opt == 7 */
		    (message, PTL_OT_CLEARUIDGID, buf_uid, NULL, NULL, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else if (clear_gid > 0) {
		sprintf(buf_gid, "%d", clear_gid);

		if (amsclient_packetmsg /* opt == 7 */
		    (message, PTL_OT_CLEARUIDGID, NULL, buf_gid, NULL, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else
		return -1;
		
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}	

	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	if(!ret)
		ams_cache_whole_erase(NULL,NULL,NULL,clear_uid,clear_gid);


	return ret;
		
}

int amsclient_clearuidgid(int clear_uid, int clear_gid)
{
	int sockfd = -1, ret = -1;

	/*TODO: pre-determine whether uid/gid is in given boundary */

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_clearuidgid_internal(sockfd, clear_uid, clear_gid);
	
	amsclient_connect_close(sockfd);

	if(!ret)
		ams_cache_whole_erase(NULL,NULL,NULL,clear_uid,clear_gid);

	return ret;	
}

int amsclient_invmappinginfo_internal(int sockfd, char *username, int user_uid, char *groupname,
			     int group_gid, AMS_GPASSWD * gpwd,
			     AMS_GGROUPS * ggrp)
{
	char message[PACKAGE_MAXLEN];
	char buf_uid[MAXBUF], buf_gid[MAXBUF];
	int message_len = -1, ret = -1;
	int t_uid, t_gid;
	int uadd = 0, gadd = 0;

	if(sockfd <= 0) return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(buf_uid, 0, MAXBUF);
	memset(buf_gid, 0, MAXBUF);
	
	if(username || user_uid > 0)
		uadd = ams_cache_uidgid_search(username,user_uid,NULL,0,gpwd,NULL);
	if(!uadd && !groupname && group_gid <= 0) return 0;

	if(groupname || group_gid > 0)
		gadd = ams_cache_uidgid_search(NULL,0,groupname,group_gid,NULL,ggrp);
	if(!gadd && !uadd) return 0;

	
	t_uid = user_uid;
	t_gid = group_gid;
	if (t_uid > 0 && t_gid > 0) {
		sprintf(buf_uid, "%d", t_uid);
		sprintf(buf_gid, "%d", t_gid);

		if (amsclient_packetmsg /* opt == 8 */
		    (message, PTL_OT_GETBYUIDGID, username, buf_uid, groupname, buf_gid)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else if (t_uid > 0) {
		sprintf(buf_uid, "%d", t_uid);
		if (amsclient_packetmsg  /* opt == 8*/
		    (message, PTL_OT_GETBYUIDGID, username, buf_uid, groupname, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else if (t_gid > 0) {
		sprintf(buf_gid, "%d", t_gid);
		if (amsclient_packetmsg  /* opt == 8 */
		    (message, PTL_OT_GETBYUIDGID, username, NULL, groupname, buf_gid)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else {
		if (amsclient_packetmsg /* opt == 8 */
		    (message, PTL_OT_GETBYUIDGID, username, NULL, groupname, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, gpwd, ggrp, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}
		     
	if(!ret){
                if(uadd) ams_cache_gpwd_insert(gpwd);
                if(gadd) ams_cache_ggrp_insert(ggrp);
	}
	return ret;			
}


int amsclient_invmappinginfo(char *username, int user_uid, char *groupname,
			     int group_gid, AMS_GPASSWD * gpwd,
			     AMS_GGROUPS * ggrp)
{
	int sockfd = -1, ret = -1;
	int uadd = 0, gadd = 0;

	if(username || user_uid > 0)
		uadd = ams_cache_uidgid_search(username,user_uid,NULL,0,gpwd,NULL);
	if(!uadd && !groupname && group_gid <= 0) return 0;

	if(groupname || group_gid > 0)
		gadd = ams_cache_uidgid_search(NULL,0,groupname,group_gid,NULL,ggrp);
	if(!gadd && !uadd) return 0;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
		
	ret = amsclient_invmappinginfo_internal(sockfd, username, user_uid, groupname,
			     group_gid, gpwd, ggrp);
		
	amsclient_connect_close(sockfd);
		     
	if(!ret){
                if(uadd) ams_cache_gpwd_insert(gpwd);
                if(gadd) ams_cache_ggrp_insert(ggrp);
	}

	return ret;
}


/* operate the mount point of uid/gid structure */

int amsclient_optmountnum_internal(int sockfd, char *opt_cmd, int user_uid, int mount_para)
{
	char message[PACKAGE_MAXLEN];
	char buf_uid[MAXBUF], buf_param[MAXBUF];
	int message_len = -1, ret = -1;
	int t_uid, t_mount_param;

	if(sockfd <= 0 || !opt_cmd) return -1;
	if(user_uid <= 0) return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(buf_uid, 0, MAXBUF);
	memset(buf_param, 0, MAXBUF);
		
	t_uid = user_uid;
	t_mount_param = mount_para;
	if (t_uid > 0 && t_mount_param > 0) {
		sprintf(buf_uid, "%d", t_uid);
		sprintf(buf_param, "%d", t_mount_param);

		if (amsclient_packetmsg /* opt == 22 */
		    (message, PTL_OT_OPTMOUNTCNT, opt_cmd, buf_uid, buf_param, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} else  {
		sprintf(buf_uid, "%d", t_uid);
		if (amsclient_packetmsg  /* opt == 22*/
		    (message, PTL_OT_OPTMOUNTCNT, opt_cmd, buf_uid, NULL, NULL)) {
			AMS_DEBUG
			    ("AMS_Client:Can not built message package");
			return -1;
		}
	} 

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	return ret;			
}

int amsclient_optmountnum(char *opt_cmd, int user_uid, int mount_para)
{
	int sockfd = -1, ret = -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret =  amsclient_optmountnum_internal(sockfd, opt_cmd, user_uid, mount_para);	
		
	amsclient_connect_close(sockfd);
		     
	return ret;
}

int amsclient_changeuserinfo_internal(int sockfd, int luid, char *hm_dir)
{
	char message[PACKAGE_MAXLEN];
	char buf_uid[MAXBUF];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	/* luid should be between MAX_UID and MIN_UID defined in
	 * AMS policy rules */
	if(luid < 0 || hm_dir == NULL) return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(buf_uid, 0, MAXBUF);
	sprintf(buf_uid, "%d", luid);


	if (amsclient_packetmsg  /* opt == 21 */
			(message, PTL_OT_CHUSERINFO, buf_uid, hm_dir, NULL,
			 NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

        if(!ret)
                ams_cache_whole_erase(NULL,NULL,NULL,luid,0);

	return ret;

}

int amsclient_changeuserinfo(int luid, char *hm_dir)
{
	int sockfd = -1, ret = -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}


	ret = amsclient_changeuserinfo_internal(sockfd, luid, hm_dir);

	amsclient_connect_close(sockfd);

	return ret;
}


int amsclient_umounthome_internal(int sockfd, int luid, char *hm_dir)
{
	char message[PACKAGE_MAXLEN];
	char buf_uid[MAXBUF];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	/* luid should be between MAX_UID and MIN_UID defined in
	 * AMS policy rules */
	if(luid < 0 || hm_dir == NULL) return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(buf_uid, 0, MAXBUF);
	sprintf(buf_uid, "%d", luid);


	if (amsclient_packetmsg  
			(message, PTL_OT_UMOUNTHOME, buf_uid, hm_dir, NULL,
			 NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}


	return ret;

}

int amsclient_umounthome(int luid, char *hm_dir)
{
	int sockfd = -1, ret = -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}


	ret = amsclient_umounthome_internal(sockfd, luid, hm_dir);

	amsclient_connect_close(sockfd);

	return ret;
}


int amsclient_policytranslate_internal(int sockfd, char *username, char *attrs)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	
	if (!username || !attrs)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
			
	if (amsclient_packetmsg(message, PTL_OT_SETUPQUOTA/* opt == 9 */, username, attrs, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	return ret;		
	
}


int amsclient_policytranslate(char *username, char *attrs)
{

	int sockfd = -1, ret = -1;

	if (!username || !attrs)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}	
	ret = amsclient_policytranslate_internal(sockfd, username, attrs);

	amsclient_connect_close(sockfd);
	
	return ret;
}

int amsclient_policyclear_internal(int sockfd, char *username, char *attrs)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	if (!username || !attrs)
		return -1;	
		
	memset(message, 0, PACKAGE_MAXLEN);
	
	if (amsclient_packetmsg(message, PTL_OT_CLEARQUOTA /* opt == 10 */, 
				username, attrs, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}
	
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	return ret;
			
}


int amsclient_policyclear(char *username, char *attrs)
{
	int sockfd = -1, ret = -1;

	if (!username || !attrs)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_policyclear_internal(sockfd, username, attrs);
	
	amsclient_connect_close(sockfd);
	
	return ret;
}

/* ams client for temp information operations */
#if 0
int amsclient_unpacketmsg_tmpinfo(char *message, char *tmpinfo, int *ret)
{
	if (!message)
		return -1;

	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '@') {
			*p = '\0';

			if (tno == 0)
				*ret = atoi(pp);

			if (tno == 1 && p != pp + 1)
				strcpy(tmpinfo, pp + 1);

			pp = p;
			tno++;
		}
		p++;
	}

	return 0;
}
#endif

int amsclient_tmpinfosave_internal(int sockfd, char *tmpinfo, char *rkey)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	if (!tmpinfo || !rkey)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	
	if (amsclient_packetmsg(message, PTL_OT_SAVETMPINFO /* opt == 11 */, 
				tmpinfo, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amsclient_unpacketmsg_tmpinfo(message, rkey, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	return ret;
		
				
}

int amsclient_tmpinfosave(char *tmpinfo, char *rkey)
{
	int sockfd = -1, ret = -1;

	if (!tmpinfo || !rkey)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}

	ret = amsclient_tmpinfosave_internal(sockfd, tmpinfo, rkey);
	
	amsclient_connect_close(sockfd);
	
	return ret;
}

int amsclient_tmpinfoget_internal(int sockfd, char *key, char **tmpinfo)
{
	char message[PACKAGE_MAXLEN];
	char info[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	if (!tmpinfo || !key)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	
	if (amsclient_packetmsg(message, PTL_OT_GETTMPINFO /* opt == 12 */, 
				key, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg_tmpinfo(message, info, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	*tmpinfo = strdup(info);

	return ret;
					
}

int amsclient_tmpinfoget(char *key, char **tmpinfo)
{
	int sockfd = -1, ret = -1;

	if (!tmpinfo || !key)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_tmpinfoget_internal(sockfd, key, tmpinfo);
	
	amsclient_connect_close(sockfd);
	
	return ret;
}


/* ams client for managing policy rules */

int amsclient_ruleput_internal(int sockfd, struct rule_t *xrule, char **handler)
{
	char message[PACKAGE_MAXLEN];
	char rule_buf[PACKAGE_MAXLEN];
	char rule_key[32];
	int message_len = -1, ret = -1;

	if (sockfd <= 0 || !xrule)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(rule_buf, 0, PACKAGE_MAXLEN);
	if (protocol_packetrule(rule_buf, xrule)) {
		AMS_DEBUG("AMS_Client:Can not packet rule data");
		return -1;
	}

	if (amsclient_packetmsg
	    (message, PTL_OT_RULEPUT, rule_buf, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}
	
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amscient_unpacketmsg_rule(message, NULL, rule_key, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	if (ret == 0)
		*handler = strdup(rule_key);

	return ret;
				
}


int amsclient_ruleput(struct rule_t *xrule, char **handler)
{
	int sockfd = -1, ret = -1;

	if (!xrule)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_ruleput_internal(sockfd, xrule, handler);
	
	amsclient_connect_close(sockfd);

	return ret;
}

int amsclient_getrulebykey_internal(int sockfd, unsigned int skey, struct rule_t **xrule,
			   char **handler)
{
	char message[PACKAGE_MAXLEN];
	char rule_key[32];
	char i_skey[32];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	
	memset(message, 0, PACKAGE_MAXLEN);
	
	if (skey < 1)
		return ret;
	else
		sprintf(i_skey, "%d", skey);

	if (amsclient_packetmsg
	    (message, PTL_OT_GETRULEBYKEY, i_skey, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	*xrule = (struct rule_t *) malloc(sizeof(struct rule_t));

	if (amscient_unpacketmsg_rule(message, *xrule, rule_key, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	if (ret == 0)
		*handler = strdup(rule_key);
	else {
		free(*xrule);
		*xrule = NULL;
	}

	return ret;
			
}


int amsclient_getrulebykey(unsigned int skey, struct rule_t **xrule,
			   char **handler)
{
	int sockfd = -1, ret = -1;

	if (skey < 1)
		return ret;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
		
	ret = amsclient_getrulebykey_internal(sockfd, skey, xrule,
			   handler);

	amsclient_connect_close(sockfd);
	
	return ret;

}

int amsclient_getrulebyhandler_internal(int sockfd, char *skey, struct rule_t **xrule)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	if (!skey || skey[0] == '\0')
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	
	if (amsclient_packetmsg
	    (message, PTL_OT_GETBYHANDLER, skey, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	*xrule = (struct rule_t *) malloc(sizeof(struct rule_t));

	if (amscient_unpacketmsg_rule(message, *xrule, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	if (ret) {
		free(*xrule);
		*xrule = NULL;
	}

	return ret;
					
}


int amsclient_getrulebyhandler(char *skey, struct rule_t **xrule)
{

	int sockfd = -1, ret = -1;

	if (!skey || skey[0] == '\0')
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_getrulebyhandler_internal(sockfd, skey, xrule);
	
	amsclient_connect_close(sockfd);

	return ret;
}

int amsclient_ruledelete_internal(int sockfd, char *handler)
{
	char message[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;
	int opt = -1;
	char *hdl = NULL;

	if(sockfd <= 0) return -1;
	
	if (!handler)
		opt = PTL_OT_CLEARRULES;
	else {
		opt = PTL_OT_DELETERULE;
		hdl = handler;
	}

	memset(message, 0, PACKAGE_MAXLEN);
	
	if (amsclient_packetmsg(message, opt, hdl, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}


	if (amscient_unpacketmsg_rule(message, NULL, NULL, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	return ret;
			
}


int amsclient_ruledelete(char *handler)
{
	int sockfd = -1, ret = -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
		
	ret = amsclient_ruledelete_internal(sockfd, handler);

	amsclient_connect_close(sockfd);
	
	return ret;
}


int amsclient_rulesearch_internal(int sockfd, struct rule_t *xrule, char **handler)
{
	char message[PACKAGE_MAXLEN];
	char rule_buf[PACKAGE_MAXLEN];
	char rule_key[32];
	int message_len = -1, ret = -1;

	if(sockfd <= 0) return -1;
	
	if (!xrule || !handler)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(rule_buf, 0, PACKAGE_MAXLEN);
	memset(rule_key, 0, 32);
	
	if (protocol_packetrule(rule_buf, xrule)) {
		AMS_DEBUG("AMS_Client:Can not packet rule data");
		return -1;
	}

	if (amsclient_packetmsg
	    (message, PTL_OT_SEARCHRULE, rule_buf, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}

	if (amscient_unpacketmsg_rule(message, NULL, rule_key, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	if (ret == 0)
		*handler = strdup(rule_key);

	return ret;
				
}


int amsclient_rulesearch(struct rule_t *xrule, char **handler)
{
	int sockfd = -1, ret = -1;

	if (!xrule || !handler)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_rulesearch_internal(sockfd, xrule, handler);
	
	amsclient_connect_close(sockfd);

	return ret;
}


int amsclient_inblacklist_internal(int sockfd, struct subject_t *sub, int *in_blacklist)
{
	char message[PACKAGE_MAXLEN];
	char sub_buf[PACKAGE_MAXLEN];
	char ret_buf[PACKAGE_MAXLEN];
	int message_len = -1, ret = -1;;

	if(sockfd <= 0) return -1;
	if (!sub || !in_blacklist)
		return -1;
	memset(message, 0, PACKAGE_MAXLEN);
	memset(sub_buf, 0, PACKAGE_MAXLEN);
	memset(ret_buf, 0, PACKAGE_MAXLEN);

	if (protocol_packetsubject(sub_buf, sub)) {
		AMS_DEBUG("AMS_Client:Can not packet subject data");
		return -1;
	}

	if (amsclient_packetmsg
	    (message, PTL_OT_ACREJECT, sub_buf, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amscient_unpacketmsg_rule(message, NULL, ret_buf, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the rule package");
		return -1;
	}

	if (ret == 0)
		*in_blacklist = atoi(ret_buf);

	return ret;
				
}


int amsclient_inblacklist(struct subject_t *sub, int *in_blacklist)
{
	int sockfd = -1, ret = -1;;

	if (!sub || !in_blacklist)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_inblacklist_internal(sockfd, sub, in_blacklist);

	amsclient_connect_close(sockfd);

	return ret;
}


int amsclient_enforcedrules_internal(int sockfd, char *username, char *rule_buf)
{
	char message[PACKAGE_MAXLEN];
	char rules[PACKAGE_MAXLEN];

	if(sockfd <= 0) return -1;
	int message_len = -1, ret = -1;

	if (!username || !rule_buf)
		return -1;

	memset(message, 0, PACKAGE_MAXLEN);
	memset(rules, 0, PACKAGE_MAXLEN);

	if (amsclient_packetmsg
	    (message, PTL_OT_ENFORCERULE, username, NULL, NULL, NULL)) {
		AMS_DEBUG("AMS_Client:Can not built message package");
		return -1;
	}
	
	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		AMS_DEBUG("AMS_Client: Can not send query to AMS");
		return -1;
	}
	
	if (amsclient_unpacketmsg_tmpinfo(message, rules, &ret)) {
		AMS_DEBUG("AMS_Client:Can not unpacket the message");
		return -1;
	}

	if (rules[0] != '\0')
		strcpy(rule_buf, rules);

	return ret;
				
}



int amsclient_enforcedrules(char *username, char *rule_buf)
{
	int sockfd = -1, ret = -1;

	if (!username || !rule_buf)
		return -1;

	if((sockfd = amsclient_connect_open()) <= 0) {
		AMS_DEBUG("AMS_Client: Can not setup the AMS connection");
		return -1;
	}
	
	ret = amsclient_enforcedrules_internal(sockfd, username, rule_buf);
	
	amsclient_connect_close(sockfd);

	return ret;
}
