/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <sys/un.h>
#include <sys/stat.h>

#include "xos_dbaux.h"
#include "map_mech.h"
#include "xos_rquota.h"
#include "xos_dbaux_tmp.h"
#include "xos_protocol.h"
#include "xos_policy_rule.h"
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"
#include "xos_cache.h"
#include "xos_lxc.h"

#include <sys/sem.h>

/* PL commented out for the moment to fix build 03 Nov 2010
 * 
 * struct ucred {
  pid_t  pid;  
  uid_t  uid;  
  gid_t  gid;  
};
*/

#define SEMKEYMOUNT (key_t)0x2002
#define IFLAGS (IPC_CREAT|IPC_EXCL)

union semun{
  int val;
  struct semid_ds *buf;
  unsigned short * ary;
};
int mutex = 0;

void sem_exit(void){
  semctl(mutex, 0, IPC_RMID, 0); 
}

int sem_init(key_t key, int inival)
{
  int semid;
  union semun arg;
  semid=semget(key,1,0660|IFLAGS);
  arg.val=inival;
  semctl(semid, 0, SETVAL, arg);
  return semid;
}


extern int errno;


static int _init = 0;
static const char *unix_sock_name = "/var/run/ams_server.socket";
static int server_pid;
static int i = 0;

void AMS_PRINT(int flag, const char *fmt, ...)
{
  /*if (flag)
    return;*/
	char msgbuf[1024];
	va_list args;
	va_start(args, fmt);
	vsnprintf(msgbuf, sizeof(msgbuf), fmt, args);
	printf(msgbuf);
	va_end(args);
}

void sigterm_handler_delete_socket(int signumber)
{
	if (getpid() == server_pid)
		unlink(unix_sock_name);
	exit(-1);
}

void initiate_databases_mapping()
{


	struct rule_t account_rule = {
		XPLY_RT_MA,
		{XPLY_ST_ACT, "*", "xtreemos"},
		{XPLY_OT_ACT, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};

	struct rule_t group_rule = {
		XPLY_RT_MA,
		{XPLY_ST_GRP, "xtreemos", "*"},
		{XPLY_OT_GRP, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};

	struct rule_t account_rule_automnt = {
		XPLY_RT_MA,
		{XPLY_ST_ACT, "*", "dff93ffe-87f3-476a-b573-983f92623f95"},
		{XPLY_OT_ACT, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};

	struct rule_t group_rule_automnt = {
		XPLY_RT_MA,
		{XPLY_ST_GRP, "dff93ffe-87f3-476a-b573-983f92623f95", "*"},
		{XPLY_OT_GRP, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};


	struct rule_t resquota_rule = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-oscar", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_NOFILE}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_nproc = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-shu", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_NPROC}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_cpu = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-haiyan", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_CPU}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_mem = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-yvon", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_MEM}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t reject_rule = {
		XPLY_RT_AC,
		{XPLY_ST_ACT, "*", "chemVO"},
		{XPLY_OT_NODE, {.objc_object = "127.0.0.1"}},
		{.rc_perm = 0},
		1,
		1234567
	};


	resquota_rule.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule.r_content.rc_quota.quo_val = 5;

	resquota_rule_nproc.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_nproc.r_content.rc_quota.quo_val = 4;

	resquota_rule_cpu.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_cpu.r_content.rc_quota.quo_val = 4;

	resquota_rule_mem.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_mem.r_content.rc_quota.quo_val = 1024 * 6;

	if (ruleinfo_db_put(&account_rule, NULL)
	    || ruleinfo_db_put(&group_rule, NULL)
	    || ruleinfo_db_put(&account_rule_automnt, NULL)
	    || ruleinfo_db_put(&group_rule_automnt, NULL)
	    || ruleinfo_db_put(&resquota_rule, NULL)
	    || ruleinfo_db_put(&resquota_rule_nproc, NULL)
	    || ruleinfo_db_put(&resquota_rule_cpu, NULL)
	    || ruleinfo_db_put(&resquota_rule_mem, NULL)
	    || ruleinfo_db_put(&reject_rule, NULL))
		printf("fail to input basical rules in database\n");

}

void initiate_databases_setting()
{
	struct rule_t setting_rule = {
		XPLY_RT_MA,
		{XPLY_ST_GRP, "xtreemos", "*"},
		{XPLY_OT_GRP, {.objc_object = "*"}},
		{.rc_perm = 0},
		1,
		1234567
	};


	sprintf(setting_rule.r_subject.sub_content_1, "%d", 60500);
	sprintf(setting_rule.r_subject.sub_content_2, "%d", 60000);
	sprintf(setting_rule.r_content.rc_driver.driver_name, "%d", 60500);
	sprintf(setting_rule.r_content.rc_driver.driver_params, "%d",
			60000);

	if (ruleinfo_db_put_aux(&setting_rule, -1, NULL))
		printf("fail to input setting rules in database\n");
}

void amsserver_init(int f_init)
{
	char *db_file = NULL;
	int size = 0;
	struct stat file_stat;

	if (f_init) {
		/* resetup databases */
		tmpinfo_db_clearall();

		/* initiate policy databases */
		ruleinfo_db_clearall();
		initiate_databases_mapping();
		clear_all();
	} else {
		/* if ruledb does not exist, initiate a new one */
		size = strlen(RULEDB_DIR) + strlen(RULEINFO_DB) + 1;
		db_file = malloc(size);
		snprintf(db_file, size, "%s%s", RULEDB_DIR, RULEINFO_DB);

		if (stat(db_file, &file_stat) != 0)
			initiate_databases_setting();
	}

	ams_cache_create();
}




void amsserver_create_sranddbkey(char *randstr, int len, int cseed)
{
	int index;
	int i;
	//static int init = 0;
	unsigned int seed;

	if (!_init) {
		seed = (unsigned int) time((time_t *) 0);
		_init = 1;
	}
	seed = cseed;

	for (i = 0; i < len; i++) {
		index =
		    (int) (((float) 62) * rand_r(&seed) /
			   (RAND_MAX + 1.0));

		if (index < 10)
			randstr[i] = (48 + index);
		else if (index < 36)
			randstr[i] = (55 + index);
		else if (index < 62)
			randstr[i] = (61 + index);
	}
	randstr[len] = '\0';
}

int
clear_mapped_group(GGROUPS * old_ggrp, GGROUPS * new_ggrp,
		   GGROUPS * r_ggrp)
{
	int ret = -1;

	if (!old_ggrp)
		return 0;

	/* reclaim the free groups */
	if (old_ggrp->l_grptoken.g_members[0] == '\0')
		ret = clear_group(old_ggrp);

	return ret;
}

int
clear_expired_account(GPASSWD * gpwd, int *expired_time, GPASSWD * r_gpwd)
{
	int ret = -1;
	if (gpwd->g_time <= *expired_time)
		ret = bdb_deldbgtk_pwd(&(gpwd->g_idtoken));
	return ret;
}

int
clear_expired_group(GGROUPS * ggrp, int *expired_time, GGROUPS * r_ggrp)
{
	int ret = -1;
	if (ggrp->g_time <= *expired_time)
		ret = bdb_deldbgtk_grp(&(ggrp->g_grptoken));
	return ret;
}

void clear_expired_id_timer(int flag, int expired_span)
{
	int lt_tick, expired_time;
	lt_tick = time(NULL);
	AMS_PRINT(flag, "local time: %d\n", lt_tick);

	/* At first day of each month, the proceduce will clear the expired uid/gid 
	 * in databases */

	expired_time = lt_tick - expired_span;
	bdb_operatedb_pwd(&expired_time, NULL,
			  (OPTPWDFUNC) clear_expired_account);
	bdb_operatedb_grp(&expired_time, NULL,
			  (OPTGRPFUNC) clear_expired_group);

}

void free_gpwd_ggrp(GPASSWD *gpwd, GGROUPS *ggrp)
{
	if(gpwd) {
		free(gpwd);
		gpwd = NULL;
	}
	if(ggrp) {
		free(ggrp);
		ggrp = NULL;
	}
}

void chld_signal(int sig)
{
	int status, child_val;
	if (waitpid(-1, &status, WNOHANG) < 0) {

		fprintf(stderr, "waitpid failed\n");
		return;
	}
	if (WIFEXITED(status)) {
		child_val = WEXITSTATUS(status);

		printf("child's exited normally with status %d\n",child_val);
		//ams_cache_destroy();
	}
}

void shmem_signal(int sig)
{
	ams_cache_destroy();
	exit(-1);
}

int amsd_memallocate(void **inbuf, int size)
{
	if (!inbuf || !size)
		return -1;

	*inbuf = malloc(size);
	memset(*inbuf, 0, size);

	return 0;
}

void usage(char *pgrname)
{
	printf("Account Mapping Service (AMS) of VO-support in node\n");
	printf("This componnet is part of XtreemOS project\n");
	printf("\nUsage: %s   [-init] [-unix] [-port xxx] [-expired xxx]\n\n",pgrname);
	printf("\t-init\t\tinitiate default configuration for AMS\n");
	printf("\t-unix\t\tuse the UNIX socket for secure connection\n");
	printf("\t-port\t\tspecify the port for AMS running\n");
	printf("\t-expired\tthe expired time to clear expired uid/gid (in second)\n");
	printf("\n--- XtreemOS team ---\n");
}


int main(int argc, char *argv[])
{
	char *mapped_dn, *mapped_vo, *mapped_role, *mapped_subgroups;
	struct sockaddr_in serveraddr;
	struct sockaddr_in clientaddr;
	struct sockaddr_un serveraddr_unix;
	int sockfd, temp_sockfd;
	int ret, i, flag, status, port, f_init;
	OPTTYPE optcode;
	socklen_t clientaddr_size;
	char inbuf[16384], outbuf[16384];
	pid_t pid, cnt_pid;
	GPASSWD *gpwd = NULL;
	GGROUPS *ggrp = NULL;
	int server_is_local = 0;
	int expired_time_span = 60 * 60 * 60 * 24 * 30 * 6;	/* 6 month */


	ret = -1;
	flag = 0;
	f_init = 0;
	port = 8000;
	clientaddr_size = sizeof(struct sockaddr_in);

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strncmp(argv[i], "-d", 2)) {
			flag = 1;
		} else if (!strncmp(argv[i], "-init", 5)) {
			f_init = 1;
		} else if (!strncmp(argv[i], "-port", 5)) {
			port = atoi(argv[++i]);
		} else if (!strncmp(argv[i], "-unix", 5)) {
			server_is_local = 1;
                } else if (!strncmp(argv[i], "-help", 5) || !strncmp(argv[i], "--help", 6) || !strncmp(argv[i], "-h", 2)) {
			usage(argv[0]);
			return 0;
		} else if (!strncmp(argv[i], "-expired", 8)) {
			expired_time_span = atoi(argv[++i]);
		} else
			printf("AMS:Unknown parameters !\n");
	}

	/* Deamon mode */
	if (!flag) {
		if ((pid = fork()) < 0)
			return -1;

		else if (pid != 0)
			exit(0);
	}
	
	struct sigaction chld_action, shmem_action;
	/*
	chld_action.sa_handler = chld_signal;
	sigemptyset(&chld_action.sa_mask);
	chld_action.sa_flags = 0;
	if (sigaction(SIGCHLD, &chld_action, NULL) < 0) {
		perror("chld sigaction");
	}
	*/
	if (!flag){
	int fd =  open("/dev/null", O_RDWR);
	dup2(fd,0);
	dup2(fd,1);
	}


	shmem_action.sa_handler = shmem_signal;
	sigemptyset(&shmem_action.sa_mask);
	shmem_action.sa_flags = 0;
	if (sigaction(SIGINT, &shmem_action, NULL) < 0) {
		perror("shmem sigaction");
	}

	sockfd =
	    server_is_local ? socket(PF_UNIX, SOCK_SEQPACKET,
				     0) : socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd == -1) {
		perror("AMS:call to socket");
		exit(1);
	}
	u_int yes = 1;
	if (server_is_local) {
		int termination_signals[7] =
		    { SIGTERM, SIGHUP, SIGINT, SIGQUIT, SIGILL, SIGABRT,
			SIGALRM
		};
		int i;

		bzero(&serveraddr_unix, sizeof(serveraddr_unix));
		serveraddr_unix.sun_family = AF_UNIX;
		strcpy(serveraddr_unix.sun_path, unix_sock_name);
		// remove socket file if exists; ignore error
		// TODO: check if other instance of the server is already running
		unlink(unix_sock_name);
		errno = 0;
		if (bind
		    (sockfd, (struct sockaddr *) &serveraddr_unix,
		     sizeof(serveraddr_unix)) == -1) {
			perror("AMS:call to bind");
			exit(1);
		}
		// write permission to all
		chmod(unix_sock_name,
		      S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IWGRP |
		      S_IWOTH);

		// register signals to delete socket at termination; only if they are not already registered
		server_pid = getpid();	// this is need to distingish server main process in signal handler.
		for (i = 0; i < 6; i++) {
			struct sigaction term_action;
			sigaction(termination_signals[i], NULL,
				  &term_action);
			if (term_action.sa_handler == SIG_DFL) {
				term_action.sa_handler =
				    sigterm_handler_delete_socket;
				sigemptyset(&term_action.sa_mask);
				sigaction(termination_signals[i],
					  &term_action, NULL);
			}
		}


	} else {
		/* allow multiple sockets to use the same PORT number */
		if (setsockopt
		    (sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes))
		    < 0) {
			perror("Reusing ADDR failed");
			exit(1);
		}

		bzero(&serveraddr, sizeof(serveraddr));
		serveraddr.sin_family = AF_INET;
		serveraddr.sin_addr.s_addr = INADDR_ANY;
		serveraddr.sin_port = htons(port);
		if (bind
		    (sockfd, (struct sockaddr *) &serveraddr,
		     sizeof(serveraddr)) == -1) {
			perror("AMS:call to bind");
			exit(1);
		}
	}
	if (listen(sockfd, 20) == -1) {
		perror("AMS:call to listen");
		exit(1);
	}
	amsserver_init(f_init);

	AMS_PRINT(flag, "xos-amsd at 'localhost:%d' starting ...\tOK\n",
		  port);

	mutex = sem_init(SEMKEYMOUNT,1);
	atexit(sem_exit);

	while (1) {
	  /*	  printf("Do accept\n");*/
		temp_sockfd = server_is_local ? accept(sockfd, NULL, 0)
		    : accept(sockfd, (struct sockaddr *) &clientaddr,
			     &clientaddr_size);
		if (temp_sockfd < 0) {
			AMS_PRINT(flag, "accept error:%s\n",
				  strerror(errno));
			continue;
		}



		if ((pid = fork()) == 0) {	/* child process the request */
			int remote_uid = -1;	// only valid if socket is of type AF_UNIX
			/*close(sockfd);*/
			clear_expired_id_timer(flag, expired_time_span);

			if (server_is_local) {
				struct ucred cred;
				unsigned int credl = sizeof(struct ucred);
				bzero(&cred, credl);

				getsockopt(temp_sockfd, SOL_SOCKET,
					   SO_PEERCRED, &cred, &credl);
				if (credl != sizeof(struct ucred)
				    || cred.uid < 0) {
					close(temp_sockfd);
					perror
					    ("AMS: get UNIX credentials from client peer socket");
					/*exit(1);*/goto fail;
				}
				remote_uid = cred.uid;
			}
			if (read(temp_sockfd, inbuf, 16383) == -1) {
				perror("AMS:call to read");
				/*exit(1);*/ goto fail;
			}

			int msg_end_len = strlen(MSG_END);
			for (; strncmp(inbuf, MSG_END, msg_end_len);) {
				AMS_PRINT(flag,
					  "get new command --> '%s'\n",
					  inbuf);

#if 1

				int t_uid, t_gid;
				optcode = -1;
				mapped_dn = mapped_vo = mapped_role =
				    mapped_subgroups = NULL;
				char *p = NULL, *pp = NULL;
				static char *istr = NULL;
				int tno = 0;
				if (istr)
				  free(istr);
				istr = (char *) malloc(strlen(inbuf) + 1);
				strcpy(istr, inbuf);
				pp = p = inbuf;
				while (*p != '\0') {
					if (*p == '#') {
						*p = '\0';
						if (tno == 0)
							optcode = atoi(pp);
						if (tno == 1 && p != pp + 1)
							mapped_dn = pp + 1;
						if (tno == 2 && p != pp + 1)
							mapped_vo = pp + 1;
						if (tno == 3 && p != pp + 1)
							mapped_role = pp + 1;
						if (tno == 4 && p != pp + 1)
							mapped_subgroups = pp + 1;
						AMS_PRINT(flag,  "current -> [%s]\n", pp + 1);
						pp = p;
						tno++;
					}
					p++;
				}

#else				/* new protocol */
				int t_uid, t_gid;
				optcode = 0;
				mapped_dn = mapped_vo = mapped_role =
				    mapped_subgroups = NULL;

				AMS_PRINT(flag, "coming here 3 !!!\n");

				AMS_PRINT(flag, "buffer[%d,%d]:%s\n",
					  *((int *) inbuf),
					  *((int *) (inbuf + 4)),
					  (char *) (inbuf + 12));

				if (amsserver_unpacketmsg(inbuf, &optcode,
							  &mapped_dn,
							  &mapped_vo,
							  &mapped_role,
							  &mapped_subgroups))
				{
					AMS_PRINT(flag,
						  "Can not parse the package\n");
				}
				AMS_PRINT(flag, "coming here 4 !!!\n");
				AMS_PRINT(flag,
					  "received: [%d,%s,%s,%s,%s]\n",
					  optcode, mapped_dn, mapped_vo,
					  mapped_role, mapped_subgroups);
#endif				/*  */


				switch (optcode) {
				case PTL_OT_USERMAPPING:	/* 1 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}

						if (gpwd) {
							free(gpwd);
							gpwd = NULL;
						}
						if (ggrp) {
							free(ggrp);
							ggrp = NULL;
						}
						if (mapped_dn)
							gpwd = gpwd_new();
						if (mapped_role || mapped_subgroups)
							ggrp = ggrp_new();
						if (xos_mapping(mapped_dn, mapped_vo, mapped_role, mapped_subgroups, gpwd, ggrp)) {
							AMS_PRINT(flag, "Can not mapping user\n");
							amsserver_packetmsg(outbuf, NULL, NULL, -1);
						} else {
							AMS_PRINT(flag, "mapping success\n");
							amsserver_packetmsg(outbuf, gpwd, ggrp, 0);
							AMS_PRINT(flag, "grp -> [%s,%s,%s,%d,%s,%ld]\n", ggrp->g_grptoken.g_vo,
								  ggrp->g_grptoken.g_subgroup,ggrp->l_grptoken.g_mappedname,
								  ggrp->l_grptoken.g_mappedgid, ggrp->l_grptoken.g_members,
								  ggrp->g_time);
						}
					}
					break;
				case PTL_OT_CLEARACCT:	/* 2 */
					{
						AMS_PRINT(flag, "remove account...\n");
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (gpwd)
							free(gpwd);
						gpwd = gpwd_new();
						strncpy(gpwd->g_idtoken.g_dn, mapped_dn,MAXBUF);
						strncpy(gpwd->g_idtoken.g_vo, mapped_vo,MAXBUF);
						/*YVON: use case 1*/
#if 1						
						strncpy(gpwd->g_idtoken.g_role,	mapped_role,MAXBUF);
#else /*in anqin's meaning role equls vo? see pam_xos.c: 389*/
						strncpy(gpwd->g_idtoken.g_role,	mapped_vo,MAXBUF);
#endif 
						if (clear_account(gpwd)) {
							AMS_PRINT(flag, "Can not remove account\n");
							ret = -1;
						} else {
							AMS_PRINT(flag, "removed account\n");
							ret = 0;
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_CLEARGRP:	/* 3 */
					{
						AMS_PRINT(flag, "remove group...");
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (ggrp)
							free(ggrp);
						ggrp = ggrp_new();
						strncpy(ggrp->g_grptoken.g_vo, mapped_vo,MAXBUF);
						strncpy(ggrp->g_grptoken.g_subgroup,mapped_subgroups,MAXBUF);
						if (clear_group(ggrp)) {
							AMS_PRINT(flag,"Can not remove group\n");
							ret = -1;
						}

						else {
							AMS_PRINT(flag, "removed group\n");
							ret = 0;
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_CLEARACTGRP:	/* 4 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (clear_all()) {
							AMS_PRINT(flag, "Can not remove all data\n");
							ret = -1;
						}

						else {
							AMS_PRINT(flag, "removed all\n");
							ret = 0;
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_GETBYACTGRP:	/* 5 */
					{
						GIDTOKEN gtk_pwd;
						if (gpwd)
							free(gpwd);
						gpwd = gpwd_new();
						strncpy(gtk_pwd.g_dn,mapped_dn, MAXBUF);
						strncpy(gtk_pwd.g_vo,mapped_vo, MAXBUF);
                                                /*YVON: choose case 1*/
#if 1						
						strncpy(gtk_pwd.g_role,mapped_role,MAXBUF);
#else /*in anqin's meaning role equls vo? see pam_xos.c: 389*/
						strncpy(gtk_pwd.g_role,	mapped_vo,MAXBUF);

#endif 


						/* get PASSWD */
						if (!bdb_getdbgtk_pwd(&gtk_pwd, gpwd)) {
							AMS_PRINT(flag, "Can not get information of PASSWD\n");
							ret = -1;
							//free_gpwd_ggrp(gpwd,NULL);
						}

						else {
							AMS_PRINT(flag,  "Success in getting PASSWD\n");
							/* adopt the timestamp of utility.
							 * should be updated after sqlite is avaliable*/
							gpwd->g_time = time(NULL);
							if (bdb_deldbgtk_pwd(&gtk_pwd) || bdb_storedben_pwd(gpwd))
								AMS_PRINT(flag,"fail to update timestamp in PASSWD\n");

							ret = 0;
						}

						/*get GROUP */
						GGRPTOKEN gtk_grp;
						if (ggrp)
							free(ggrp);
						ggrp = ggrp_new();
						strncpy(gtk_grp.g_vo,mapped_vo, MAXBUF);
						strncpy(gtk_grp.g_subgroup,mapped_role,MAXBUF);

						/* If fail, not return anything */
						if (!bdb_getdbgtk_grp(&gtk_grp, ggrp)) {
							AMS_PRINT(flag, "Can not get information of GGROUP\n");
							//free_gpwd_ggrp(NULL,ggrp);
						}

						else {
							AMS_PRINT(flag, "Success in getting GGROUP\n");
							/* adopt the timestamp of utility.
							 * should be updated after sqlite is avaliable*/
							ggrp->g_time = time(NULL);
							if (bdb_deldbgtk_grp(&gtk_grp) || bdb_storedben_grp(ggrp))
								AMS_PRINT(flag, "fail to update timestamp in GGROUPS\n");

						}
						if(ret) {free_gpwd_ggrp(gpwd,ggrp);gpwd = NULL; ggrp = NULL;}
						amsserver_packetmsg(outbuf, gpwd, ggrp, ret);
					}
					break;
				case PTL_OT_GETGRP:	/* 6 */
					{
						GGRPTOKEN gtk_grp;
						if (ggrp)
							free(ggrp);
						ggrp = ggrp_new();
						strncpy(gtk_grp.g_vo,mapped_vo, MAXBUF);
						strncpy(gtk_grp.g_subgroup,mapped_role,MAXBUF);

						/* If fail, not return anything */
						if (!bdb_getdbgtk_grp(&gtk_grp, ggrp)) {
							AMS_PRINT(flag, "Can not get information of GGROUP\n");
							ret = -1;
						}

						else {
							AMS_PRINT(flag, "Success in getting GGROUP\n");
							/* adopt the timestamp of utility.
							 * should be updated after sqlite is avaliable*/
							ggrp->g_time = time(NULL);
							if (bdb_deldbgtk_grp(&gtk_grp) || bdb_storedben_grp(ggrp))
								AMS_PRINT(flag, "fail to update timestamp in GGROUPS\n");

							ret = 0;
						}
						if(ret) {free_gpwd_ggrp(NULL,ggrp);ggrp = NULL;}
						amsserver_packetmsg(outbuf, NULL, ggrp, ret);
					}
					break;
				case PTL_OT_CLEARUIDGID:	/* 7 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag,  "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}

						ret = -1;
						if (mapped_dn) {
							t_uid = atoi(mapped_dn);
							AMS_PRINT(flag,"clear [%d]...\n", t_uid);
							if (gpwd)
								free(gpwd);
							gpwd = gpwd_new();
							if (!bdb_getdbuid_pwd(t_uid, gpwd)) {
								AMS_PRINT(flag, "Can not get PASSWD by uid\n");
							} else {
								if (clear_account(gpwd))
									AMS_PRINT(flag, "Can not remove account\n");

								else {
									ret = 0;
									/* If the group has no member, clear it */
									if (bdb_operatedb_grp(NULL, NULL, (OPTGRPFUNC) clear_mapped_group))
										AMS_PRINT(flag, "Can not remove groups");
								}
							}
						}
						if (mapped_vo) {
							t_gid = atoi(mapped_vo);
							AMS_PRINT(flag,"clear [%d]...\n", t_gid);
							if (ggrp) free(ggrp);
							ggrp = ggrp_new();
							if (!bdb_getdbgid_grp(t_gid, ggrp)) {
								AMS_PRINT(flag, "Can not get GROUPS by gid\n");
							} else {
								if (clear_group(ggrp))
									AMS_PRINT(flag, "Can not remove group\n");

								else
									ret = 0;
							}
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_GETBYUIDGID:	/* 8 */
					{
						ret = -1;
						if (mapped_dn) {	/* DN or local username */
							if (gpwd) free(gpwd);
							gpwd = gpwd_new();
							if (bdb_getdbname_pwd(mapped_dn, gpwd))
								ret = 0;

							else if (bdb_getdbuname_pwd(mapped_dn, gpwd))	/*local username */
								ret = 0;

							else
								AMS_PRINT(flag, "Can not get GPASSWD by DN or local username\n");
						}
						if (mapped_vo && ret == -1) {	/* uid */
							t_uid =  atoi(mapped_vo);
							AMS_PRINT(flag,"get GPASSWD of [%d]...\n", t_uid);
							if (gpwd)free(gpwd);
							gpwd = gpwd_new();
							if (!bdb_getdbuid_pwd(t_uid, gpwd)) {
								AMS_PRINT(flag, "Can not get PASSWD by uid\n");
							} else {
								ret = 0;
							}
						}
						if (ret == -1 && gpwd) {
							free(gpwd);
							gpwd = NULL;
						}
						int t_ret = -1;
						if (mapped_role) {	/* DN or local groupname */
							if (ggrp) free(ggrp);
							ggrp = ggrp_new();
							if (bdb_getdbname_grp(mapped_role, ggrp))
								t_ret = 0;

							else if (bdb_getdbgname_grp(mapped_role, ggrp))	/*local username */
								t_ret = 0;

							else
								AMS_PRINT(flag, "Can not get GGROUPS by DN or local groupname\n");
						}
						if (mapped_subgroups && t_ret == -1) {	/* uid */
							t_gid =  atoi(mapped_subgroups);
							AMS_PRINT(flag, "get GGROUPS of [%d]...\n", t_gid);
							if (ggrp) free(ggrp);
							ggrp = ggrp_new();
							if (!bdb_getdbgid_grp(t_gid, ggrp)) {
								AMS_PRINT(flag, "Can not get GROUPS by gid\n");
							} else {
								t_ret = 0;
							}
						}
						if (t_ret == -1 && ggrp) {
							free(ggrp);
							ggrp = NULL;
						}
						ret = ret & t_ret;
						amsserver_packetmsg(outbuf, gpwd, ggrp, ret);
					}
					break;

				case PTL_OT_OPTMOUNTCNT:	/* 22 */
					{
						ret = -1;
						if (mapped_dn) {	/*opt_cmd for mount counter */
						
							AMS_PRINT(flag,"get '%s' for operating mount counter\n",mapped_dn);

							if (mapped_vo) {	/* uid */
								t_uid = atoi(mapped_vo);
								AMS_PRINT(flag, "get uid '%d for operating mount count\n", t_uid);
								if (gpwd) free(gpwd);
								gpwd = gpwd_new();
								if (!bdb_getdbuid_pwd(t_uid, gpwd)) {
									AMS_PRINT(flag, "Can not get PASSWD by uid for operating mount counter\n");
								} else {
									ret = 0;
								}
			
							}else 
								AMS_PRINT(flag,"Can not get uid for operating mount counter\n");

							if(ret == 0) {
								/* set */
								if(!strncmp(mapped_dn,"set",strlen("set")))  {
									t_gid = atoi(mapped_role);
									AMS_PRINT(flag, "get param '%d for operating mount count\n", t_gid);
									gpwd->g_mount = t_gid;
									if(bdb_deldbgtk_pwd(&(gpwd->g_idtoken)) || bdb_storedben_pwd(gpwd))
										AMS_PRINT(flag,"Can not update the mount counter by 'set'\n");	
				
								}else if(!strncmp(mapped_dn,"inc",strlen("inc")))  {
									gpwd->g_mount += 1;
									if(bdb_deldbgtk_pwd(&(gpwd->g_idtoken)) || bdb_storedben_pwd(gpwd))
										AMS_PRINT(flag,"Can not update the mount counter by 'inc'\n");	
			
								}else if(!strncmp(mapped_dn,"dec",strlen("dec")))  {
									gpwd->g_mount -= 1;
									if(bdb_deldbgtk_pwd(&(gpwd->g_idtoken)) || bdb_storedben_pwd(gpwd))
										AMS_PRINT(flag,"Can not update the mount counter by 'dec'\n");	
			
								}else if(!strncmp(mapped_dn,"get",strlen("get")))  {
									ret = gpwd->g_mount;
								}
								ret = gpwd->g_mount;
							}
						
						}
						
						free_gpwd_ggrp(gpwd,ggrp);
						gpwd = NULL; ggrp = NULL;
						amsserver_packetmsg(outbuf, NULL, NULL,  ret);
					}
					break;
				case PTL_OT_SETUPQUOTA:	/* 9 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						ret = -1;
						ResQuota rquota;
						xos_resquota_init(&rquota);
						if (mapped_vo) {	/* role */
							if (xos_resquota_interpret(mapped_vo, &rquota))
								AMS_PRINT(flag, "Can not interpret policy strategy\n");

							else
								ret = 0;
						}
						if (mapped_dn && !ret) {	/* username */
							rquota.rname = strdup(mapped_dn);
							if (xos_resquota_modify(&rquota, 7)) {
								AMS_PRINT(flag, "Can not setup mapped policy\n");
								ret = -1;
							}
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_CLEARQUOTA:	/* 10 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/ goto fail;
						}
						ret = -1;
						ResQuota rquota;
						xos_resquota_init(&rquota);
						if (mapped_vo) {	/* role */
							if (xos_resquota_interpret(mapped_vo, &rquota))
								AMS_PRINT(flag, "Can not interpret policy strategy\n");

							else
								ret = 0;
						}
						if (mapped_dn && !ret) {	/* username */
							rquota.rname = strdup(mapped_dn);
							if (xos_resquota_clear(&rquota)) {
								AMS_PRINT(flag, "Can not setup mapped policy\n");
								ret = -1;
							}
						}
						amsserver_packetmsg(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_SAVETMPINFO:	/* 11 */
					{
						ret = -1;
						char rkey[KEY_LEN];
						amsserver_create_sranddbkey(rkey, KEY_LEN - 1,getpid());
						AMS_PRINT(flag, "srand key --> [%s]\n", rkey);
						if (mapped_dn) {
							if (tmpinfo_db_put(rkey, mapped_dn))
								AMS_PRINT(flag, "Cann't save information to tmp database\n");
							else
								ret = 0;
						}
						if (ret == 0)
							amsserver_packetmsg_tmpinfo(outbuf, rkey, 0);
						else
							amsserver_packetmsg_tmpinfo(outbuf, NULL, -1);
					}
					break;
				case PTL_OT_GETTMPINFO:	/* 12 */
					{
						ret = -1;
						int data_len = 0;
						char *data = NULL;
						if (mapped_dn) {
							if (strlen(mapped_dn) !=  KEY_LEN - 1)
								AMS_PRINT(flag, "key is wrong\n");
							else if(tmpinfo_db_get(mapped_dn,&data, &data_len))
								AMS_PRINT(flag, "Cann't get information from tmp database\n");
							else if(tmpinfo_db_delete (mapped_dn))
								AMS_PRINT(flag, "Cann't remove information from tmp database\n");
							else
								ret = 0;
						}
						if (ret == 0 && data_len < 8192 - 2)	/* 8192 is the max-length of message */
							amsserver_packetmsg_tmpinfo(outbuf, data, 0);
						else
							amsserver_packetmsg_tmpinfo(outbuf, NULL,  -1);
					}
					break;
				case PTL_OT_RULEPUT:	/* 13 */
					{
						struct rule_t xrule;
						char *handler = NULL;

						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						ret = -1;
						if (mapped_dn) {
							if (protocol_unpacketrule(mapped_dn, &xrule))
								AMS_PRINT(flag, "Can not parse the package to rule data\n");
							else if(ruleinfo_db_put(&xrule,&handler))
								AMS_PRINT(flag, "Can not store rule to databases\n");
							else
								ret = 0;
						}
						amsserver_packetmsg_rule(outbuf, NULL, handler, ret);
					}
					break;
				case PTL_OT_GETRULEBYKEY:	/* 14 */
					{
						ret = -1;
						int rule_ikey;
						struct rule_t *xrule =  NULL;
						char *handler = NULL;

						if (server_is_local && remote_uid) {
							AMS_PRINT(flag,"Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (mapped_dn) {
							rule_ikey =  atoi(mapped_dn);
							if (ruleinfo_db_getbykey(rule_ikey, &xrule, &handler))
								AMS_PRINT(flag, "Can not get rule by key\n");
							else
								ret = 0;
						}
						amsserver_packetmsg_rule(outbuf, xrule, handler, ret);
					}
					break;
				case PTL_OT_GETBYHANDLER:	/* 15 */
					{
						ret = -1;
						struct rule_t *xrule =  NULL;
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (mapped_dn) {
							if (ruleinfo_db_getbyhdler(mapped_dn, &xrule))
								AMS_PRINT(flag, "Can not get rule by handler\n");
							else
								ret = 0;
						}
						amsserver_packetmsg_rule(outbuf, xrule, NULL, ret);
					}
					break;
				case PTL_OT_DELETERULE:	/* 16 */
					{
						ret = -1;
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							exit(-1);
						}
						if (mapped_dn) {
							if (ruleinfo_db_delete(mapped_dn))
								AMS_PRINT(flag, "Can not delete rule by handler\n");
							else
								ret = 0;
						}
						amsserver_packetmsg_rule(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_CLEARRULES:	/* 17 */
					{
						ret = -1;
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
//                                      if(ruleinfo_db_clearall())
//                                      AMS_PRINT(flag,"Can not clear rules\n");
//                                      else ret = 0;
						ruleinfo_db_clearall();
						ret = 0;
						amsserver_packetmsg_rule(outbuf, NULL, NULL, ret);
					}
					break;
				case PTL_OT_SEARCHRULE:	/* 18 */
					{
						ret = -1;
						struct rule_t xrule;
						char *rule_key = NULL;
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag,  "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (mapped_dn) {
							if (protocol_unpacketrule(mapped_dn, &xrule))
								AMS_PRINT(flag, "Can not unpacket rule data\n");
							else if
							    (ruleinfo_db_search(&xrule, &rule_key))
								AMS_PRINT(flag,"Can not search rule\n");
							else
								ret = 0;
						}
						ruleinfo_db_printdata(&xrule);
						amsserver_packetmsg_rule(outbuf, NULL, rule_key, ret);
					}
					break;
				case PTL_OT_ACREJECT:
					{
						ret = -1;
						struct subject_t sub;
						int in_blacklist = 0;
						char ret_buf[32];
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (mapped_dn) {
							if (protocol_unpacketsubject(mapped_dn, &sub))
								AMS_PRINT(flag, "Can not unpacket subject data\n");
							else if(xos_policy_ACreject(&sub, &in_blacklist))
								AMS_PRINT(flag, "Can not judge whether is in blacklist\n");
							else
								ret = 0;
						}
						sprintf(ret_buf, "%d",in_blacklist);
						amsserver_packetmsg_rule(outbuf, NULL, ret_buf, ret);
					}
					break;
				case PTL_OT_CHUSERINFO:
					{
						ret = -1;
						if (mapped_dn && mapped_vo) {	/* uid */
							t_uid =  atoi(mapped_dn);
							AMS_PRINT(flag,  "get GPASSWD of [%d]...\n", t_uid);
							if (gpwd) free(gpwd);
							gpwd = gpwd_new();
							if (!bdb_getdbuid_pwd(t_uid, gpwd)) {
								AMS_PRINT(flag, "Can not get PASSWD by uid\n");
							} else
							    if (strncmp(gpwd-> g_home, mapped_vo, strlen(mapped_vo))){
								if (bdb_deldbgtk_pwd(&(gpwd->g_idtoken)))
									AMS_PRINT(flag, "Can not remove the old GPWD in changing user info\n");
								else {
									strncpy(gpwd->g_home, mapped_vo, MAXFIELD);
									if (bdb_storedben_pwd(gpwd)) {
										AMS_PRINT(flag, "Can not replace the old GPWD\n");
									} else
										ret = 0;
								}
							} else
								ret = 0;

						
							//if(!ret)
							//ams_cache_whole_erase(NULL,NULL,NULL,t_uid,0);
						}

						amsserver_packetmsg_rule(outbuf, NULL, NULL, ret);

					}
					break;
				case PTL_OT_UMOUNTHOME:
					{
						ret = -1;
						char * mnt_command = malloc(2000);
						if (mapped_dn && mapped_vo) {	/* uid */
						  sprintf(mnt_command, "xtreemfs_mnt umount %s",
							  mapped_vo);
						  printf("Command: %s", mnt_command);
						  system(mnt_command);
						  ret = 0;
						}
						free(mnt_command);

						amsserver_packetmsg_rule(outbuf, NULL, NULL, ret);

					}
					break;
				case PTL_OT_ENFORCERULE:
					{
						ret = -1;
						char enforced_rules[8192];
						char *packet_str = NULL;

						if (server_is_local  && remote_uid) {
							AMS_PRINT(flag,  "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						if (mapped_dn) {
							if (gpwd) {
								free(gpwd);
								gpwd = NULL;
							}
							gpwd = gpwd_new();
							memset(enforced_rules, 0, sizeof(enforced_rules));

							if (!bdb_getdbuname_pwd(mapped_dn, gpwd))
								AMS_PRINT(flag, "Can not get GPASSWD of user = '%s'\n", mapped_dn);
							else if(xos_policy_pwdbinding(gpwd, enforced_rules))
								AMS_PRINT(flag, "Can not bind rules to user = '%s'\n", mapped_dn);
							else
								ret = 0;
						}
						if (enforced_rules[0] !=   '\0')
							packet_str =   enforced_rules;
						amsserver_packetmsg_tmpinfo(outbuf, packet_str,  ret);
					}
					break;
				case PTL_OT_CGRPADDPID:	/* 23 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag,"Access denied: remote user is not root\n");
							/*	exit(-1);*/goto fail;
						}
						AMS_PRINT(flag,"subsys = '%s', name = '%s', pid = '%s'\n", 
								mapped_dn, mapped_vo,mapped_role);
	
						if(!mapped_vo || !mapped_role) ret = 1;
						else {
							cnt_pid = atoi(mapped_role);
							ret = hd_cgroup_addpid(mapped_dn,mapped_vo,cnt_pid);
						}
						amsserver_packetmsg(outbuf,NULL,NULL,ret);

					}
					break;
				case PTL_OT_CGRPDELPID:	/* 24 */
					{
						AMS_PRINT(flag, "remove pid from cgroup...\n");
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag,"Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						AMS_PRINT(flag,"name = '%s', pid = '%s'\n", mapped_dn, mapped_vo);
						amsserver_packetmsg(outbuf,NULL,NULL,0);
					}
					break;
				case PTL_OT_CGRPCOUNTPID:	/* 25 */
					{
						AMS_PRINT(flag,  "get total number of cgroup...");
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						AMS_PRINT(flag,"name = '%s'\n", mapped_dn);
						amsserver_packetmsg(outbuf,NULL,NULL,0);

					}
					break;
				case PTL_OT_MOUNTFS:	/* 26 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
							/*exit(-1);*/goto fail;
						}
						AMS_PRINT(flag,"name = '%s'\n", mapped_dn);
						ret = -1;
						if(mapped_dn) {
							ret = hd_cgroupfs_mount(mapped_dn);
						}
						amsserver_packetmsg(outbuf,NULL,NULL,ret);
					}
					break;
				case PTL_OT_UMOUNTFS:	/* 27 */
					{
						AMS_PRINT(flag,"name = '%s'\n", mapped_dn);
						ret = 0;
						if(mapped_dn) {
							ret = hd_cgroupfs_umount(mapped_dn);
						}
						AMS_PRINT(flag,"unmount = '%d'\n", ret);
						amsserver_packetmsg(outbuf,NULL,NULL,ret);
	
					}
					break;
				case PTL_OT_CNTCRT:	/* 28 */
					{
						AMS_PRINT(flag,"name = '%s', subsys = '%s'\n", mapped_dn, mapped_vo);
						ret = -1;
						if(mapped_dn){
						  if (!mapped_role)
							ret = hd_create_cgroup(mapped_dn, mapped_vo);
						  /*						  else 
												  ret = hd_create_cgroup_withquota(mapped_dn, mapped_vo, mapped_role);*/
						}
						amsserver_packetmsg(outbuf,NULL,NULL,ret);
					}
					break;
				case PTL_OT_CNTDESTY:	/* 29 */
					{
						if (server_is_local && remote_uid) {
							AMS_PRINT(flag, "Access denied: remote user is not root\n");
						/*exit(-1);*/goto fail;
						}
						AMS_PRINT(flag,"name = '%s', subsys = '%s'\n", mapped_dn, mapped_vo);
						if(mapped_dn)
							ret = hd_destroy_cgroup(mapped_dn, mapped_vo);
						amsserver_packetmsg(outbuf,NULL,NULL,ret);
					}
					break;
	
				default:

					sprintf(outbuf,
						"token -> [%s,%s,%s,%s][%d]",
						mapped_dn, mapped_vo,
						mapped_role,
						mapped_subgroups, optcode);
				}
				AMS_PRINT(flag, "message -> [%s]\n",
					  outbuf);
				if (write
				    (temp_sockfd, outbuf,
				     strlen(outbuf)) == -1) {
					perror("call to write");
					exit(1);
				}

				AMS_PRINT(flag, "retrieve message ... \n");
				if (read(temp_sockfd, inbuf, 16383) == -1) {
					perror("AMS:call to read");
					/*exit(-1);*/goto fail;
					
				}

			}
		fail:			
			/*close(temp_sockfd);*/
			exit(0);
		}
		
		else if (pid > 0) {	
		  /*  i++;
		      printf("line 1379 got called\n");*/
		  waitpid(-1, &status, 0/* WNOHANG*/);
		}
		close(temp_sockfd);
		
	}
	return 0;
}
