/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file  xos_cgrp.c
 * @brief The implementatio of AMS clien for cgroup interfaces
 *
 * \todo The codes of client is building.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

//#include "xos.h"
#include "xos_cgrp.h"
#include "xos_utils.h"
#include "xos_protocol.h"


#define MAXBUF		128



extern int hd_cgroup_insertpid_internal(int sockfd, char *subsys, char *cnt, int pid)
{
        char message[PACKAGE_MAXLEN];
        char buf_pid[MAXBUF];
        int message_len = -1, ret = -1;
        int t_pid;

        if(sockfd <= 0 || !cnt || pid <= 0) return -1;

        memset(message, 0, PACKAGE_MAXLEN);
        memset(buf_pid, 0, MAXBUF);

	t_pid = pid;
	sprintf(buf_pid, "%d", t_pid);

	if (amsclient_packetmsg  /* opt == 1 */
	    (message, PTL_OT_CGRPADDPID, subsys, cnt, buf_pid, NULL)) {
		hd_log_error("Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		hd_log_error("Client: Can not send query to hdtmd");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		hd_log_error("Client:Can not unpacket the message");
		return -1;
	}

	return ret;
	
}

extern int hd_cgroup_insertpid(char *subsys, char *cnt, int pid)		
{							
	int sockfd = -1, ret = -1;			
							
	if(!cnt || pid <= 0)				
		return -1;				
							
        if((sockfd = amsclient_connect_open()) <= 0) {	
                hd_log_error("Client: Can not setup the hdtmd connection");
                return -1;					
        }							
								
        ret = hd_cgroup_insertpid_internal(sockfd, subsys, cnt, pid);
								
        amsclient_connect_close(sockfd);				
								
        return ret;						
}

extern int hd_cgroup_delpid_internal(int sockfd, char *cnt, int pid)
{
        char message[PACKAGE_MAXLEN];
        char buf_pid[MAXBUF];
        int message_len = -1, ret = -1;
        int t_pid;

        if(sockfd <= 0 || !cnt || pid <= 0) return -1;

        memset(message, 0, PACKAGE_MAXLEN);
        memset(buf_pid, 0, MAXBUF);

	t_pid = pid;
	sprintf(buf_pid, "%d", t_pid);

	if (amsclient_packetmsg  /* opt == 1 */
	    (message, PTL_OT_CGRPDELPID, cnt, buf_pid, NULL, NULL)) {
		hd_log_error("Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		hd_log_error("Client: Can not send query to hdtmd");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		hd_log_error("Client:Can not unpacket the message");
		return -1;
	}

	return ret;
	
}

/* pid is unused */
extern int hd_cgroup_countpid_internal(int sockfd, char *cnt, int pid)
{
        char message[PACKAGE_MAXLEN];
        int message_len = -1, ret = -1;

        if(sockfd <= 0 || !cnt) return -1;

        memset(message, 0, PACKAGE_MAXLEN);

	if (amsclient_packetmsg  /* opt == 1 */
	    (message, PTL_OT_CGRPCOUNTPID, cnt, NULL, NULL, NULL)) {
		hd_log_error("Client:Can not built message package");
		return -1;
	}

	/* send query to AMS */
	if(amsclient_connect_query(sockfd,message,&message_len)) {
		hd_log_error("Client: Can not send query to hdtmd");
		return -1;
	}
	
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {
		hd_log_error("Client:Can not unpacket the message");
		return -1;
	}

	return ret;
	
}



#define CGROUP_PID_FUNC(__opt)				\
extern int hd_cgroup_##__opt##pid(char *cnt, int pid)		\
{							\
	int sockfd = -1, ret = -1;			\
							\
	if(!cnt || pid <= 0)				\
		return -1;				\
							\
        if((sockfd = amsclient_connect_open()) <= 0) {	\
                hd_log_error("Client: Can not setup the hdtmd connection");	\
                return -1;					\
        }							\
								\
        ret = hd_cgroup_##__opt##pid_internal(sockfd, cnt, pid);	\
								\
        amsclient_connect_close(sockfd);				\
								\
        return ret;						\
}	
CGROUP_PID_FUNC(del);
CGROUP_PID_FUNC(count);
#undef CGROUP_PID_FUNC

#define CGROUP_FS_FUNC(__opt,__tag)  \
extern int hd_cgroup_##__opt##_internal(int sockfd, char *subsys)		\
{\
        char message[PACKAGE_MAXLEN];					\
        int message_len = -1, ret = -1;					\
									\
        if(sockfd <= 0 || !subsys) return -1;				\
									\
        memset(message, 0, PACKAGE_MAXLEN);				\
									\
	if (amsclient_packetmsg  /* opt == 4 (MOUNTFS) or 5 (UMOUNTFS) */ \
	    (message, PTL_OT_##__tag, subsys, NULL, NULL, NULL)) {		\
		hd_log_error("Client:Can not built message package");	\
		return -1;						\
	}								\
									\
	/* send query to AMS */						\
	if(amsclient_connect_query(sockfd,message,&message_len)) {	\
		hd_log_error("Client: Can not send query to hdtmd");	\
		return -1;						\
	}								\
									\
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {		\
		hd_log_error("Client:Can not unpacket the message");	\
		return -1;						\
	}								\
									\
	return ret;							\
									\
}									\
	\
extern int hd_cgroup_##__opt(char *subsys)					\
{	\
	int sockfd = -1, ret = -1;					\
									\
	if(!subsys)							\
		return -1;						\
									\
        if((sockfd = amsclient_connect_open()) <= 0) {			\
                hd_log_error("Client: Can not setup the hdtmd connection");\
                return -1;						\
        }								\
									\
        ret = hd_cgroup_##__opt##_internal(sockfd, subsys);		\
									\
        amsclient_connect_close(sockfd);					\
									\
        return ret;							\
}
CGROUP_FS_FUNC(mountfs, MOUNTFS);
CGROUP_FS_FUNC(umountfs, UMOUNTFS);
#undef CGROUP_FS_FUNC


#define CGROUP_CONTAINER_FUNC(__opt,__tag)  \
extern int hd_cgroup_##__opt##_internal(int sockfd, char *name, char *subsys)	\
{\
        char message[PACKAGE_MAXLEN];					\
        int message_len = -1, ret = -1;					\
									\
        if(sockfd <= 0 || !subsys) return -1;				\
									\
        memset(message, 0, PACKAGE_MAXLEN);				\
									\
	/* opt == 6 (create container) or 7 (destroy container) */	\
	if (amsclient_packetmsg  						\
	    (message, PTL_OT_##__tag, name, subsys, NULL, NULL)) {	\
		hd_log_error("Client:Can not built message package");	\
		return -1;						\
	}								\
									\
	/* send query to AMS */						\
	if(amsclient_connect_query(sockfd,message,&message_len)) {	\
		hd_log_error("Client: Can not send query to AMS");	\
		return -1;						\
	}								\
									\
	if (amsclient_unpacketmsg(message, NULL, NULL, &ret)) {		\
		hd_log_error("Client:Can not unpacket the message");	\
		return -1;						\
	}								\
									\
	return ret;							\
									\
}									\
	\
extern int hd_cgroup_##__opt(char *name, char *subsys)				\
{	\
	int sockfd = -1, ret = -1;					\
									\
	if(!subsys)							\
		return -1;						\
									\
        if((sockfd = amsclient_connect_open()) <= 0) {			\
                hd_log_error("Client: Can not setup the hdtmd connection");\
                return -1;						\
        }								\
									\
        ret = hd_cgroup_##__opt##_internal(sockfd, name, subsys);	\
									\
        amsclient_connect_close(sockfd);					\
									\
        return ret;							\
}
CGROUP_CONTAINER_FUNC(create, CNTCRT);
CGROUP_CONTAINER_FUNC(destroy, CNTDESTY);
#undef CGROUP_CONTAINER_FUNC

