/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "xos_policy_rule.h"
#include "xos_policy_mgmt.h"
#include "xos_policy_db.h"


int to_numcmd(char *istr)
{
	int i, flag;
	flag = 0;
	for (i = 0; i < strlen(istr) && !flag; i++) {
		if (!isdigit(istr[i]))
			flag = 1;
	}
	if (flag)
		return -1;
	else
		return atol(istr);
}

void initiate_databases()
{


	struct rule_t account_rule = {
		XPLY_RT_MA,
		{XPLY_ST_ACT, "*", "xtreemos"},
		{XPLY_OT_ACT, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};

	struct rule_t group_rule = {
		XPLY_RT_MA,
		{XPLY_ST_GRP, "xtreemos", "*"},
		{XPLY_OT_GRP, {.objc_object = "*"}},
		{.rc_driver.driver_name = {'\0'},.rc_driver.driver_params =
		 {'\0'}},
		1,
		1234567
	};


	struct rule_t setting_rule = {
		XPLY_RT_MA,
		{XPLY_ST_GRP, "xtreemos", "*"},
		{XPLY_OT_GRP, {.objc_object = "*"}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-oscar", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_NOFILE}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_nproc = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-shu", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_NPROC}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_cpu = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-haiyan", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_CPU}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t resquota_rule_mem = {
		XPLY_RT_RU,
		{XPLY_ST_ACT, "-xtreemos-yvon", "xtreemos"},
		{XPLY_OT_QUOTA, {.objc_restype = XPLY_REST_MEM}},
		{.rc_perm = 0},
		1,
		1234567
	};

	struct rule_t reject_rule = {
		XPLY_RT_AC,
		{XPLY_ST_ACT, "*", "chemVO"},
		{XPLY_OT_NODE, {.objc_object = "127.0.0.1"}},
		{.rc_perm = 0},
		1,
		1234567
	};


	sprintf(setting_rule.r_subject.sub_content_1, "%d", 60500);
	sprintf(setting_rule.r_subject.sub_content_2, "%d", 60000);
	sprintf(setting_rule.r_content.rc_driver.driver_name, "%d", 60500);
	sprintf(setting_rule.r_content.rc_driver.driver_params, "%d",
		60000);

	resquota_rule.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule.r_content.rc_quota.quo_val = 5;

	resquota_rule_nproc.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_nproc.r_content.rc_quota.quo_val = 4;

	resquota_rule_cpu.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_cpu.r_content.rc_quota.quo_val = 4;

	resquota_rule_mem.r_content.rc_quota.quo_type = XPLY_QT_ABS;
	resquota_rule_mem.r_content.rc_quota.quo_val = 1024 * 6;

	if (ruleinfo_db_put(&account_rule, NULL)
	    || ruleinfo_db_put(&group_rule, NULL)
	    || ruleinfo_db_put(&resquota_rule, NULL)
	    || ruleinfo_db_put(&resquota_rule_nproc, NULL)
	    || ruleinfo_db_put(&resquota_rule_cpu, NULL)
	    || ruleinfo_db_put(&resquota_rule_mem, NULL)
	    || ruleinfo_db_put(&reject_rule, NULL)
	    || ruleinfo_db_put_aux(&setting_rule, -1, NULL))
		printf("fail to input basical rules in database\n");

}

#if 0
int input_xrule(struct rule_t *xrule)
{
	if (!xrule)
		return -1;
	int need_driver = 0;

	printf("Input the params:\n");
	printf("Rule Type: ");
	scanf("%d", &(xrule->r_type));

	if (xrule->r_type == XPLY_RT_NUL)
		return 0;

	printf("Rule subject:\n");
	printf("\tSubject Type:");
	scanf("%d", &(xrule->r_subject.sub_type));
	printf("\tSubject content_1: ");
	scanf("%s", xrule->r_subject.sub_content_1);
	printf("\tSubject content_2: ");
	scanf("%s", xrule->r_subject.sub_content_2);

	printf("Rule object:\n");
	printf("\tObject Type:");
	scanf("%d", &(xrule->r_object.obj_type));
	if (xrule->r_object.obj_type == XPLY_OT_QUOTA) {
		printf("\tResouce Type: ");
		scanf("%d", &(xrule->r_object.obj_content.objc_restype));
	} else {
		printf("\tObject content: ");
		scanf("%s", &(xrule->r_object.obj_content.objc_object));
	}

	printf("Rule content:\n");
	if (xrule->r_type == XPLY_RT_AC) {
		printf("\tAC permission:");
		scanf("%d", &(xrule->r_content.rc_perm));
	} else if (xrule->r_type == XPLY_RT_RU) {
		printf("\tQuota Type:");
		scanf("%d", &(xrule->r_content.rc_quota.quo_type));
		printf("\tQuota Value:");
		scanf("%f", &(xrule->r_content.rc_quota.quo_val));
	} else {
		printf("Need drivers ? [0/1]");
		scanf("%d", &need_driver);
		if (need_driver) {
			printf("\tDriver name:");
			scanf("%s",
			      xrule->r_content.rc_driver.driver_name);
			printf("\tDriver params:");
			scanf("%s",
			      xrule->r_content.rc_driver.driver_params);
		} else {
			xrule->r_content.rc_driver.driver_name[0] = '\0';
			xrule->r_content.rc_driver.driver_params[0] = '\0';
		}
	}

	printf("Valid? [1/0]");
	scanf("%d", &(xrule->r_valid));
	printf("Time: ");
	scanf("%ld", &(xrule->r_time));

	return 0;
}
#else
char *inputrule_steps[] = {
	"Step [1]  -- set rule type",
	"Step [2]  -- set subject section",
	"Step [3]  -- set object section",
	"Step [4]  -- set rule content",
	"Step [5]  -- set valid or not",
	"Step [6]  -- set time stamp",
	"Exit [0]  -- Exit the guid"
};

char *rule_type_names[] = {
	"XPLY_RT_MA",
	"XPLY_RT_AC",
	"XPLY_RT_RU",
	"XPLY_RT_AP"
};

char *subject_type_names[] = {
	"XPLY_ST_ACT",
	"XPLY_ST_GRP"
};

char *object_type_names[] = {
	"XPLY_OT_ACT",
	"XPLY_OT_GRP",
	"XPLY_OT_FILE",
	"XPLY_OT_NODE",
	"XPLY_OT_QUOTA"
};

char *quota_type_names[] = {
	"XPLY_QT_ABS",
	"XPLY_QT_REL"
};

char *resouce_type_names[] = {
	"XPLY_REST_CPU",
	"XPLY_REST_MEM",
	"XPLY_REST_DISK",
	"XPLY_REST_NETWORK",
	"XPLY_REST_NPROC",
	"XPLY_REST_NOFILE"
};

char *valid_choices[] = {
	"YES",
	"NO"
};

void print_other_steps()
{
	int i;
	for (i = 0; i < 7; i++)
		printf("%s\n", inputrule_steps[i]);
}

void print_hint_hear()
{
	printf
	    ("\n+------------- Guider of inputting policy rule ---------------+\n");
	printf
	    ("|     I am the guider of policy manager. Before you           |\n");
	printf
	    ("|     enjoy the new features of XtreemOS Policy manager.      |\n");
	printf
	    ("|     You should know how to add the XOS rules to system,     |\n");
	printf
	    ("|     I will help you create your customized policy rule      |\n");
	printf
	    ("|     which is the first step to do your jobs.                |\n");
	printf
	    ("|           				                      |\n");
	printf
	    ("|     Six steps to finish are:                                |\n");
	printf
	    ("|         Step [1]  -- set rule type                          |\n");
	printf
	    ("|         Step [2]  -- set subject section                    |\n");
	printf
	    ("|         Step [3]  -- set object section                     |\n");
	printf
	    ("|         Step [4]  -- set rule content                       |\n");
	printf
	    ("|         Step [5]  -- set valid or not                       |\n");
	printf
	    ("|         Step [6]  -- set time stamp                         |\n");
	printf
	    ("|        			                              |\n");
	printf
	    ("|     Any question? Type '?' at any time, thanks !!           |\n");
	printf
	    ("+-------------------------------------------------------------+\n\n");
}

void print_hint_ruletype()
{
	printf
	    ("\n+-- Hints ---------------------------------------------------+\n");
	printf("|   [%d] %s -- Mapping Account                        |\n",
	       1, rule_type_names[0]);
	printf("|   [%d] %s -- Access control                         |\n",
	       2, rule_type_names[1]);
	printf("|   [%d] %s -- Resource Usage                         |\n",
	       3, rule_type_names[2]);
	printf("|   [%d] %s -- Application specific                   |\n",
	       4, rule_type_names[3]);
	printf
	    ("|   [0] Jump to other steps or exit                          |\n");
	printf
	    ("+------------------------------------------------------------+\n");
}

void print_hint_subjecttype()
{
	printf
	    ("\n\t+-- Hints -----------------------------------------------+\n");
	printf("\t|   [%d] %s -- DN,VO,ROLE                        |\n", 1,
	       subject_type_names[0]);
	printf("\t|   [%d] %s -- GROUP,ATTRS                       |\n", 2,
	       subject_type_names[1]);
	printf
	    ("\t|   [0] Jump to other steps or exit                      |\n");
	printf
	    ("\t+--------------------------------------------------------+\n");
}

void print_hint_objecttype()
{
	printf
	    ("\n\t+-- Hints -----------------------------------------------+\n");
	printf
	    ("\t|   [1] XPLY_OT_ACT -- DN,VO,ROLE                        |\n");
	printf
	    ("\t|   [2] XPLY_OT_GRP -- GROUP,ATTRS                       |\n");
	printf
	    ("\t|   [3] XPLY_OT_FILE -- Object is files                  |\n");
	printf
	    ("\t|   [4] XPLY_OT_NODE -- Object is VO node                |\n");
	printf
	    ("\t|   [5] XPLY_OT_QUOTA -- Object is quota item            |\n");
	printf
	    ("\t|   [0] Jump to other steps or exit                      |\n");
	printf
	    ("\t+--------------------------------------------------------+\n");
}

void print_hint_restype()
{
	printf
	    ("\n\t+-- Hints -----------------------------------------------------+\n");
	printf
	    ("\t|   [1] XPLY_REST_CPU         -- CPU to be limited             |\n");
	printf
	    ("\t|   [2] XPLY_REST_MEM         -- Memory to be limited          |\n");
	printf
	    ("\t|   [3] XPLY_REST_DISK        -- Disk to be limited            |\n");
	printf
	    ("\t|   [4] XPLY_REST_NETWORK     -- Network to be limited         |\n");
	printf
	    ("\t|   [5] XPLY_REST_NPROC       -- Maximal number of process     |\n");
	printf
	    ("\t|   [6] XPLY_REST_NOFILE      -- Maximal number of opened file |\n");
	printf
	    ("\t|   [0] Jump to other steps or exit                            |\n");
	printf
	    ("\t+--------------------------------------------------------------+\n");
}

void print_hint_driver()
{
	printf
	    ("\n\t+-- Hints -----------------------------------------------------+\n");
	printf
	    ("\t|   [0] Need not to dirve external application                 |\n");
	printf
	    ("\t|   [1] Drive external application                             |\n");
	printf
	    ("\t+--------------------------------------------------------------+\n");
}

void print_hint_quotatype()
{
	printf
	    ("\n\t+-- Hints -----------------------------------------------+\n");
	printf
	    ("\t|   [1] XPLY_QT_ABS -- Absolute variable                 |\n");
	printf
	    ("\t|   [2] XPLY_QT_REL -- Relative variable                 |\n");
	printf
	    ("\t|   [0] Jump to other steps or exit                      |\n");
	printf
	    ("\t+--------------------------------------------------------+\n");
}


void print_hint_valid()
{
	printf
	    ("\n\t+-- Hints ------------------------------------------------+\n");
	printf
	    ("\t|   [1] YES -- The rule will be enabled after adding     |\n");
	printf
	    ("\t|   [2] NO -- The rule will be add to database           |\n");
	printf
	    ("\t|   [0] Jump to other steps or exit                      |\n");
	printf
	    ("\t+--------------------------------------------------------+\n");
}

/* return valid command value 
 * valid command: integer and '?', '\n'
 * Error - return -1;
 */
int get_command(int need_ret)
{
	char cmd_str[128];
	int valid_cmd = -1;
	int input_cmd = -1, i, j;


	input_cmd = getchar();
	if (input_cmd == '?')
		valid_cmd = input_cmd;
	else if (need_ret && input_cmd == '\n')
		valid_cmd = input_cmd;
	else {
		i = 0;
		j = -1;
		memset(cmd_str, 0, sizeof(cmd_str));
		while (input_cmd == '\n')
			input_cmd = getchar();
		if (input_cmd == '?')
			return input_cmd;
		do {
			cmd_str[i++] = input_cmd;
			if (input_cmd >= '0' && input_cmd <= '9'
			    && j == -1)
				j = i - 1;
			input_cmd = getchar();

		} while (input_cmd != '\n' && i < 128);
		cmd_str[i] = '\0';

		valid_cmd = to_numcmd(cmd_str + j);
	}

	return valid_cmd;

}

/* Friendly interface */
int input_xrule(struct rule_t *xrule)
{
	if (!xrule)
		return -1;
	int cmd = 0, cmd2 = 0;
	char cmd_str[32];


	print_hint_hear();

      step1:
	printf("\n* Step 1 *\n");
	printf(" - This step is to set the type of whole rule.\n\n");


	if (xrule->r_type != XPLY_RT_NUL) {
		printf("Rule Type: ");
		printf("%s", rule_type_names[xrule->r_type - 1]);
		getchar();
		printf("\nChange ? [y/N]:");
		cmd = getchar();
		if (cmd != 'y' && cmd != 'Y')
			goto step2;
	}
	getchar();
	do {
		printf("\nRule Type [0 - 4]: ");

		cmd = get_command(0);
		if (cmd == '?')
			print_hint_ruletype();
	} while (!(cmd >= 0 && cmd < 5));

	if (cmd > 0 && cmd < 5)
		xrule->r_type = cmd;
	else if (cmd == 0) {
		print_other_steps();
		printf("Select [0 - 6]: ");
		scanf("%d", &cmd2);
		if (cmd2 < 0 && cmd2 > 6)
			goto step1;
		else
			switch (cmd2) {
			case 0:
				goto step7;
				break;
			case 1:
				goto step1;
				break;
			case 2:
				goto step2;
				break;
			case 3:
				goto step3;
				break;
			case 4:
				goto step4;
				break;
			case 5:
				goto step5;
				break;
			case 6:
				goto step6;
				break;
			default:
				goto step1;
				break;

			}

	}


	if (xrule->r_type == XPLY_RT_NUL)
		goto step1;
      step2:
	printf("\n* Step 2 *\n");
	printf(" - This step is to set the section of rule subject.\n\n");
	printf("Rule subject:\n");

	if (xrule->r_subject.sub_type != XPLY_ST_NUL) {
		printf("\tSubject Type:");
		printf("%s",
		       subject_type_names[xrule->r_subject.sub_type - 1]);
		getchar();
		printf("\nChange ? [y/N]:");
		cmd = getchar();
		if (cmd != 'y' && cmd != 'Y')
			goto step3;
	}

	do {

		printf("\n\tSubject Type [0 - 2]: ");
		cmd = get_command(0);
		if (cmd == '?')
			print_hint_subjecttype();
	} while (!(cmd >= 0 && cmd <= 2));

	if (cmd > 0 && cmd <= 2)
		xrule->r_subject.sub_type = cmd;
	else if (cmd == 0) {
		print_other_steps();
		printf("Select [0 - 6]: ");
		scanf("%d", &cmd2);
		if (cmd2 < 0 && cmd2 > 6)
			goto step1;
		else
			switch (cmd2) {
			case 0:
				goto step7;
				break;
			case 1:
				goto step1;
				break;
			case 2:
				goto step2;
				break;
			case 3:
				goto step3;
				break;
			case 4:
				goto step4;
				break;
			case 5:
				goto step5;
				break;
			case 6:
				goto step6;
				break;
			default:
				goto step1;
				break;

			}

	}
	printf("\n");
	if (xrule->r_subject.sub_type == XPLY_ST_ACT)
		printf("\tDN expression: ");
	else
		printf("\tVO name expression: ");
	scanf("%s", xrule->r_subject.sub_content_1);
	if (xrule->r_subject.sub_type == XPLY_ST_ACT)
		printf("\tVO name expression: ");
	else
		printf("\tRole/Groups expression: ");
	scanf("%s", xrule->r_subject.sub_content_2);
	getchar();

      step3:

	printf("\n* Step 3 *\n");
	printf(" - This step is to set the section of rule object.\n\n");
	printf("Rule object:\n");

	printf("\tObject Type:");
	if (xrule->r_object.obj_type != XPLY_OT_NUL) {
		printf("%s",
		       object_type_names[xrule->r_object.obj_type - 1]);
		getchar();
		printf("\nChange ? [y/N]:");
		cmd = getchar();
		if (cmd == 'y' || cmd == 'Y') {
			getchar();
			do {

				printf("\n\tObject Type [0 - 5]: ");
				cmd2 = get_command(0);
				if (cmd2 == '?')
					print_hint_objecttype();
			} while (!(cmd2 >= 0 && cmd2 <= 5));
			if (cmd2 != 0)
				xrule->r_object.obj_type = cmd2;
			else
				switch (cmd2) {
				case 0:
					goto step7;
					break;
				case 1:
					goto step1;
					break;
				case 2:
					goto step2;
					break;
				case 3:
					goto step3;
					break;
				case 4:
					goto step4;
					break;
				case 5:
					goto step5;
					break;
				case 6:
					goto step6;
					break;
				default:
					goto step1;
					break;

				}
		} else
			goto step4;
	} else {
		do {

			printf("\n\tSelect [0 - 5]: ");
			cmd = get_command(0);
			if (cmd == '?')
				print_hint_objecttype();
		} while (!(cmd >= 0 && cmd <= 5));
		xrule->r_object.obj_type = cmd;
	}
	if (xrule->r_object.obj_type == XPLY_OT_QUOTA) {
	      step31:
		printf("\tResouce Type: ");
		printf("%s\n",
		       resouce_type_names[xrule->r_object.obj_content.
					  objc_restype]);
		printf("\n\tChange ? [y/N]:");
		cmd = getchar();
		if (cmd == 'y' || cmd == 'Y') {
			getchar();
			do {

				printf("\n\tResouce Type [0 - 6]: ");
				cmd2 = get_command(0);
				if (cmd2 == '?')
					print_hint_restype();
			} while (!(cmd2 >= 0 && cmd2 <= 6));
			xrule->r_object.obj_content.objc_restype = cmd2;
		} else if (cmd == '?') {
			print_hint_restype();
			getchar();
			goto step31;
		}
	} else {
		printf("\tObject content: ");
		scanf("%s", (xrule->r_object.obj_content.objc_object));
		getchar();
	}
      step4:

	printf("\n* Step 4 *\n");
	printf(" - This step is to set the section of rule content.\n\n");

	printf("Rule content:\n");
	if (xrule->r_type == XPLY_RT_AC) {
		printf("\tAC permission:");
		scanf("%d", &(xrule->r_content.rc_perm));
	} else if (xrule->r_type == XPLY_RT_RU) {
		printf("\tQuota Type:");
		printf("%s\n",
		       quota_type_names[xrule->r_content.rc_quota.
					quo_type]);

		printf("\n\tChange ? [y/N]:");
		cmd = getchar();
		if (cmd == 'y' || cmd == 'Y') {
			do {
				printf("\n\tQuota Type[0 - 2]: ");
				cmd2 = get_command(0);
				if (cmd2 == '?')
					print_hint_quotatype();
			} while (!(cmd2 >= 0 && cmd2 <= 2));
			xrule->r_content.rc_quota.quo_type = cmd2;
		} else if (cmd == '?') {
			print_hint_quotatype();
			goto step4;
		} else
			goto step5;

		printf("\tQuota Value:");
		scanf("%f", &(xrule->r_content.rc_quota.quo_val));

	} else {
		do {
			printf("Need drivers ? [0/1]");
			cmd = get_command(0);
			if (cmd == 1) {
				printf("\tDriver name:");
				scanf("%s",
				      xrule->r_content.rc_driver.
				      driver_name);
				printf("\tDriver params:");
				scanf("%s",
				      xrule->r_content.rc_driver.
				      driver_params);
			} else if (cmd == 0) {
				xrule->r_content.rc_driver.driver_name[0] =
				    '\0';
				xrule->r_content.rc_driver.
				    driver_params[0] = '\0';
			} else
				print_hint_driver();
		}
		while (cmd != 1 && cmd != 0);
	}
      step5:
	printf("\n* Step 5 *\n");
	printf
	    (" - This step is to set the variable of rule validity.\n\n");

	printf("Valid?: %s\n", valid_choices[(xrule->r_valid + 1) % 2]);
	getchar();
	printf("\nChange ? [y/N]:");
	cmd = getchar();
	if (cmd == '?')
		print_hint_valid();
	else if (cmd == 'y' || cmd == 'Y') {
		do {

			printf("\nValid?[0 - 2]: ");
			cmd2 = get_command(0);
			if (cmd2 == '?')
				print_hint_valid();
		} while (!(cmd2 >= 0 && cmd2 <= 2));
		if (cmd != 0)
			xrule->r_valid = cmd2 % 2;
		else
			switch (cmd) {
			case 0:
				goto step7;
				break;
			case 1:
				goto step1;
				break;
			case 2:
				goto step2;
				break;
			case 3:
				goto step3;
				break;
			case 4:
				goto step4;
				break;
			case 5:
				goto step5;
				break;
			case 6:
				goto step6;
				break;
			default:
				goto step1;
				break;
			}
	} else if (cmd == '?') {
		print_hint_valid();
		goto step5;
	}

      step6:
	printf("\n* Step 6 *\n");
	printf(" - This step is to set the variable of rule time.\n\n");

	do {
		printf("Time: ");
		scanf("%s", cmd_str);
		cmd = to_numcmd(cmd_str);
		if (cmd == -1)
			printf("Time must be integer\n");
	} while (cmd == -1);
	xrule->r_time = cmd;
      step7:
	return 0;
}
#endif

int main()
{
	char cmd_str[32];
	int cmd = -1, is_root = 0;

	is_root = !getuid();

	struct rule_t xrule;
	memset(&xrule, 0, sizeof(struct rule_t));

	while (cmd) {
		switch (cmd) {
		case 0:
			printf("Quit, thanks!\n");
			break;
		case 1:
			{
				input_xrule(&xrule);
			}
			break;
		case 2:
			ruleinfo_db_printdb();
			break;
		case 3:
			if (!is_root) {
				printf("You need the root privilege\n");
				break;
			}
			ruleinfo_db_clearall();
			break;
		case 4:
			ruleinfo_db_printdata(&xrule);
			break;
		case 5:
			initiate_databases();
			break;
		case 6:
			{
				if (!is_root) {
					printf
					    ("You need the root privilege\n");
					break;
				}
				char *handler = NULL;
				if (xos_policy_addrule(&xrule, &handler))
					printf("fail to add rule\n");
				else
					printf
					    ("Success ! handler is [%s]\n",
					     handler);
			}
			break;
		case 7:
			{
				if (!is_root) {
					printf
					    ("You need the root privilege\n");
					break;
				}
				int uidmax, uidmin, gidmax, gidmin;
				printf("uid_max: ");
				scanf("%d", &uidmax);
				printf("uid_min: ");
				scanf("%d", &uidmin);
				printf("gid_max: ");
				scanf("%d", &gidmax);
				printf("gid_min: ");
				scanf("%d", &gidmin);

				if (xos_policy_vosetting
				    (uidmax, uidmin, gidmax, gidmin))
					printf("fail to change setting\n");
				else {
					printf
					    ("Success in changing setting\n");

				}
			}
			break;
		case 8:
			{
				if (!is_root) {
					printf
					    ("You need the root privilege\n");
					break;
				}

				if (xos_policy_apply(&xrule)) {
					printf("Can not apply the rule\n");
				} else
					printf("Apply !!\n");
			}
			break;
		case 9:
			{

				if (!is_root) {
					printf
					    ("You need the root privilege\n");
					break;
				}

				int rulekey = -1;
				char *handler = NULL;
				struct rule_t *xrule_output = NULL;
				int sub_cmd = 1;

				printf("Input the modified rule: ");
				scanf("%d", &rulekey);

				if (rulekey != -1
				    && ruleinfo_db_getbykey(rulekey,
							    &xrule_output,
							    &handler)) {
					printf
					    ("Warning: The rule to be modified dose not exist\n");
				} else if (xrule_output == NULL) {
					printf("Can not get given rule\n");
					break;
				}
				memcpy(&xrule, xrule_output,
				       sizeof(xrule));

				printf
				    ("You will change following rule:\n");
				ruleinfo_db_printdata(&xrule);
				do {
					printf("Ready: [0/1]");
					getchar();
					sub_cmd = get_command(1);
				} while (sub_cmd != 0 && sub_cmd != 1
					 && sub_cmd != '\n');

				if (sub_cmd == 1 || sub_cmd == '\n')
					input_xrule(&xrule);
				else
					break;

				if (handler
				    && xos_policy_modifyrule(handler,
							     &xrule)) {
					printf
					    ("Can not modfy the given rule\n");
					break;
				}
				printf("Success to modify the rule\n");
			}
			break;
		case 10:
			if (!is_root) {
				printf("You need the root privilege\n");
				break;
			}
			int rulekey = -1;
			char *handler = NULL;

			printf("Input the removed rule No: ");
			scanf("%d", &rulekey);

			if (rulekey != -1
			    && ruleinfo_db_getbykey(rulekey,
						    NULL, &handler)) {
				printf
				    ("The rule to be removed dose not exist\n");
				break;
			}
			if (handler == NULL) {
				printf("The handler of rule is invalid\n");
				break;
			}
			if (xos_policy_removerule(handler)) {
				printf("Can not remove the rule of '%s'\n",
				       handler);
			} else
				printf("Success to remove rule\n");

			break;
		case 11:
			{
				int rule_no, rule_location;

				if (!is_root) {
					printf
					    ("You need the root privilege\n");
					break;
				}

				printf("No. of rule: ");
				scanf("%d", &rule_no);
				printf("Change Rule[%d] to priority: ",
				       rule_no);
				scanf("%d", &rule_location);

				if (xos_policy_setriority
				    (rule_no, rule_location)) {
					printf
					    ("Can not change the priority of given rule.%d\n",
					     rule_no);
				} else
					printf
					    ("Success to change priority of rule\n");

			}
			break;
		default:
			printf("what?\ncmd -> \n");
			printf("0: Quit\n");
			printf("1: Input rule content\n");
			printf("2: print databases\n");
			printf("3: clear rules\n");
			printf("4: print current rule\n");
			printf("5: initiate rules\n");
			printf("6: add and enable new rule \n");
			printf("7: set the vo setting \n");
			printf("8: apply current rule\n");
			printf("9: modify rule\n");
			printf("10: remove given rule from database\n");
			printf("11: change the priority of given rule \n");
		}

		printf("Pls,next cmd --> ");
		scanf("%s", cmd_str);
		cmd = to_numcmd(cmd_str);

	}


	return 0;
}
