/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 *	xos-policy-admin-ac [-dn | -grp]xxx -vo xxx -object xxx -perm xxx
 */

#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"


void usage(char *program)
{
	printf
	    ("Usage:\n\t%s [-dn | -grp] xxx -vo xxx [-file | -node] xxx -perm xxx\n",
	     program);
}

int is_alldigit(char *str)
{
	char *p = str;
	while (*p != '\n' && *p != '\0') {
		if (!isdigit(*p))
			return 0;
		p++;
	}

	return 1;
}

int main(int argc, char *argv[])
{

	char *dn_exp, *vo_exp, *grp_exp;
	char *file_exp, *node_exp, *perm_exp;
	struct rule_t xrule;
	int i;

	dn_exp = vo_exp = grp_exp = file_exp = node_exp = perm_exp = NULL;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-dn")) {
			dn_exp = argv[++i];
		} else if (!strcmp(argv[i], "-vo")) {
			vo_exp = argv[++i];
		} else if (!strcmp(argv[i], "-grp")) {
			grp_exp = argv[++i];
		} else if (!strcmp(argv[i], "-file")) {
			file_exp = argv[++i];
		} else if (!strcmp(argv[i], "-node")) {
			node_exp = argv[++i];
		} else if (!strcmp(argv[i], "-perm")) {
			perm_exp = argv[++i];
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

	if (getuid() != 0) {
		printf("run with root privilege,thanks\n");
		return 0;
	}

	memset(&xrule, 0, sizeof(struct rule_t));

	if (!vo_exp || !perm_exp) {
		printf("Missing parameters: -vo or -perm!\n");
		usage(argv[0]);
		return 1;
	}

	/* check and fill in the rule */
	if (dn_exp) {
		xrule.r_type = XPLY_RT_AC;

		/* Subject */
		xrule.r_subject.sub_type = XPLY_ST_ACT;
		sprintf(xrule.r_subject.sub_content_1, "%s", dn_exp);
		sprintf(xrule.r_subject.sub_content_2, "%s", vo_exp);

		/* Object */
		if (file_exp) {
			xrule.r_object.obj_type = XPLY_OT_FILE;
			sprintf(xrule.r_object.obj_content.objc_object,
				"%s", file_exp);
		} else if (node_exp) {
			xrule.r_object.obj_type = XPLY_OT_NODE;
			sprintf(xrule.r_object.obj_content.objc_object,
				"%s", node_exp);
		} else {
			printf("Missing paramter: -file or -node\n");
			return 1;
		}
		//sprintf(xrule.r_object.obj_content.objc_object, "%s", file_exp);
	} else if (grp_exp) {
		xrule.r_type = XPLY_RT_AC;

		/* Subject */
		xrule.r_subject.sub_type = XPLY_ST_GRP;
		sprintf(xrule.r_subject.sub_content_1, "%s", vo_exp);
		sprintf(xrule.r_subject.sub_content_2, "%s", grp_exp);

		/* Object */
		if (file_exp) {
			xrule.r_object.obj_type = XPLY_OT_FILE;
			sprintf(xrule.r_object.obj_content.objc_object,
				"%s", file_exp);
		} else if (node_exp) {
			xrule.r_object.obj_type = XPLY_OT_NODE;
			sprintf(xrule.r_object.obj_content.objc_object,
				"%s", node_exp);
		} else {
			printf("Missing paramter: -file or -node\n");
			return 1;
		}
		//sprintf(xrule.r_object.obj_content.objc_object, "%s", node_exp);              
	} else {
		printf("Missing parameters: -dn or -grp\n");
		usage(argv[0]);
		return 1;
	}

	if (is_alldigit(perm_exp) && *perm_exp >= '0' && *perm_exp <= '7')
		xrule.r_content.rc_perm = atoi(perm_exp);
	else {
		printf("Invalid parameter: '%s'\n", perm_exp);
		return 1;
	}



	xrule.r_valid = 1;
	xrule.r_time = time(0);

	char *handler = NULL;
	if (xos_policy_addrule(&xrule, &handler))
		printf("fail to add rule\n");


	return 0;
}
