/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/* usage:
 *	xos-policy-admin-am  	-dn xxx -vo xxx 
 *				-locnam xxx 
 *				-drvname xxx -drvparam xxx
 *	xos-policy-admin-am	-vo xxx --force
 */

#include <stdio.h>
#include <time.h>
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"


void usage(char *program)
{
	printf("Usage:\n");
	printf("\t%s\t -dn xxx -vo xxx -localname xxx"
	       " [-drvname xxx -drvparam xxx]\n", program);
	printf("\t%s\t -vo xxx --force\n",program);
	printf("\nExamples:\n");
	printf("\t%s -dn /CN=62a57c32-9c7f-4da2-a08d-1093f5e6bee8  \\ \n"
			"\t\t-vo dff93ffe-87f3-476a-b573-983f92623f95 \\ \n"
			"\t\t-localname john\n",program);
	printf("\t%s -vo dff93ffe-87f3-476a-b573-983f92623f95 --force\n",program);

}

int main(int argc, char *argv[])
{

	char *dn_exp, *vo_exp, *local_name;
	char *drv_name, *drv_param;
	struct rule_t xrule;
	int i,force_created;

	dn_exp = vo_exp = local_name = drv_name = drv_param = NULL;
	force_created = 0;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-dn")) {
			dn_exp = argv[++i];
		} else if (!strcmp(argv[i], "-vo")) {
			vo_exp = argv[++i];
		} else if (!strcmp(argv[i], "-localname")) {
			local_name = argv[++i];
		} else if (!strcmp(argv[i], "-drvname")) {
			drv_name = argv[++i];
		} else if (!strcmp(argv[i], "-drvparam")) {
			drv_param = argv[++i];
		} else if (!strcmp(argv[i], "--force")) {
			 force_created = 1;
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters: '%s'!\n", argv[i]);
	}

	if (getuid() != 0) {
		printf("Please run with root privilege, thanks\n");
		return 0;
	}

	memset(&xrule, 0, sizeof(struct rule_t));
	if(force_created && vo_exp != NULL) {
		dn_exp = local_name = strdup("*");
	}

	/* check and fill in the rule */
	if (dn_exp && vo_exp && local_name) {
		xrule.r_type = XPLY_RT_MA;

		/* Subject */
		xrule.r_subject.sub_type = XPLY_ST_ACT;
		sprintf(xrule.r_subject.sub_content_1, "%s", dn_exp);
		sprintf(xrule.r_subject.sub_content_2, "%s", vo_exp);

		/* Object */
		xrule.r_object.obj_type = XPLY_OT_ACT;
		sprintf(xrule.r_object.obj_content.objc_object, "%s",
			local_name);
	} else {
		printf("Missing parameters: dn, vo, or localname?\n");
		usage(argv[0]);
		return 1;
	}

	if (!drv_name || !drv_param) {
		//printf
		//    ("Warning: missing driver name or driver parameters\n");
		//usage(argv[0]);
	} else {
		sprintf(xrule.r_content.rc_driver.driver_name, "%s",
			drv_name);
		sprintf(xrule.r_content.rc_driver.driver_params, "%s",
			drv_param);
	}

	xrule.r_valid = 1;
	xrule.r_time = time(0);

	char *handler = NULL;
	if (xos_policy_addrule(&xrule, &handler))
		printf("fail to add rule\n");

	return 0;
}
