/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 *	xos-policy-admin-cleardb [ -ma  -ru  -ac  -ap -mapdb -ruledb -all]
 *
 */

#include <stdio.h>
#include "map_mech.h"
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"
#include "xos_cache.h"


void usage(char *program)
{
	printf
	    ("Usage:\n\t%s [-h|-help|--help] [-ma -ru -ac -ap -set -mapdb -ruledb -cache -all]\n",
	     program);
	printf("\t-ma\t\tremove data of mapping rules from database\n");
	printf("\t-ru\t\tremove data of resource usage rules from database\n");
	printf("\t-ac\t\tremove data of access control rules from database\n");
	printf("\t-set\t\tremove data of uid/gid scope from databases\n");
	printf("\t-mapdb\t\tremove data of mapping information from database\n");
	printf("\t-ruledb\t\tremove all rules from databases\n");
	printf("\t-cache\t\tremove AMS client \n");
	printf("\t-all\t\tremove all database\n");
	printf("\t-h|-help|--help\tprint help\n");

}

int main(int argc, char *argv[])
{

	struct rule_t *xrule = NULL;
	char *xhandler = NULL;
	int ret, i, clear_ma, clear_ru, clear_ac, clear_ap, clear_setting, clear_cache,
	    clear_alldb;
	int clear_mapdb, clear_ruledb;
	char cmd = 'n';

	clear_ma = clear_ru = clear_ac = clear_ap = clear_setting =
	    clear_alldb = 0;
	clear_mapdb = clear_ruledb = clear_cache = 0;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-ma")) {
			clear_ma = 1;
		} else if (!strcmp(argv[i], "-ru")) {
			clear_ru = 1;
		} else if (!strcmp(argv[i], "-ac")) {
			clear_ac = 1;
		} else if (!strcmp(argv[i], "-ap")) {
			clear_ap = 1;
		} else if (!strcmp(argv[i], "-set")) {
			clear_setting = 1;
		} else if (!strcmp(argv[i], "-mapdb")) {
			clear_mapdb = 1;
                } else if (!strcmp(argv[i], "-cache")) {
			clear_cache = 1;
		} else if (!strcmp(argv[i], "-ruledb")) {
			clear_ruledb = 1;
		} else if (!strcmp(argv[i], "-all")) {
			clear_alldb = 1;
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

	if (getuid() != 0) {
		printf("run with root privilege,thanks\n");
		return 0;
	}


	if (clear_alldb) {
		printf("Are you sure all databases are deleted? (y/n):");
		scanf("%c",&cmd);
		if(cmd != 'y') return 0;
		clear_mapdb = 1;
		clear_ruledb = 1;
		clear_cache = 1;

	}
	/* print rule databases */

	if (clear_ruledb) {
		//printf("Are you sure rule databases are deleted? [y/N]");
		ruleinfo_db_clearall();
	} else {
		ret = -1;
		for (i = 1; i < ruleinfo_db_newhandler(); i++) {

			if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
				printf("Can not get rule !\n");
				break;
			}
			if (!xrule)
				continue;


			/* find the rule */
			if ((xrule->r_type == XPLY_RT_MA && clear_ma)
			    || (xrule->r_type == XPLY_RT_AC && clear_ac)
			    || (xrule->r_type == XPLY_RT_RU && clear_ru)
			    || (xrule->r_type == XPLY_RT_AP && clear_ap)
			    ) {
				printf("clearing [key:%d]:%32s .... \t", i,
				       xhandler);
				if (xos_policy_removerule(xhandler) == 0)
					printf("OK");
				printf("\n");
			}


		}

		if (clear_setting) {
			if (ruleinfo_db_getbykey(-1, &xrule, &xhandler) ==
			    0) {
				printf("clearing [key:-1]:%32s .... \t",
				       xhandler);
				if (xos_policy_removerule(xhandler) == 0)
					printf("OK");
				printf("\n");
			}

		}
	}

	if (clear_mapdb) {
		//printf
		//    ("Are you sure mapping databases are deleted? [y/N]\n");
		clear_all();
	}

	if (clear_cache) {
		printf("Clear cache ...");
		ams_cache_clear();
		printf("done\n");
	}

	return 0;
}
