/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 * xos-policy-admin-del [-num | -key] xxx
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <xos_policy_db.h>
#include "xos_policy_mgmt.h"



void usage(char *program)
{
	printf("Usage:\n\t%s [-num | -key] xxx\n", program);
}

int is_alldigit(char *str)
{
	char *p = str;
	while (*p != '\n' && *p != '\0') {
		if (!isdigit(*p))
			return 0;
		p++;
	}

	return 1;
}

int main(int argc, char *argv[])
{

	char *rulekey_exp, *rule_handler;
	int i, rulekey_type, rulekey_no;

	rulekey_exp = rule_handler = NULL;
	rulekey_type = rulekey_no = -1;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-num")) {
			if (rulekey_type == -1) {
				rulekey_type = 0;
				rulekey_exp = argv[++i];
			} else
				printf("Parameter: '-num' error!\n");
		} else if (!strcmp(argv[i], "-key")) {
			if (rulekey_type == -1) {
				rulekey_type = 1;
				rulekey_exp = argv[++i];
			} else
				printf("Parameter: '-key' error!\n");
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

	if (getuid() != 0) {
		printf("run with root privilege,thanks\n");
		return 0;
	}

	/* remove the rule */
	if (rulekey_type == -1) {
		printf("Missing parameter: -num or -key !\n");
		usage(argv[0]);
		return 1;
	}

	if (rulekey_type == 0) {

		if (!is_alldigit(rulekey_exp)) {
			printf("'%s' should be integer\n", rulekey_exp);
			return 1;
		}

		rulekey_no = atoi(rulekey_exp);
		if (ruleinfo_db_getbykey(rulekey_no, NULL, &rule_handler)) {
			printf("Can not file the Rule.[NO.%d]\n",
			       rulekey_no);
			return 1;
		}

	} else
		rule_handler = rulekey_exp;

	if (!rule_handler || xos_policy_removerule(rule_handler))
		printf("fail to remove rule\n");


	return 0;
}
