/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 *	xos-policy-admin-gm  	-vo xxx [-grp | -role] xxx
 *				-locgrp xxx 
 *				-drvname xxx -drvparam xxx
 *	xos-policy-admin-gm	-vo xxx --force
 */

#include <stdio.h>
#include <time.h>
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"


void usage()
{
	printf("Usage:\n");
	printf("\txos-policy-admin-gm\t-vo xxx -grp xxx"
	       " -locgrp xxx -drvname xxx -drvparam xxx\n");
	printf("\txos-policy-admin-gm\t-vo xxx --force\n");
	printf("\nExamples:\n");
	printf("\txos-policy-admin-gm\t-vo 62a57c32-9c7f-4da2-a08d-1093f5e6bee8  --force\n");

}

int main(int argc, char *argv[])
{

	char *grp_exp, *vo_exp, *local_group;
	char *drv_name, *drv_param;
	struct rule_t xrule;
	int i,force_created;

	grp_exp = vo_exp = local_group = drv_name = drv_param = NULL;
	force_created = 1;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-grp") || !strcmp(argv[i], "-role")) {
			grp_exp = argv[++i];
		} else if (!strcmp(argv[i], "-vo")) {
			vo_exp = argv[++i];
		} else if (!strcmp(argv[i], "-locgrp")) {
			local_group = argv[++i];
		} else if (!strcmp(argv[i], "-drvname")) {
			drv_name = argv[++i];
		} else if (!strcmp(argv[i], "-drvparam")) {
			drv_param = argv[++i];
		} else if (!strcmp(argv[i], "--force")) {
			force_created = 1;
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage();
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

        if (getuid() != 0) {
                printf("Please run with root privilege, thanks\n");
                return 0;
        }

	memset(&xrule, 0, sizeof(struct rule_t));
	if(force_created && vo_exp) {
		grp_exp = local_group = strdup("*");
	}

	/* check and fill in the rule */
	if (grp_exp && vo_exp && local_group) {
		xrule.r_type = XPLY_RT_MA;

		/* Subject */
		xrule.r_subject.sub_type = XPLY_ST_GRP;
		sprintf(xrule.r_subject.sub_content_1, "%s", vo_exp);
		sprintf(xrule.r_subject.sub_content_2, "%s", grp_exp);

		/* Object */
		xrule.r_object.obj_type = XPLY_OT_GRP;
		sprintf(xrule.r_object.obj_content.objc_object, "%s",
			local_group);
	} else {
		printf("Missing parameters: dn, vo, or localname?\n");
		usage();
		return 1;
	}

	if (drv_name) {
		sprintf(xrule.r_content.rc_driver.driver_name, "%s",
			drv_name);
		if (drv_param)
			sprintf(xrule.r_content.rc_driver.driver_params,
				"%s", drv_param);
	}

	xrule.r_valid = 1;
	xrule.r_time = time(0);

	char *handler = NULL;
	if (xos_policy_addrule(&xrule, &handler))
		printf("fail to add rule\n");

	return 0;
}
