/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 * xos-policy-admin-priority  -rule xxx -priority xxx
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include "xos_policy_mgmt.h"



void usage(char *program)
{
	printf("Usage:\n\t%s -rule xxx -priority xxx\n", program);
}

int is_alldigit(char *str)
{
	char *p = str;
	while (*p != '\n' && *p != '\0') {
		if (!isdigit(*p))
			return 0;
		p++;
	}

	return 1;
}

int main(int argc, char *argv[])
{

	char *rulekey_exp, *rulepri_exp;
	int i, rule_no, rule_pri;

	rulekey_exp = rulepri_exp = NULL;
	rule_no = rule_pri = -1;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-rule")) {
			rulekey_exp = argv[++i];
		} else if (!strcmp(argv[i], "-priority")) {
			rulepri_exp = argv[++i];
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

	if (getuid() != 0) {
		printf("run with root privilege,thanks\n");
		return 0;
	}

	/* check parameters */
	if (!rulekey_exp || !rulepri_exp) {
		usage(argv[0]);
		return 1;
	}

	if (is_alldigit(rulekey_exp))
		rule_no = atoi(rulekey_exp);
	else {
		printf("Parameter '%s' need to be integer\n", rulekey_exp);
		return 1;
	}

	if (is_alldigit(rulepri_exp))
		rule_pri = atoi(rulepri_exp);
	else {
		printf("Parameter '%s' need to be integer\n", rulepri_exp);
		return 1;
	}


	if (xos_policy_setriority(rule_no, rule_pri))
		printf("Can not change priority of rule [%d -> %d]\n",
		       rule_no, rule_pri);


	return 0;
}
