/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 *	xos-policy-admin-prt [ -ma  -ru  -ac  -ap -pwd -grp]
 *
 */

#include <stdio.h>
#include <time.h>
#include "xos_dbaux.h"
#include "xos_policy_db.h"
#include "xos_cache.h"

void usage(char *program)
{
	printf("Usage:\n\t%s 	[ -ma  -ru  -ac  -ap -set -pwd -grp -cache]\n",
	       program);
        printf("\t-ma\tprint mapping rules of database\n");
        printf("\t-ru\tprint resource usage rules of databases\n");
        printf("\t-ac\tprint access control rules of database\n");
        printf("\t-set\tprint seting rules of uid/gid scope\n");
        printf("\t-pwd\tdump GPASSWD mapping information of database\n");
        printf("\t-grp\tdump GGROUPS mapping information of database\n");
        printf("\t-cache\tdump the AMS cache\n");
}

int main(int argc, char *argv[])
{

	struct rule_t *xrule = NULL;
	char *xhandler = NULL;
	int ret, i, prt_ma, prt_ru, prt_ac, prt_ap, prt_setting, prt_all;
	int prt_pwd, prt_grp, prt_rule, prt_null,prt_cache;

	prt_ma = prt_ru = prt_ac = prt_ap = prt_setting = prt_all = 0;
	prt_pwd = prt_grp = prt_rule = prt_null = prt_cache = 0;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-ma")) {
			prt_ma = 1;
		} else if (!strcmp(argv[i], "-ru")) {
			prt_ru = 1;
		} else if (!strcmp(argv[i], "-ac")) {
			prt_ac = 1;
		} else if (!strcmp(argv[i], "-ap")) {
			prt_ap = 1;
		} else if (!strcmp(argv[i], "-set")) {
			prt_setting = 1;
		} else if (!strcmp(argv[i], "-pwd")) {
			prt_pwd = 1;
		} else if (!strcmp(argv[i], "-grp")) {
			prt_grp = 1;
                } else if (!strcmp(argv[i], "-cache")) {
			prt_cache = 1;
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}


	if (prt_ma && prt_ru && prt_ac && prt_ap && prt_setting)
		prt_all = 1;
	else if (!prt_ma && !prt_ru && !prt_ac && !prt_ap && !prt_setting)
		prt_null = 1;

	if (prt_ma || prt_ru || prt_ac || prt_ap || prt_setting || prt_all)
		prt_rule = 1;
	else if (prt_null && !prt_pwd && !prt_grp && !prt_cache) {
		prt_all = 1;
		prt_rule = 1;
	}


	/* print rule databases */
	if (prt_rule)
		printf("----------- rule database ---------\n");

	ret = -1;
	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			printf("Can not get rule !\n");
			break;
		}
		if (!xrule)
			continue;


		/* find the rule */
		if (prt_all || ((xrule->r_type == XPLY_RT_MA && prt_ma)
				|| (xrule->r_type == XPLY_RT_AC && prt_ac)
				|| (xrule->r_type == XPLY_RT_RU && prt_ru)
				|| (xrule->r_type == XPLY_RT_AP && prt_ap)
		    )) {
			printf("[key:%d]:%32s\n", i, xhandler);
			ruleinfo_db_printdata(xrule);
			printf("\n");
		}


	}

	if (prt_all || prt_setting) {
		if (ruleinfo_db_getbykey(-1, &xrule, &xhandler) == 0) {
			printf("[key:-1]:%32s\n", xhandler);
			//ruleinfo_db_printdata(xrule);
			printf("uid space:\n");
			printf("\tuid_max: %s\n",
			       xrule->r_subject.sub_content_1);
			printf("\tuid_min: %s\n",
			       xrule->r_subject.sub_content_2);
			printf("gid space:\n");
			printf("\tgid_max: %s\n",
			       xrule->r_content.rc_driver.driver_name);
			printf("\tgid_min: %s\n",
			       xrule->r_content.rc_driver.driver_params);
		}

	}

	if (prt_rule)
		printf("----------------------------------\n");

	/* print mapping databases */
	if (prt_pwd) {
		printf("\n------------ PASSWD -------------\n");
		bdb_printdb_pwd();
		printf("\n");
	}

	if (prt_grp) {
		printf("\n------------ GROUP -------------\n");
		bdb_printdb_grp();
		printf("\n");
	}

	if(prt_cache) {
		printf("------------ dump cache -----------\n");
		ams_cache_dump();
		printf("\n");
	}

	return 0;
}
