/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* usage:
 * xos-policy-admin-set -uidmax xxx -uidmin xxx -gidmax xxx -gidmin xxx
 *
 */

#include <stdio.h>
#include <ctype.h>
#include "xos_policy_rule.h"
#include "xos_policy_db.h"



void usage(char *program)
{
	printf
	    ("Usage:\n\t%s -uidmax xxx -uidmin xxx -gidmax xxx -gidmin xxx\n",
	     program);
	printf("\t-uidmax\tset maximal number for uid mapping\n");
	printf("\t-uidmin\tset minimal number for uid mapping\n");
	printf("\t-gidmax\tset maximal number for gid mapping\n");
	printf("\t-gidmin\tset minimal number for gid mapping\n");
	printf("\nExamples:\n");
	printf("\t%s -uidmax 60500 -uidmin 60000 -gidmax 60500 -gidmin 60000\n",program);
}

int is_alldigit(char *str)
{
	char *p = str;
	while (*p != '\n' && *p != '\0') {
		if (!isdigit(*p))
			return 0;
		p++;
	}

	return 1;
}

int main(int argc, char *argv[])
{

	char *uidmax_exp, *uidmin_exp, *gidmax_exp, *gidmin_exp;
	char *xhandler = NULL;
	int i, idparam, created;
	struct rule_t *xrule = NULL;

	uidmax_exp = uidmin_exp = gidmax_exp = gidmin_exp = NULL;
	idparam = created = -1;

	/* parse parameters */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-uidmax")) {
			uidmax_exp = argv[++i];
		} else if (!strcmp(argv[i], "-uidmin")) {
			uidmin_exp = argv[++i];
		} else if (!strcmp(argv[i], "-gidmax")) {
			gidmax_exp = argv[++i];
		} else if (!strcmp(argv[i], "-gidmin")) {
			gidmin_exp = argv[++i];
		} else if (!strcmp(argv[i], "-help")||!strcmp(argv[i], "--help")||!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			return 0;
		} else
			printf("Unknown parameters !\n");
	}

	if (getuid() != 0) {
		printf("run with root privilege,thanks\n");
		return 0;
	}

	/* check parameters */
	if (!uidmax_exp && !uidmin_exp && !gidmax_exp && !gidmin_exp) {
		usage(argv[0]);
		return 1;
	}

	if (ruleinfo_db_getbykey(-1, &xrule, &xhandler)) {
		/* VO-setting rule does not exist, create one */
		if (uidmax_exp && uidmin_exp && gidmax_exp && gidmin_exp)
			created = 1;
		xrule = (struct rule_t *) malloc(sizeof(struct rule_t));
		if (!xrule)
			return -1;
		memset(xrule, 0, sizeof(struct rule_t));
		xrule->r_type = XPLY_RT_MA;
	}

	/* change uidmax */
	if (uidmax_exp) {
		if (is_alldigit(uidmax_exp)) {
			idparam = atoi(uidmax_exp);
			sprintf(xrule->r_subject.sub_content_1, "%d",
				idparam);
		} else {
			printf("Parameter '%s' need to be integer\n",
			       uidmax_exp);
			return 1;
		}
	}

	/* change uidmin */
	if (uidmin_exp) {
		if (is_alldigit(uidmin_exp)) {
			idparam = atoi(uidmin_exp);
			sprintf(xrule->r_subject.sub_content_2, "%d",
				idparam);
		} else {
			printf("Parameter '%s' need to be integer\n",
			       uidmin_exp);
			return 1;
		}
	}

	/* change gidmax */
	if (gidmax_exp) {
		if (is_alldigit(gidmax_exp)) {
			idparam = atoi(gidmax_exp);
			sprintf(xrule->r_content.rc_driver.driver_name,
				"%d", idparam);
		} else {
			printf("Parameter '%s' need to be integer\n",
			       gidmax_exp);
			return 1;
		}
	}


	/* change gidmin */
	if (gidmin_exp) {
		if (is_alldigit(gidmin_exp)) {
			idparam = atoi(gidmin_exp);
			sprintf(xrule->r_content.rc_driver.driver_params,
				"%d", idparam);
		} else {
			printf("Parameter '%s' need to be integer\n",
			       gidmin_exp);
			return 1;
		}
	}

	if (!created && ruleinfo_db_delete(xhandler)) {
		printf("Can not remove old parameters\n");
		return 1;
	}

	if (ruleinfo_db_put_aux(xrule, -1, NULL)) {
		printf("Can not set new parameters\n");
	}



	return 0;
}
