/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* old protocol */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/un.h>

#include "xos.h"
#include "xos_utils.h"
#include "xos_dbaux.h"
#include "xos_policy_rule.h"
#include "xos_protocol.h"
#include "xos_readconf.h"

#define NULL_POINTER	"####"



#define DEBUG_INFO	1

#if DEBUG_INFO
#define PTL_DEBUG(args...)	DEBUG(args)
#else
#define	PTL_DEBUG(args...)	((void)(0))
#endif



/** AMS client interfaces */
int amsclient_packetmsg(char *message, OPTTYPE optcode, char *mapped_dn,
			char *mapped_vo, char *mapped_role,
			char *mapped_subgroups)
{

	sprintf(message, "%d#", optcode);
	if (mapped_dn)
		sprintf(message, "%s%s#", message, mapped_dn);
	else
		sprintf(message, "%s#", message);

	if (mapped_vo)
		sprintf(message, "%s%s#", message, mapped_vo);
	else
		sprintf(message, "%s#", message);

	if (mapped_role)
		sprintf(message, "%s%s#", message, mapped_role);
	else
		sprintf(message, "%s#", message);

	if (mapped_subgroups)
		sprintf(message, "%s%s#", message, mapped_subgroups);
	else
		sprintf(message, "%s#", message);

	return 0;

}



int amsclient_unpacketmsg(char *message, GPASSWD * gpwd,
			  GGROUPS * ggrp, int *ret)
{
	if (!message)
		return -1;
	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '#') {
			*p = '\0';

			if (tno == 0)
				*ret = atoi(pp);
			if (gpwd) {
				if (tno == 1 && p != pp + 1)
					strncpy(gpwd->g_idtoken.g_dn,
						pp + 1, MAXFIELD);
				if (tno == 2 && p != pp + 1)
					strncpy(gpwd->g_idtoken.g_vo,
						pp + 1, MAXFIELD);
				if (tno == 3 && p != pp + 1)
					strncpy(gpwd->g_idtoken.g_role,
						pp + 1, MAXFIELD);
				if (tno == 4 && p != pp + 1)
					strncpy(gpwd->l_idtoken.
						g_mappedname, pp + 1,
						MAXFIELD);
				if (tno == 5 && p != pp + 1)
					gpwd->l_idtoken.g_mappeduid =
					    atoi(pp + 1);
				if (tno == 6 && p != pp + 1)
					gpwd->l_idtoken.g_mappedgid =
					    atoi(pp + 1);
				if (tno == 7 && p != pp + 1)
					strncpy(gpwd->g_home, pp + 1,
						MAXFIELD);
				if (tno == 8 && p != pp + 1)
					strncpy(gpwd->g_shell, pp + 1,
						MAXFIELD);
				if (tno == 9 && p != pp + 1)
					gpwd->g_time = atol(pp + 1);
			}
			if (ggrp) {
				if (tno == 10 && p != pp + 1)
					strncpy(ggrp->g_grptoken.g_vo,
						pp + 1, MAXFIELD);
				if (tno == 11 && p != pp + 1)
					strncpy(ggrp->g_grptoken.
						g_subgroup, pp + 1,
						MAXFIELD);
				if (tno == 12 && p != pp + 1)
					strncpy(ggrp->l_grptoken.
						g_mappedname, pp + 1,
						MAXFIELD);
				if (tno == 13 && p != pp + 1)
					ggrp->l_grptoken.g_mappedgid =
					    atoi(pp + 1);
				if (tno == 14 && p != pp + 1)
					strncpy(ggrp->l_grptoken.g_members,
						pp + 1, MAXFIELD);
				if (tno == 15 && p != pp + 1)
					ggrp->g_time = atol(pp + 1);
			}
			pp = p;
			tno++;
		}


		p++;
	}

	return 0;

}


int amsclient_unpacketmsg_tmpinfo(char *message, char *tmpinfo, int *ret)
{
	if (!message)
		return -1;

	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '#') {
			*p = '\0';

			if (tno == 0)
				*ret = atoi(pp);

			if (tno == 1 && p != pp + 1)
				strcpy(tmpinfo, pp + 1);

			pp = p;
			tno++;
		}
		p++;
	}

	return 0;
}


/* client interfaces */

/** open the connection and return a socket
 * if successful.
 */
int amsclient_connect_open()
{
	int sockfd, port;
	struct sockaddr_in pin;
	struct hostent *shost_name;
	char *host;
	ConfData confdata;
	
	/* read configuration */
	initialize_confdata(&confdata);
	if (xos_read_config_file(PAM_CONF, &confdata, 0) != 1) {
		PTL_DEBUG
		    ("AMS_Client:Can not read configuration file '%s'",
		     PAM_CONF);
		return -1;
	}	
        if (confdata.ams_server_is_local) {
            struct sockaddr_un sock_address;
            const char *sock_name="/var/run/ams_server.socket";

            bzero(&sock_address,sizeof(sock_address));
            sock_address.sun_family=AF_UNIX;
            strcpy(sock_address.sun_path,sock_name);

            sockfd=socket(PF_UNIX,SOCK_SEQPACKET,0);
            if (sockfd<0) {
		PTL_DEBUG("Error opening socket\n");
		return -1;
            }
            if (connect(sockfd,(struct sockaddr*)&sock_address,
              sizeof(sock_address))) {
            }
            return sockfd;

        } else {
	    host = confdata.ams_server;
	    port = confdata.ams_port;
	
	    /* open AMS connection */
	    if ((shost_name = gethostbyname(host)) == 0) {
		PTL_DEBUG("Error resolving local host\n");
		return -1;
	    }

	    bzero(&pin, sizeof(pin));
	    pin.sin_family = AF_INET;
	    pin.sin_addr.s_addr = htonl(INADDR_ANY);
	    pin.sin_addr.s_addr =
	      ((struct in_addr *) (shost_name->h_addr))->s_addr;
	    pin.sin_port = htons(port);

	   if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		PTL_DEBUG("Error opening socket\n");
		return -1;
	   }

	   if (connect(sockfd, (void *) &pin, sizeof(pin)) == -1) {
		PTL_DEBUG("Error connecting to socket\n");
		return -1;
	   }
	
	   return sockfd;
       }
}

int amsclient_connect_query(int sockfd, char *message, int *message_len)
{
	int n;
	
	if(sockfd <= 0 || !message || !message_len)
		return -1;
	
	PTL_DEBUG("AMS_Client:Sending message '%s' ...", message);
	
	if (write(sockfd, message, strlen(message)) == -1) {
		PTL_DEBUG("Error in write\n");
		return -1;
	}
	PTL_DEBUG("AMS_Client:sent message.. wait for response...");
#if 0
	while ((n = read(sockfd, message, PACKAGE_MAXLEN)) > 0) {
		*message_len = n;
	}
#else
	int i = 0;
        while (i < 1 && (n = read(sockfd, message, PACKAGE_MAXLEN)) > 0) {
                *message_len = n;
		i++;
        }
#endif
	if (n < 0) {

		PTL_DEBUG("AMS_Client:\nResponse from server:\n\n%s\n",
			  message);
		return -1;
	}
	
	return 0;
}

int amsclient_connect_close(int sockfd)
{
	
	int msg_len = 0;

	if(sockfd > 0) {
		
		amsclient_connect_query(sockfd, MSG_END, &msg_len);
		close(sockfd);
	}
	
	return 0;
}



/** AMS server interfaces */
int amsserver_packetmsg(char *message, GPASSWD * gpwd, GGROUPS * ggrp,
			int ret)
{
	if (!message)
		return -1;

	sprintf(message, "%d#", ret);

	if (gpwd) {
		sprintf(message, "%s%s#%s#%s#%s#%d#%d#%s#%s#%ld#",
			message, gpwd->g_idtoken.g_dn,
			gpwd->g_idtoken.g_vo, gpwd->g_idtoken.g_role,
			gpwd->l_idtoken.g_mappedname,
			gpwd->l_idtoken.g_mappeduid,
			gpwd->l_idtoken.g_mappedgid, gpwd->g_home,
			gpwd->g_shell, gpwd->g_time);
	} else
		sprintf(message, "%s#########", message);
	if (ggrp) {
		sprintf(message, "%s%s#%s#%s#%d#%s#%ld#", message,
			ggrp->g_grptoken.g_vo,
			ggrp->g_grptoken.g_subgroup,
			ggrp->l_grptoken.g_mappedname,
			ggrp->l_grptoken.g_mappedgid,
			ggrp->l_grptoken.g_members, ggrp->g_time);
	} else
		sprintf(message, "%s######", message);
	return 0;
}



int amsserver_packetmsg_tmpinfo(char *message, char *tmpinfo, int ret)
{
	if (!message)
		return -1;

	sprintf(message, "%d#", ret);


	if (tmpinfo) {
		sprintf(message, "%s%s#", message, tmpinfo);
	} else
		sprintf(message, "%s#", message);

	return 0;
}

int protocol_packetrule(char *message, struct rule_t *xrule)
{
	if (!message)
		return -1;
	if (!xrule)
		return 0;

	if (xrule->r_type == XPLY_RT_NUL)
		return 0;
	else
		sprintf(message, "%d|", xrule->r_type);

	/* struct subject_t */
	sprintf(message, "%s%d|", message, xrule->r_subject.sub_type);
	sprintf(message, "%s%s|", message, xrule->r_subject.sub_content_1);
	sprintf(message, "%s%s|", message, xrule->r_subject.sub_content_2);

	/* struct object_t */
	sprintf(message, "%s%d|", message, xrule->r_object.obj_type);
	if (xrule->r_object.obj_type == XPLY_OT_QUOTA)
		sprintf(message, "%s%d|", message,
			xrule->r_object.obj_content.objc_restype);
	else
		sprintf(message, "%s%s|", message,
			xrule->r_object.obj_content.objc_object);

	/*the content of rule */
	if (xrule->r_type == XPLY_RT_AC)
		sprintf(message, "%s%d|", message,
			xrule->r_content.rc_perm);
	else if (xrule->r_type == XPLY_RT_RU) {
		sprintf(message, "%s%d|", message,
			xrule->r_content.rc_quota.quo_type);
		sprintf(message, "%s%f|", message,
			xrule->r_content.rc_quota.quo_val);
	} else {
		sprintf(message, "%s%s|", message,
			xrule->r_content.rc_driver.driver_name);
		sprintf(message, "%s%s|", message,
			xrule->r_content.rc_driver.driver_params);
	}

	/* valid or not */
	sprintf(message, "%s%d|", message, xrule->r_valid);

	/* time */
	sprintf(message, "%s%ld|", message, xrule->r_time);

	return 0;
}

int protocol_unpacketrule(char *message, struct rule_t *xrule)
{
	if (!message || !xrule)
		return -1;
	if (message[0] == '\0')
		return -1;

	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '|') {
			*p = '\0';

			if (tno == 0) {
				xrule->r_type = atoi(pp);
				if (xrule->r_type == XPLY_RT_NUL)
					break;
			}

			if (tno == 1 && p != pp + 1)
				xrule->r_subject.sub_type = atoi(pp + 1);

			if (tno == 2 && p != pp + 1)
				strncpy(xrule->r_subject.sub_content_1,
					pp + 1, XPLY_MAXLEN);
			if (tno == 3 && p != pp + 1)
				strncpy(xrule->r_subject.sub_content_2,
					pp + 1, XPLY_MAXLEN);
			if (tno == 4 && p != pp + 1)
				xrule->r_object.obj_type = atoi(pp + 1);
			if (tno == 5 && p != pp + 1) {
				if (xrule->r_object.obj_type ==
				    XPLY_OT_QUOTA)
					xrule->r_object.obj_content.
					    objc_restype = atoi(pp + 1);
				else
					strncpy(xrule->r_object.
						obj_content.objc_object,
						pp + 1, XPLY_MAXLEN);
			}
			if (tno == 6 && p != pp + 1) {
				if (xrule->r_type == XPLY_RT_AC)
					xrule->r_content.rc_perm =
					    atoi(pp + 1);
				else if (xrule->r_type == XPLY_RT_RU)
					xrule->r_content.rc_quota.
					    quo_type = atoi(pp + 1);
				else
					strncpy(xrule->r_content.rc_driver.
						driver_name, pp + 1,
						XPLY_MAXLEN);
			}
			if (tno == 7 && p != pp + 1) {
				if (xrule->r_type == XPLY_RT_AC)
					xrule->r_valid = atoi(pp + 1);
				else if (xrule->r_type == XPLY_RT_RU)
					xrule->r_content.rc_quota.quo_val =
					    atof(pp + 1);
				else
					strncpy(xrule->r_content.rc_driver.
						driver_params, pp + 1,
						XPLY_MAXLEN);
			}
			if (tno == 8 && p != pp + 1) {
				if (xrule->r_type == XPLY_RT_AC)
					xrule->r_time = atol(pp + 1);
				else
					xrule->r_valid = atoi(pp + 1);
			}
			if (tno == 9 && p != pp + 1)
				xrule->r_time = atol(pp + 1);

			pp = p;
			tno++;
		}
		p++;
	}

	return 0;
}


int protocol_packetsubject(char *message, struct subject_t *sub)
{
	if (!message)
		return -1;
	if (!sub)
		return 0;

	sprintf(message, "%d|", sub->sub_type);
	sprintf(message, "%s%s|", message, sub->sub_content_1);
	sprintf(message, "%s%s|", message, sub->sub_content_2);

	return 0;
}

int protocol_unpacketsubject(char *message, struct subject_t *sub)
{
	if (!message || !sub)
		return -1;

	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '|') {
			*p = '\0';

			if (tno == 0)
				sub->sub_type = atoi(pp);

			if (tno == 1 && p != pp + 1)
				strncpy(sub->sub_content_1, pp + 1,
					XPLY_MAXLEN);

			if (tno == 2 && p != pp + 1)
				strncpy(sub->sub_content_2, pp + 1,
					XPLY_MAXLEN);

			pp = p;
			tno++;
		}
		p++;
	}

	return 0;
}


int amscient_unpacketmsg_rule(char *message, struct rule_t *xrule,
			      char *handler, int *ret)
{
	if (!message)
		return -1;

	char *p = NULL, *pp = NULL;
	char *istr = NULL;
	int tno = 0;
	char rule_buf[8192];

	istr = (char *) malloc(strlen(message) + 1);
	if (!istr)
		return -1;

	strcpy(istr, message);
	pp = p = message;

	while (*p != '\0') {
		if (*p == '#') {
			*p = '\0';

			if (tno == 0)
				*ret = atoi(pp);
			if (tno == 1 && p != pp + 1)
				strcpy(rule_buf, pp + 1);
			if (tno == 2 && p != pp + 1) {
				if (handler)
					strncpy(handler, pp + 1, 32);
			}

			pp = p;
			tno++;
		}
		p++;
	}

	if (*ret == 0 && xrule) {
		if (protocol_unpacketrule(rule_buf, xrule))
			return -1;
	}

	return 0;

}

int amsserver_packetmsg_rule(char *message, struct rule_t *xrule,
			     char *handler, int ret)
{
	if (!message)
		return -1;
	char rule_buf[8190];
	char *packet_rule = NULL, *packet_handler = NULL;

	if (xrule) {
		if (protocol_packetrule(rule_buf, xrule) == 0)
			packet_rule = rule_buf;
		else
			return -1;
	}

	if (handler)
		packet_handler = handler;


	sprintf(message, "%d#", ret);

	if (packet_rule)
		sprintf(message, "%s%s#", message, packet_rule);
	else
		sprintf(message, "%s#", message);

	if (packet_handler)
		sprintf(message, "%s%s#", message, packet_handler);
	else
		sprintf(message, "%s#", message);

	return 0;
}
