/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_utils.c
 * @brief provide some handlers of debug, log, etc.
 */
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>		/* getenv */
#include <string.h>
#include <fcntl.h>

#include <syslog.h>
#include <unistd.h>

#include "xos_readconf.h"
#include "xos_utils.h"


int xos_is_user_global(void)
{
	return getenv("XOS_ENV") != NULL;
}

#if 0
void xos_debug(const char *file, int line, const char *fmt, ...)
{
	char msgbuf[1024];

	va_list args;

	va_start(args, fmt);
#ifdef _DEBUG_SYSLOG
	xos_log("xos_log", LOG_CONS | LOG_PID, LOG_USER, LOG_DEBUG, fmt,
		args);
#else
	vsnprintf(msgbuf, sizeof(msgbuf), fmt, args);
	fprintf(stderr, "%s:%d: %s\n", file, line, msgbuf);
#endif
	va_end(args);
}

#else

void xos_debug(const char *level, const char *file, int line, const char *fmt, ...)
{
	char msgbuf[1024];
	va_list args;

#ifndef _DEBUG_SYSLOG
	ConfData confdata;

	initialize_confdata(&confdata);
	if (xos_read_config_file(PAM_CONF, &confdata, 0) != 1)
		fill_default_confdata(&confdata);
#endif


	va_start(args, fmt);
	vsnprintf(msgbuf, sizeof(msgbuf), fmt, args);

#ifdef _DEBUG_SYSLOG
	xos_log(LOG_CONS | LOG_PID, LOG_USER, LOG_DEBUG, msgbuf);
#else
	//vsnprintf(msgbuf, sizeof(msgbuf), fmt, args); 
	if (confdata.use_syslog)
		xos_log(LOG_CONS | LOG_PID, LOG_USER, LOG_DEBUG, msgbuf);

	if (confdata.use_debug)
		fprintf(stderr, "[%s] %s:%d: %s\n", level, file, line, msgbuf);
#endif
	va_end(args);
}

#endif


void xos_log(int option, int facility, int level, const char *format, ...)
{
	va_list args;

	va_start(args, format);
	openlog("XtreemOS", option /*LOG_CONS|LOG_PID */ ,
		facility /*LOG_AUTHPRIV */ );
	vsyslog(level, format, args);
	va_end(args);
	closelog();
}

void fatal(const char *fmt, ...)
{
	char msgbuf[1024];

	va_list args;

	va_start(args, fmt);
	vsnprintf(msgbuf, sizeof(msgbuf), fmt, args);
	printf(msgbuf);
	printf("\n");
	va_end(args);

	exit(2);
}
