#include <sys/types.h>
#include <errno.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xos.h"
#include "xos_utils.h"
#include "xos_dbaux.h"
#include "xos_cache.h"
#include "xos_protocol.h"

#define CACHE_HASH_MAXLEN  50

const char *cache_name = "/etc/xos/nss_pam";

struct ams_cache_t {
	int type;
	char data[MAXFIELD];
};

#define IPC_KEY_XOS     654321

void ams_cache_create()
{
	int shm_id;
	key_t key;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	DEBUG("CACHE:shared mem size: %d",sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN);
	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			IPC_CREAT | 0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return;
	}
}

void ams_cache_destroy()
{
	int shm_id;
	key_t key;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			IPC_CREAT | 0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return;
	}
	shmctl (shm_id, IPC_RMID, NULL);
	DEBUG("CACHE:remove shared memory: %d",shm_id);

}

void ams_cache_dump()
{
	int i;

	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);

	for(i = 0; i < CACHE_HASH_MAXLEN; i++ ){
		if((*(tab+i)).type == 0) continue;
		printf("(%d) type: '[%d] %s'\n",i,(*(tab+i)).type,(*(tab+i)).type == 1?"GPWD":"GGRP");
		printf("     data: '%s'\n",(*(tab+i)).data);

	}
	if(shmdt(tab)==-1)
		perror(" detach error ");

	return;

}

void ams_cache_clear()
{
	int i;

	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);

	for(i = 0; i < CACHE_HASH_MAXLEN; i++ ){
		if((*(tab+i)).type == 0) continue;
                        (*(tab+i)).type = 0;
			(*(tab+i)).data[0] = '\0';

	}
	if(shmdt(tab)==-1)
		perror(" detach error ");

	return;


}

int ams_cache_gpwd_insert(GPASSWD *gpwd)
{
	char buffer[MAXFIELD];
	int i, ret = -1;

	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	if(!gpwd) return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return -1;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);

	amsserver_packetmsg(buffer,gpwd,NULL,0);
	DEBUG("CHACHE: gpwd insert '%s'",buffer);

	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++) {
		if((*(tab+i)).type == 0) {
			(*(tab+i)).type = 1;
			sprintf((*(tab+i)).data,"%s",buffer);
			ret = 0;
			break;
		}
	}

	if(shmdt(tab)==-1)
		perror(" detach error ");
	return ret;

}


int ams_cache_ggrp_insert(GGROUPS *ggrp)
{
	char buffer[MAXFIELD];
	int i, ret = -1;

	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	if(!ggrp) return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return -1;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);

	amsserver_packetmsg(buffer,NULL,ggrp,0);
	DEBUG("CACHE: ggrp insert '%s'",buffer);

	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++) {
		if((*(tab+i)).type == 0) {
			(*(tab+i)).type = 2;
			sprintf((*(tab+i)).data,"%s",buffer);
			ret = 0;
			break;
		}
	}

	if(shmdt(tab)==-1)
		perror(" detach error ");
	return ret;

}



int ams_cache_gpwd_erase(GPASSWD *gpwd)
{
	char hash_buffer[MAXFIELD];
	int i,ret = -1;



	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	if(!gpwd) return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return -1;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);
	amsserver_packetmsg(hash_buffer,gpwd,NULL,0);

	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++ ){
		if((*(tab+i)).type == 1
				&& !strncmp((*(tab+i)).data, hash_buffer,sizeof(hash_buffer))){ 
			(*(tab+i)).type = 0;
			(*(tab+i)).data[0] = '\0';
			ret = 0;
		}
	}
	if(shmdt(tab)==-1)
		perror(" detach error ");

	return ret;

}


int ams_cache_ggrp_erase(GGROUPS *ggrp)
{
	char hash_buffer[MAXFIELD];
	int i,ret = -1;



	int shm_id;
	key_t key;
	struct ams_cache_t *tab;

	if(!ggrp) return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,
			0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return -1;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);
	amsserver_packetmsg(hash_buffer,NULL,ggrp,0);

	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++ ){
		if((*(tab+i)).type == 1
				&& !strncmp((*(tab+i)).data, hash_buffer,sizeof(hash_buffer))){ 
			(*(tab+i)).type = 0;
			(*(tab+i)).data[0] = '\0';
			ret = 0;
		}
	}
	if(shmdt(tab)==-1)
		perror(" detach error ");

	return ret;

}


/*
void ams_gpwd_copy(GPASSWD *dest, GPASSWD *src)
{
	if(!dest || !src) return;
}

void ams_ggrp_copy(GGROUPS *dest, GGROUPS *src)
{
	if(!dest || !src) return;

	strcpy(dest->g_grptoken.g_vo,
		src->g_grptoken.g_vo);
	strcpy(dest->g_grptoken.g_subgroup,
		src->g_grptoken.g_subgroup);
	strcpy(dest->l_grptoken.g_mappedname,
		src->l_grptoken.g_mappedname);
	dest->l_grptoken.g_mappedgid =
		src->l_grptoken.g_mappedgid;
	strcpy(dest->l_grptoken.g_members,
		src->l_grptoken.g_members);
	dest->g_time = src->g_time;

}
*/
int ams_cache_gpwd_search(char *key_dn, char *key_vo, char *key_role, GPASSWD *gpwd)
{
	int i,ret = -1;
	int shm_id;
	key_t key;
	struct ams_cache_t *tab;
	GPASSWD *tgpwd = gpwd_new();

	if(!gpwd || !key_dn) return ret;
	DEBUG("CACHE: gpwd search enter ...\n");

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return ret;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,SHM_RDONLY);
	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++ ){
		if((*(tab+i)).type == 1) {
			amsclient_unpacketmsg(strdup((*(tab+i)).data), tgpwd, NULL, &ret);
			if(tgpwd->g_idtoken.g_dn[0] != '\0' 
					&&!strncmp(tgpwd->g_idtoken.g_dn, key_dn,sizeof(key_dn))
				&& tgpwd->g_idtoken.g_vo[0] != '\0'
                                        &&!strncmp(tgpwd->g_idtoken.g_vo, key_vo,sizeof(key_vo))
				&& tgpwd->g_idtoken.g_role[0] != '\0'
                                        &&!strncmp(tgpwd->g_idtoken.g_role, key_role,sizeof(key_role))
					){
				ret = 0;
			}
		}

	}

	if(!ret && tgpwd && gpwd) 
		memcpy(gpwd,tgpwd,sizeof(*gpwd));

	if(shmdt(tab)==-1)
		perror(" detach error ");

	if(tgpwd)
		free(tgpwd);

	return ret;
}


int ams_cache_ggrp_search(char *key_vo, char *key_role, GGROUPS *ggrp)
{
	int i,ret = -1;
	int shm_id;
	key_t key;
	struct ams_cache_t *tab;
	GGROUPS *tggrp = ggrp_new();

	if(!ggrp || !key_vo || !key_role) return ret;
	DEBUG("CACHE: ggrp search enter ... \n");

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return ret;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,SHM_RDONLY);
	for(i = 0; i < CACHE_HASH_MAXLEN && ret; i++ ){
		if((*(tab+i)).type == 2) {
			amsclient_unpacketmsg(strdup((*(tab+i)).data), NULL, tggrp, &ret);
			if(tggrp->g_grptoken.g_vo[0] != '\0' 
					&&!strncmp(tggrp->g_grptoken.g_vo, key_vo,sizeof(key_vo))
				&& tggrp->g_grptoken.g_subgroup[0] != '\0'
                                        &&!strncmp(tggrp->g_grptoken.g_subgroup, key_role,sizeof(key_role))
					){
				ret = 0;
			}
		}

	}

	if(!ret && tggrp && ggrp) 
		memcpy(ggrp,tggrp,sizeof(*ggrp));

	if(shmdt(tab)==-1)
		perror(" detach error ");

	if(tggrp)
		free(tggrp);

	return ret;
}



int ams_cache_uidgid_search(char *username, int user_uid, char *grpname, int user_gid, 
			GPASSWD *gpwd, GGROUPS *ggrp)
{
	int i,ret = -1, gfound = 1, ufound = 1;
	int shm_id;
	key_t key;
	struct ams_cache_t *tab;
	GPASSWD *tgpwd = gpwd_new();
	GGROUPS *tggrp = ggrp_new();

	if(!username && !grpname && !user_uid && !user_gid) 
		return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return ret;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,SHM_RDONLY);

	if(username || user_uid > 0) ufound = 0;
	if(grpname || user_gid > 0) gfound = 0;

	for(i = 0; i < CACHE_HASH_MAXLEN && (!ufound || !gfound); i++ ){

		if((*(tab+i)).type != 0 && strlen((*(tab+i)).data) > 2) {
			//DEBUG("CACHE: length of data '%d'",strlen((*(tab+i)).data));
			amsclient_unpacketmsg(strdup((*(tab+i)).data), tgpwd, tggrp, &ret);
			ret = -1;
		}
		else continue;	

		if((*(tab+i)).type == 1 && !ufound) {
			if(tgpwd->l_idtoken.g_mappedname[0] != '\0'&& username 
					&&!strncmp(tgpwd->l_idtoken.g_mappedname, username,sizeof(username)))
				ufound = 1;
			else if(tgpwd->l_idtoken.g_mappeduid != 0
                                        && tgpwd->l_idtoken.g_mappeduid == user_uid)
				ufound = 1;
			ret = !ufound;
		}else if((*(tab+i)).type == 2 && !gfound) {
			if(tggrp->l_grptoken.g_mappedname[0] != '\0' && grpname
					&&!strncmp(tggrp->l_grptoken.g_mappedname, grpname,sizeof(grpname)))
				gfound = 1;
			else if(tggrp->l_grptoken.g_mappedgid != 0
                                        && tggrp->l_grptoken.g_mappedgid == user_gid)
				gfound = 1;
			ret = ret && !gfound;
		}

	}

	if(!ret && tgpwd && gpwd) 
		memcpy(gpwd,tgpwd,sizeof(GPASSWD));

        if(!ret && tggrp && ggrp)
                memcpy(ggrp,tggrp,sizeof(GGROUPS));

	if(shmdt(tab)==-1)
		perror(" detach error ");

	if(tgpwd)
		free(tgpwd);
	if(tggrp)
		free(tggrp);

	return ret;
}

int ams_cache_whole_erase(char *key_dn, char *key_vo, char *key_role, int user_uid,  int user_gid)
{
	int i,ret = -1;
	int shm_id;
	key_t key;
	struct ams_cache_t *tab;
	GPASSWD *tgpwd = gpwd_new();
	GGROUPS *tggrp = ggrp_new();

	if(!(key_dn && key_vo && key_role) && user_uid < 0 && user_gid < 0) 
		return ret;

	key = ftok(cache_name,0);
	if(key==-1)
		perror("ftok error");

	shm_id=shmget(IPC_KEY_XOS,sizeof(struct ams_cache_t)*CACHE_HASH_MAXLEN,0666);
	if(shm_id==-1)
	{
		perror("shmget error");
		return ret;
	}
	tab=(struct ams_cache_t*)shmat(shm_id,NULL,0);


	for(i = 0; i < CACHE_HASH_MAXLEN; i++){

		if((*(tab+i)).type != 0)
			amsclient_unpacketmsg((*(tab+i)).data, tgpwd, tggrp, &ret);	

		if((*(tab+i)).type == 1) {
			if((key_dn && key_vo && key_role && tgpwd->g_idtoken.g_dn[0] != '\0' 
					&&!strncmp(tgpwd->g_idtoken.g_dn, key_dn,sizeof(key_dn))
				&& tgpwd->g_idtoken.g_vo[0] != '\0' 
                                        &&!strncmp(tgpwd->g_idtoken.g_vo, key_vo,sizeof(key_vo))
				&& tgpwd->g_idtoken.g_role[0] != '\0' 
                                        &&!strncmp(tgpwd->g_idtoken.g_role, key_role,sizeof(key_role))
				) || tgpwd->l_idtoken.g_mappeduid == user_uid) {
				(*(tab+i)).type = 0;
				(*(tab+i)).data[0] = '\0';
			}
	
		}else if((*(tab+i)).type == 2) {
			if((key_vo && key_role && tggrp->g_grptoken.g_vo[0] != '\0'
					&&!strncmp(tggrp->g_grptoken.g_vo, key_vo,sizeof(key_vo))
				&& tggrp->g_grptoken.g_subgroup[0] != '\0'
                                        &&!strncmp(tggrp->g_grptoken.g_subgroup, key_role,sizeof(key_role))
			) || tggrp->l_grptoken.g_mappedgid == user_gid){
				(*(tab+i)).type = 0;
				(*(tab+i)).data[0] = '\0';
			}	
		}

	}


	if(shmdt(tab)==-1)
		perror(" detach error ");

	if(tgpwd)
		free(tgpwd);
	if(tggrp)
		free(tggrp);

	return ret;
}

