/*
 *  Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 *  All rights reserved.
 *  
 *  This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 *  a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 *  The XtreemOS project has been developed with the financial support of the European
 *  Commission's IST program under contract #FP6-033576.
 *  
 *  Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software Foundation,
 *  either version 2 of the License, or (at your option) any later version.
 *  
 *  Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 *  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  */

/** @file  xos_lxc.c
 *  @brief The implementatio of cgroups interfaces
 *  
 *  
 *  */

#include<sys/stat.h>
#include<unistd.h>
#include <mntent.h>
#include <string.h>
#include <sys/mount.h>
#include "xos_utils.h"
#include "xos_lxc.h"

#define MTAB 		"/etc/mtab"
#define MNT_POINT	"/mnt/xos_cgrp"
#define MAXPATHLEN	512

static int get_cgroupfs_mount(const char *mtab, char *mnt, char *subsys)
{
        struct mntent *mntent;
        FILE *file = NULL;
        int err = -1;
	
	if(!mtab || !mnt) return err;

        file = setmntent(mtab, "r");
        if (!file) {
                hd_log_syserror("failed to open %s", mtab);
                goto out;
        }

        while ((mntent = getmntent(file))) {
                if (strcmp(mntent->mnt_type, "cgroup"))
                        continue;
		/*
		hd_log_info("[%s][%s][%s][%s][%d][%d]",
			mntent->mnt_fsname,mntent->mnt_dir,mntent->mnt_type,
			mntent->mnt_opts,mntent->mnt_freq,mntent->mnt_passno);
		*/
		if(subsys && strcmp(mntent->mnt_fsname,subsys))
			continue;

                strcpy(mnt, mntent->mnt_dir);
                err = 0;
                break;
        };

        //fclose(file);
	endmntent(file);
out:
        return err;
}

#if 1 

static int add_cgroupfs_mtab(char *mtab, char *cgroup, char *subsys)
{
	struct mntent mntent;
	FILE *file = NULL;
	char buf[MAXPATHLEN] = {'\0'};
	char *perm = "rw";

	if(!cgroup || !subsys) return -1;

	mntent.mnt_fsname = strdup(subsys);
	mntent.mnt_dir = strdup(cgroup);

	mntent.mnt_type = strdup("cgroup");

	snprintf(buf,MAXPATHLEN,"%s,%s",perm,subsys);
	mntent.mnt_opts = strdup(buf);

	mntent.mnt_freq = 0;
	mntent.mnt_passno = 0;

        file = setmntent(mtab, "a");
        if (!file) {
                hd_log_syserror("failed to open %s", mtab);
                goto out;
        }

	if(addmntent(file,&mntent)) {
		hd_log_syserror("failed to add mountpoint in mtab");
		return -1;
	}
	endmntent(file);
	return 0;
out:
	return -1;
}

static void del_cgroupfs_mtab(char *cgroup)
{
	static char **args;
	int nbargs;
	char *MOUNT_CMD = "/bin/umount";
	char cmd[40];
	bzero(cmd, 40);
#if ZZY
        args = malloc(3*sizeof(*args));
        if (!args) {
                hd_log_syserror("failed to allocate memory for '%s'", cgroup);
                return;
        }

        nbargs = 0;
        args[nbargs++] = strdup(MOUNT_CMD);
        args[nbargs++] = strdup(cgroup);
        args[nbargs++] = '\0';

	execvp(MOUNT_CMD,args);
#endif
	sprintf(cmd, "%s %s", MOUNT_CMD, cgroup);
	system(cmd);
}


#else
#define FUNC_MTAB_OPT(_OPT,_PERM)	\
static int #_OPT#_cgroupfs_mtab(char *mtab, char *cgroup, char *subsys) \
{ \
	struct mntent mntent;						\
	FILE *file = NULL;						\
	char buf[MAXPATHLEN] = {'\0'};					\
	char *perm = "rw";						\
									\
	if(!cgroup || !subsys) return -1;				\
									\
	mntent.mnt_fsname = strdup(subsys);				\
	mntent.mnt_dir = strdup(cgroup);				\
									\
	mntent.mnt_type = strdup("cgroup");				\
									\
	snprintf(buf,MAXPATHLEN,"%s,%s",perm,subsys);			\
	mntent.mnt_opts = strdup(buf);					\
									\
	mntent.mnt_freq = 0;						\
	mntent.mnt_passno = 0;						\
									\
        file = setmntent(mtab, "#_PERM#");					\
        if (!file) {							\
                hd_log_syserror("failed to open %s", mtab);		\
                goto out;						\
        }								\
									\
	if(#_OPT#mntent(file,&mntent)) {					\
		hd_log_syserror("failed to add mountpoint in mtab");	\
		return -1;						\
	}								\
	endmntent(file);						\
	return 0;							\
out:									\
	return -1;							\
}
FUNC_MTAB_OPT(add,a);
#undef FUNC_MTAB_OPT

#endif	

int hd_cgroupfs_mount(char *subsys)
{
	char cgroup[MAXPATHLEN] = {'\0'};
	struct stat file_stat;
	int ret = -1;

	if(!get_cgroupfs_mount(MTAB,cgroup,subsys)) {
		hd_log_info("cgroupfs is mounted");
		return 0;
	}
	/* mount the subsys */
	if(cgroup[0] == '\0') 
		snprintf(cgroup,MAXPATHLEN,"%s",MNT_POINT);
	
	if(stat(cgroup, &file_stat) && mkdir(cgroup,0755)) {
		hd_log_error("failed to mkdir '%s'",cgroup);
		return -1;
	}

	ret = mount(NULL,cgroup,"cgroup",0,subsys);
	if(!ret)
		ret = add_cgroupfs_mtab(MTAB,cgroup,subsys);

	return ret;
}

int hd_cgroupfs_umount(char *subsys)
{
	char cgroup[MAXPATHLEN];
	int ret = -1;

	if(get_cgroupfs_mount(MTAB,cgroup,subsys)) {
		hd_log_info("cgroupfs is not mounted");
		return -1;
	}

	/* umount the subsys */
#if 1
	if(cgroup[0] != '\0') {
		del_cgroupfs_mtab(cgroup);
		ret = 0;
	}
#else
	/* umount() not clear the infomation in /etc/mtab */
	if(cgroup[0] != '\0' && (ret = umount(cgroup)))
		hd_log_error("failed to umount '%s'",cgroup);
#endif
	return ret;
}

int hd_create_cgroup(char *name, char *subsys)
{
	char cgroup[MAXPATHLEN];
	char nsgroup[MAXPATHLEN];

	if(!name) return -1;

        if (get_cgroupfs_mount(MTAB, cgroup,subsys)) {
                hd_log_info("cgroupfs is not mounted");
                return -1;
        }
	snprintf(nsgroup, MAXPATHLEN, "%s/%s", cgroup, name);

	/* mkdir */
	if(mkdir(nsgroup,0755))	{
		hd_log_error("failed to create dir for '%s'",nsgroup);
		return -1;
	}
	return 0;
}

int hd_create_cgroup_withquota(char *name, char *subsys, char * quotas)
{
	char cgroup[MAXPATHLEN];
	char nsgroup[MAXPATHLEN];
	char * subsyss[4];
	char  buf[20];
	char  cmdbuf[200];
	int j = 0;
	unsigned long long quota[4];

	char * cp;
	sscanf(quotas, "%llu,%llu,%llu,%llu", &quota[0], &quota[1], &quota[2], &quota[3]);

	if(!name) return -1;

        if (get_cgroupfs_mount(MTAB, cgroup,subsys)) {
                hd_log_info("cgroupfs is not mounted");
                return -1;
        }
	snprintf(nsgroup, MAXPATHLEN, "%s/%s", cgroup, name);

	/* mkdir */
	if(mkdir(nsgroup,0755))	{
		hd_log_error("failed to create dir for '%s'",nsgroup);
		return -1;
	}
	cp = subsys;
	while ( *cp != 0){
	  int i = 0;
	  bzero(buf, 20);
	  while (*cp != ',' && *cp != 0){
	    buf[i++] = *cp;
	    cp++;
	  }
	  buf[i] = 0;
	  if (!strcmp("memory", buf)){
	    sprintf(cmdbuf, "echo %llu > %s/%s", quota[j], nsgroup, "memory.limit_in_bytes");
	    system(cmdbuf);
	  }else if(!strcmp("net", buf)){
	    sprintf(cmdbuf, "echo %llu > %s/%s", quota[j]/1000, nsgroup, "net.tot");
	    system(cmdbuf);
	  }
	  if (*cp == 0)
	    break;
	  else 
	    cp++;
	  j++;  
	}
	return 0;
}


int get_file_lines(char *path)
{
	FILE *file;
	int lines = 0;
	char buf[MAXPATHLEN] = {'\0'};

	if(!path) return -1;

	file = fopen(path,"r");
	if(!file) {
		hd_log_error("failed to open '%s'",path);
		return -1;
	}
	while(fgets(buf,MAXPATHLEN,file)) lines++;

	fclose(file);
	return lines;
}

int hd_destroy_cgroup(char *name, char *subsys)
{
	char cgroup[MAXPATHLEN];
	char nsgroup[MAXPATHLEN];
	char task[MAXPATHLEN];
	struct stat file_stat;

	if(!name) return -1;

	/* mounted pointer exist? */

        if (get_cgroupfs_mount(MTAB, cgroup,subsys)) {
                hd_log_info("cgroupfs is not mounted");
                return -1;
        }
	snprintf(nsgroup, MAXPATHLEN, "%s/%s", cgroup, name);

	if(stat(nsgroup, &file_stat)) {
		hd_log_info("not exist '%s'",nsgroup);
		return 0;
	}

	/* task has been cleared? */
	snprintf(task,MAXPATHLEN,"%s/tasks",nsgroup);
	if(get_file_lines(task)) {
		hd_log_info("'%s' is busy",name);
		return -1;
	}

	/* destroy */
	if(rmdir(nsgroup)) {
		hd_log_info("failed to remove '%s'",nsgroup);
		return -1;
	}
	return 0;
}

int hd_cgroup_addpid(char *subsys, char *name, int pid)
{
        char cgroup[MAXPATHLEN];
	char nsgroup[MAXPATHLEN];
        char task[MAXPATHLEN];
	struct stat file_stat;

	if(!name || pid <= 0) return -1;

	/* check mounting of subsys */
        if (get_cgroupfs_mount(MTAB, cgroup,subsys)) {
                hd_log_info("cgroupfs is not mounted");
                return -1;
        }

	/* check <name> exist */
	snprintf(nsgroup, MAXPATHLEN, "%s/%s/tasks", cgroup, name);
        if(stat(nsgroup, &file_stat)) {
                hd_log_error("not exist '%s'",nsgroup);
                return -1;
        }

	/* check pid is an existing process  */
	snprintf(task,MAXPATHLEN,"/proc/%d",pid);
        if(stat(task, &file_stat)) {
                hd_log_error("process '%d' does not exist",pid);
                return -1;
        }

	/* add pid into <name>/tasks */
	snprintf(task,MAXPATHLEN,"echo %d > %s",pid,nsgroup);
	system(task);

	return 0;
}
