/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/**
 * @file dn_mapper_rawdn.c
 * @brief RAWDN mapper module.
 *
 * This is a dummy mapper module that performs no mapping. The raw DN is passed
 * as username to the NSS related functions (i.e. getpwnam()).
 */
#include <string.h>		/* strlen, strcpy */

#include "dn_mapper_rawdn.h"

static int dn_mapper_is_map_rawdn(const char *name)
{
	return name[0] == '/';
}

static size_t dn_mapper_dn2name_rawdn(const char *dn, char *buf,
				      size_t bufsiz)
{
	size_t len;

	len = strlen(dn);
	if (bufsiz < len + 1) {
		return 0;
	}
	strcpy(buf, dn);
	return len + 1;
}

static size_t dn_mapper_name2dn_rawdn(const char *name, char *buf,
				      size_t bufsiz)
{
	size_t len;

	len = strlen(name);
	if (bufsiz < len + 1) {
		return 0;
	}
	strcpy(buf, name);
	return len + 1;
}

int _nss_xos_dn_mapper_rawdn_new(struct _nss_xos_dn_mapper_module *m)
{
	if (m == NULL)
		return 1;
	m->is_map = &dn_mapper_is_map_rawdn;
	m->dn2name = &dn_mapper_dn2name_rawdn;
	m->name2dn = &dn_mapper_name2dn_rawdn;
	return 0;
}
