/*
  * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <syslog.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fsuid.h>
#include <pwd.h>
#include <sys/syscall.h>
#include <sys/fsuid.h>
#include <creds_store.h>

#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#include <security/pam_modules.h>
#include <security/_pam_macros.h>

#include "xos.h"
#include "xos_utils.h"
#include "xos_ams.h"
#include "xos_cgrp.h"
#include "map_mech.h"
#include "xos_readconf.h"
#include "xos_sslutils.h"
#include "pam_plymgt.h"
#include "xos_interfaces.h"
#include "xos_policy_rule.h"
#include <sys/sem.h>

#define SEMKEYMOUNT (key_t)0x2002
#define IFLAGS IPC_EXCL

#define DATA_PROXY 		"PAM_XOS_PROXY"
#define DATA_DN			"PAM_XOS_DN"
#define DATA_ATTRS		"PAM_XOS_ATTRS"
#define DATA_USER_UID		"PAM_XOS_USER_UID"
#define DATA_USER_LNAME		"PAM_XOS_USER_LNAME"

#define EXT_ATTR_VO		"1.34.5.0.14.1"
#define EXT_ATTR_ROLE		"1.34.5.0.14.6"

#define	DEBUG_INFO			1
#define DEBUG_SHAREBUFFER		1
#define _NO_LOG

#ifndef _NO_LOG
#	define _pam_log(err, ...) __pam_log(pamh, err, __VA_ARGS__)
#else
#	define _pam_log(err, ...) ((void)0)
#endif /* _NO_LOG */

union semun
{
  int val;
  struct semid_ds *buf;
  unsigned short *ary;
};


void
P (int semid)
{
  struct sembuf sb;
  sb.sem_num = 0;
  sb.sem_op = -1;
  sb.sem_flg = 0;
  semop (semid, &sb, 1);
}

void
V (int semid)
{
  struct sembuf sb;
  sb.sem_num = 0;
  sb.sem_op = 1;
  sb.sem_flg = 0;
  semop (semid, &sb, 1);
}

#ifndef _NO_LOG
static void
__pam_log (pam_handle_t * pamh, int err, const char *format, ...)
{
  struct pam_message m[1];
  const struct pam_message *mp = m;
  struct pam_response *resp;
  struct pam_conv *conv = NULL;
  struct pam_conv **convp = &conv;
  char buf[1024];
  va_list args;
  int pam_err;

  va_start (args, format);
  /*pam_err = pam_get_item(pamh, PAM_CONV, (const void **)convp); */
  pam_err = pam_get_item (pamh, PAM_CONV, (const void **) convp);
  if (pam_err == PAM_SUCCESS && conv != NULL)
    {
      m[0].msg_style = PAM_TEXT_INFO;
      vsnprintf (buf, 1024, format, args);
      m[0].msg = buf;
      resp = NULL;
      pam_err = (*conv->conv) (1, &mp, &resp, conv->appdata_ptr);
      if (pam_err != PAM_SUCCESS)
	{
	  printf ("pam_err = %d\n", err);
	}
      if (resp != NULL)
	{
	  if (resp->resp != NULL)
	    free (resp->resp);
	  free (resp);
	}
    }
  va_end (args);
}
#endif /* _NO_LOG */


/*
 ******************************************************************************************
 */

int
xos_paraparse (int argc, const char **argv, ConfData * confdata)
{
  int ctrl = 0;
  const char *conffile = NULL;

  int i;
  for (i = 0; i < argc; i++)
    {
      if (!strcmp (argv[i], "-f"))
	{
	  conffile = argv[i++];
	}
      else if (!strcmp (argv[i], "--conf"))
	{
	  conffile = argv[i++];
	}
      else if (!strncmp (argv[i], "--conf=", 7))
	{
	  conffile = argv[i] + 7;
	}
      else if (!strcmp (argv[i], "-d"))
	{
	  ctrl |= USE_DEBUG;
	  DEBUG ("PAM:%s: Debug mode", __FUNCTION__);
	}
      else
	{
	  DEBUG ("PAM:%s: Unrecognized option: %s", __FUNCTION__, argv[i]);
	}
    }


  if (!conffile)
    conffile = PAM_CONF;



  initialize_confdata (confdata);
  if (xos_read_config_file (conffile, confdata, 0) != 1)
    {
      if (ctrl & USE_DEBUG)
	DEBUG
	  ("PAM:%s: xos_read_config_file fail! using default setup...",
	   __FUNCTION__);
      fill_default_confdata (confdata);
    }

  ctrl = 0;
  if (confdata->use_debug)
    ctrl |= USE_DEBUG;

  if (ctrl & USE_DEBUG)
    {
      DEBUG ("PAM: configure file is: %s", conffile);
    }

  if (confdata->mode & M_USESHAREBUFFER)
    ctrl |= USE_SHARE_BUFFER;

  if (confdata->mode & M_USEEXTERNALAMS)
    ctrl |= USE_EXTERNAL_AMS;
  if (confdata->mode & M_USEEXTERNALAES)
    ctrl |= USE_EXTERNAL_AES;


  return ctrl;


}





static void
cleanup (pam_handle_t * pamh, void *data, int err)
{
  free (data);
}


/********************************************************************************/

/* AUTH FUNCTIONS */

/* pam_sm_authenticate - authenticates the applicant, usually by requesting an
 * authentication token and comparing it with a value stored in a database or
 * obtained from an authentication server. */
PAM_EXTERN int
pam_sm_authenticate (pam_handle_t * pamh, int flags,
		     int argc, const char **argv)
{
#if DEBUG_INFO
  char *current_user = NULL;
  char **current_userp = &current_user;
  if (pam_get_user (pamh, (void *) current_userp, NULL) != PAM_SUCCESS)
    return PAM_SYSTEM_ERR;
  else
    DEBUG ("PAM:%s: uid = %d, user = [%s]", __FUNCTION__,
	   getuid (), current_user);
#endif


  struct pam_conv *conv = NULL;
  struct pam_conv **convp = &conv;
  struct pam_message msg;
  const struct pam_message *msgp;
  struct pam_response *resp;
  char *proxy_name = NULL;
  unsigned char *proxy_content = NULL;

  int ret, ctrl, retry, proxy_len;
  ConfData confdata;
  int is_get = 0;

  const struct creds_store_funcs *credstore_funcs = get_creds_store_funcs ();

  ret = PAM_AUTH_ERR;
  ctrl = xos_paraparse (argc, argv, &confdata);

  /* get proxy name */
  ret = pam_get_item (pamh, PAM_CONV, (const void **) convp);
  if (ret != PAM_SUCCESS)
    return (PAM_SYSTEM_ERR);
  msg.msg_style = PAM_PROMPT_ECHO_ON;
  msg.msg = "XOS's Proxy Name: ";
  msgp = &msg;

  for (retry = 0; retry < 3; ++retry)
    {
      resp = NULL;
      ret = (*conv->conv) (1, &msgp, &resp, conv->appdata_ptr);
      if (resp != NULL)
	{
	  if (ret == PAM_SUCCESS)
	    {
	      proxy_name = strdup (resp->resp);
	    }
	  else
	    free (resp->resp);
	  free (resp);
	}

      if (ret == PAM_SUCCESS)
	break;
    }

  /* Get certificate from (inputed) path */
  if (proxy_name && *proxy_name != '\0')
    {
      char *proxy_path = proxy_name;
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM:Fetch proxy from directory: %s", proxy_path);
      if (!strcmp ("credstore", proxy_name)
	  || xos_cert_buffromfile (proxy_path, &proxy_content,
				   &proxy_len) != XOS_SUCCESS)
	{
	  if (ctrl & USE_DEBUG)
	    DEBUG
	      ("PAM:Gan not get certificate [%s],try keyring...\n",
	       proxy_path);

	  ret = PAM_AUTH_ERR;
	  /* from credstore */
	  proxy_content =
	    (unsigned char *) credstore_funcs->retrieve_cred (NULL);
	  if (proxy_content)
	    is_get = 1;

	}
      else
	is_get = 1;		/*Success in geting certificate content */
    }

  /* get proxy content */
  if (!is_get)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM:current uid is: %d", getuid ());
      if (proxy_content)
	{
	  free (proxy_content);
	  proxy_content = NULL;
	}

      msg.msg_style = PAM_PROMPT_ECHO_OFF;
      msg.msg = "XOS's Proxy: ";

      resp = NULL;
      ret = (*conv->conv) (1, &msgp, &resp, conv->appdata_ptr);
      if (resp != NULL)
	{
	  if (ret == PAM_SUCCESS)
	    {
	      proxy_content = (unsigned char *) strdup (resp->resp);
	    }
	  else
	    free (resp->resp);
	  free (resp);
	}


    }


  /* Verify current user's proxy */
#if 0
  /* Bugs of proxy verify routines:
   * It can not find the expired certificate in
   * trusted chain. 
   * It is fixing now.
   * */
  if (xos_cert_verify
      (confdata.vo_cacertdir, (char *) proxy_content, 0) != XOS_SUCCESS)
    {
#else
  if (xos_cdacert_verify (proxy_content, confdata.vo_cacertdir,
			  confdata.vo_cacertf) != XOS_SUCCESS)
    {
#endif
      ret = PAM_AUTH_ERR;
      pam_sm_setcred (pamh, PAM_DELETE_CRED, argc, argv);
      goto err;
    }

  XOS_CREDS *creds = NULL;
  char *dn_buffer = NULL, *attr_buffer = NULL, *attr_vo_buffer = NULL;


  ret = PAM_SYSTEM_ERR;
  creds = xos_creds_new ();

  /* modifed for CDA certificate */
  if (xos_cert_loadfrombuffer
      (proxy_content, strlen ((char *) proxy_content), creds) == XOS_SUCCESS)
    {

      /* get DN from proxy */
      dn_buffer = (char *) malloc (1025);
      if (xos_cert_subject_get
	  (creds->certificate, dn_buffer, 1025) != XOS_SUCCESS)
	{
	  DEBUG ("PAM:Can not fetch DN from proxy!");
	  goto err;
	}
      else if (ctrl & USE_DEBUG)
	DEBUG ("PAM:DN = [%s]", dn_buffer);

#if DEBUG_SHAREBUFFER
      if (ctrl & USE_SHARE_BUFFER)
	{
	  /* Use share buffer to exchange data */
	  char dn_key[32];
	  char dn_env[1024];
	  if (amsclient_tmpinfosave (dn_buffer, dn_key))
	    {
	      DEBUG ("PAM:Can not save dn_buffer to tmp database!");
	      goto err;
	    }
	  sprintf (dn_env, "%s=%s", DATA_DN, dn_key);
	  if (pam_putenv (pamh, dn_env) != PAM_SUCCESS)
	    {
	      DEBUG ("PAM:Can not set dn_key to ENV variable!");
	      goto err;
	    }
	}
      else
#endif
      if (pam_set_data (pamh, DATA_DN, dn_buffer, cleanup) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:pam_set_data: Can not store data !");
	}

      /* get attributes from proxy */
      attr_buffer = (char *) malloc (1025);
      /* modified for CDA certificates, fetch attributes from extension */
      char ext_buf[1024] = { '\0' };
      char attr_vo_buf[1024] = { '\0' };
      char attr_role_buf[1024] = { '\0' };
      if (xos_cert_attributes_fetch_ext
	  (creds->certificate, ext_buf) != XOS_SUCCESS)
	{
	  DEBUG ("PAM:Can not get extension from CDA certificate\n");
	  /* Have not attributes ? */
	  strcpy (attr_buffer, "/VO=xtreemos/ROLE=other");
	}
      else
	if (xos_cert_attributes_fetch_attr
	    (ext_buf, EXT_ATTR_VO, attr_vo_buf) != XOS_SUCCESS)
	{
	  DEBUG ("PAM:Can not get VO name from extensions\n");
	  /* Have not vo name ? */
	  strcpy (attr_vo_buf, "xtreemos");
	}
      else
	if (xos_cert_attributes_fetch_attr
	    (ext_buf, EXT_ATTR_ROLE, attr_role_buf) != XOS_SUCCESS)
	{
	  DEBUG ("PAM:Can not get ROLE from extensions\n");
	  /* Have not vo name ? */
	  strcpy (attr_role_buf, "admin");
	}
      else
#if 0
	sprintf (attr_buffer, "/VO=%s/ROLE=%s", attr_vo_buf, attr_role_buf);
#else
	/* XtreemFS: GGID ussd to primary gid mapping */
	sprintf (attr_buffer, "/VO=%s/ROLE=%s", attr_vo_buf, attr_vo_buf);
#endif

      if (ctrl & USE_DEBUG)
	DEBUG ("PAM: Get Attrs = [%s]", attr_buffer);

      attr_vo_buffer = strdup (attr_vo_buf);

#if DEBUG_SHAREBUFFER
      if (ctrl & USE_SHARE_BUFFER)
	{
	  /* Use share buffer to exchange data */
	  char attr_key[32];
	  char attr_env[1024];
	  if (amsclient_tmpinfosave (attr_buffer, attr_key))
	    {
	      DEBUG ("PAM:Can not save attr_buffer to tmp database!");
	      goto err;
	    }
	  sprintf (attr_env, "%s=%s", DATA_ATTRS, attr_key);
	  if (pam_putenv (pamh, attr_env) != PAM_SUCCESS)
	    {
	      DEBUG ("PAM:Can not set attr_key to ENV variable!");
	      goto err;
	    }
	}
      else
#endif
      if (pam_set_data
	    (pamh, DATA_ATTRS, attr_buffer, cleanup) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:pam_set_data: Can not store data !");
	}

    }
#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      char proxy_key[32];
      char proxy_env[1024];
      if (amsclient_tmpinfosave ((char *) proxy_content, proxy_key))
	{
	  DEBUG ("PAM:Can not save proxy_buffer to tmp database!");
	  goto err;
	}
      sprintf (proxy_env, "%s=%s", DATA_PROXY, proxy_key);
      if (pam_putenv (pamh, proxy_env) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:Can not set proxy_key to ENV variable!");
	  goto err;
	}
    }
  else
#endif
  if (pam_set_data (pamh, DATA_PROXY, proxy_content, cleanup) != PAM_SUCCESS)
    {
      DEBUG ("PAM:pam_set_data: Can not store data !");
    }
  ret = PAM_SUCCESS;
err:
  return ret;
}


/* pam_sm_setcred - establishes account credentials such as user ID, group
 * membership and resource limits. */
PAM_EXTERN int pam_sm_setcred(pam_handle_t * pamh, int flags,
			      int argc, const char **argv)
{
	int ret = PAM_IGNORE;
	void *proxy_content = NULL;
	int ctrl = 0;
	ConfData confdata;
	const struct creds_store_funcs *credstore_funcs =
	    get_creds_store_funcs();

	DEBUG("PAM: %s: enter ...", __FUNCTION__);

	/* change uid and fsuid to PAM_USER uid ; uid is used to select
	   keyring and fsuid to assign owner to new entry */

	uid_t fs_uid = getuid(), current_uid = getuid();
	{
#if 0
		char *username;
		char **usernamep = &username;
		struct passwd *p;
		uid_t uid;
		pam_get_item(pamh, PAM_USER, (const void **) usernamep);

		p = getpwnam(username);
		if (p) {
			uid = p->pw_uid;
			if (uid != current_uid && geteuid() == 0) {
				setreuid(uid, -1);
				fs_uid = setfsuid(uid);
				credstore_funcs->
				    join_default_user_session();
			}
		} else
			DEBUG("%s: getpwnam() fail", __FUNCTION__);
#else

#if DEBUG_SHAREBUFFER
		uid_t user_id = -1;

		if (ctrl & USE_SHARE_BUFFER) {
			/* Use share buffer to exchange data */
			const char *uid_key = NULL;
			char *uid_data = NULL;
			uid_key = pam_getenv(pamh, DATA_USER_UID);
			DEBUG("PAM:get uid_key --> [%s]", uid_key);
			if (uid_key
			    && amsclient_tmpinfoget((char *) uid_key,
						    &uid_data) == 0) {
				user_id = atoi(uid_data);
			} else
				DEBUG
				    ("PAM:Can not get uid from tmp database!");

		} else
#endif
			if (pam_get_data
			    (pamh, DATA_USER_UID, (const void **)
			     &proxy_content) == PAM_SUCCESS) {

			user_id = *(int *) proxy_content;
		}

		if (user_id == -1)
			DEBUG("%s: fail to get uid for credstore()",
			      __FUNCTION__);


		if (user_id != -1 && user_id != current_uid
		    && geteuid() == 0) {
			setreuid(user_id, -1);
			fs_uid = setfsuid(user_id);
			credstore_funcs->join_default_user_session();
		}

#endif

	}

	ctrl = xos_paraparse(argc, argv, &confdata);
	if (flags & PAM_DELETE_CRED) {
		_pam_log(LOG_ERR, "%s: PAM_DELETE_CRED ...", __FUNCTION__);

		if (credstore_funcs->delete_cred()) {
			DEBUG("PAM: %s: Can not revoke the key",
			      __FUNCTION__);
		}

		/* get user id from other hook and remove from database */
#if DEBUG_SHAREBUFFER
		if (ctrl & USE_SHARE_BUFFER) {
			/* Use share buffer to exchange data */
			const char *uid_key = NULL;
			char *uid_data = NULL;
			uid_key = pam_getenv(pamh, DATA_USER_UID);
			DEBUG("PAM:get uid_key --> [%s]", uid_key);
			if (uid_key && !confdata.use_fixedidalloc
			    && amsclient_tmpinfoget((char *) uid_key,
						    &uid_data) == 0) {
				int user_id = atoi(uid_data);
				/* clear the mapped account in databases */
				if (amsclient_clearuidgid(user_id, 0))
					DEBUG
					    ("PAM:Can not delete user (%d)",
					     user_id);
			} else
				DEBUG
				    ("PAM:Can not get uid from tmp database!");

		} else
#endif
			if (pam_get_data
			    (pamh, DATA_USER_UID, (const void **)
			     &proxy_content) == PAM_SUCCESS) {

			int user_id = *(int *) proxy_content;

		        /* check the mount counter of usage*/
#if ZZY
		        int mount_cnt = amsclient_optmountnum("dec",user_id,-1);
		        DEBUG("PAM: mount counter is: %d",mount_cnt);

			/* clear the mapped account in databases */
			if(mount_cnt == 0 && !confdata.use_fixedidalloc)
			if (amsclient_clearuidgid(user_id, 0))
				DEBUG("PAM:Can not delete user (%d)",
				      user_id);
#endif
		}



	} else if (flags & PAM_REINITIALIZE_CRED) {
		_pam_log(LOG_ERR,
			 "PAM:%s: PAM_REINITIALIZE_CRED not supported",
			 __FUNCTION__);
#if DEBUG_SHAREBUFFER
		if (ctrl & USE_SHARE_BUFFER) {
			/* Use share buffer to exchange data */
			const char *proxy_key = NULL;
			char *proxy_data = NULL;
			proxy_key = pam_getenv(pamh, DATA_PROXY);
			DEBUG("PAM:get proxy_key --> [%s]", proxy_key);
			if (proxy_key
			    && amsclient_tmpinfoget((char *) proxy_key,
						    &proxy_data) == 0) {
				/* Store to credstore */
				if (credstore_funcs->
				    store_cred(proxy_data)) {
					DEBUG
					    ("PAM:Can not store proxy in keyring !");

				}

				ret = PAM_SUCCESS;
			} else
				DEBUG
				    ("PAM:Can not get proxy from tmp database!");

		} else
#endif
			if (pam_get_data(pamh, DATA_PROXY, (const void **)
					 &proxy_content) == PAM_SUCCESS) {
			DEBUG("PAM:%s: Get proxy [%d] : /n%s/n",
			      __FUNCTION__, strlen((char *) proxy_content),
			      (char *) proxy_content);

			/* Store to credstore */
			if (credstore_funcs->store_cred(proxy_content)) {
				DEBUG
				    ("PAM:Can not store proxy in keyring !");

			}

			ret = PAM_SUCCESS;

		} else
			DEBUG("PAM:%s: No data for keyring store!",
			      __FUNCTION__);

	} else if (flags & PAM_REFRESH_CRED) {
		_pam_log(LOG_ERR, "%s: PAM_REFRESH_CRED not supported",
			 __FUNCTION__);
	} else {		/* Default: PAM_ESTABLISH_CRED */
		/* TODO: Install keyring with certificate */
		_pam_log(LOG_NOTICE, "%s: installing keyrings...",
			 __FUNCTION__);
#if DEBUG_SHAREBUFFER
		if (ctrl & USE_SHARE_BUFFER) {
			/* Use share buffer to exchange data */
			const char *key = NULL;
			char *proxy_data = NULL;
			key = pam_getenv(pamh, DATA_PROXY);
			DEBUG("PAM:get proxy_key --> [%s]", key);
			if (key
			    && amsclient_tmpinfoget((char *) key,
						    &proxy_data) == 0) {
				/* Store to credstore */
				if (credstore_funcs->
				    store_cred(proxy_data)) {
					DEBUG
					    ("PAM:Can not store proxy in keyring !");

				}

				ret = PAM_SUCCESS;
			} else
				DEBUG
				    ("PAM:Can not get proxy from tmp database!");

		} else
#endif
			if (pam_get_data(pamh, DATA_PROXY, (const void **)
					 &proxy_content) == PAM_SUCCESS) {
			/* Store to credstore */
			if (credstore_funcs->store_cred(proxy_content)) {
				DEBUG
				    ("PAM:Can not store proxy in keyring !");
			}

			ret = PAM_SUCCESS;

		} else
			DEBUG("PAM:%s: No data for keyring store!",
			      __FUNCTION__);

	}
	// restore uid and fsuid
	setfsuid(fs_uid);
	setreuid(current_uid, -1);

	DEBUG("PAM: %s: finish ...", __FUNCTION__);

	return ret;


}

/* ACCOUNT FUNCTIONS */

/* pam_sm_acct_mgmt - verifies that the requested account is available. */
PAM_EXTERN int
pam_sm_acct_mgmt (pam_handle_t * pamh, int flags, int argc, const char **argv)
{

  int ret = PAM_USER_UNKNOWN;
#if DEBUG_INFO
  const char *requested_user;

  if (pam_get_user (pamh, (void *) (&requested_user), NULL) != PAM_SUCCESS)
    return PAM_SYSTEM_ERR;
  else
    DEBUG ("PAM:%s: uid = %d, user = [%s]", __FUNCTION__,
	   getuid (), requested_user);
#endif
  void *dn_buffer = NULL, *attr_buffer = NULL;
  void **dn_bufferp = &dn_buffer, **attr_bufferp = &attr_buffer;
  char *vop = NULL, *rolep = NULL;
  ConfData confdata;
  int ctrl = 0;

  ctrl = xos_paraparse (argc, argv, &confdata);

  DEBUG ("PAM: %s: enter ...", __FUNCTION__);

#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      const char *dn_key = NULL;
      //char *dn_data = NULL;
      dn_key = pam_getenv (pamh, DATA_DN);
      DEBUG ("PAM:get dn_key --> [%s]", dn_key);
      if (!dn_key
	  || amsclient_tmpinfoget ((char *) dn_key,
				   (char **) dn_bufferp) != 0)
	{
	  DEBUG ("PAM:Can not get dn from tmp database!");
	  goto err;
	}

    }
  else
#endif
  if (pam_get_data (pamh, DATA_DN, (const void **) &dn_buffer) != PAM_SUCCESS)
    {
      DEBUG ("PAM:Can not fetch dn from authentication function");
      goto err;
    }
#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      const char *attr_key = NULL;
      //void **attr_bufferp = &attr_buffer;
      attr_key = pam_getenv (pamh, DATA_ATTRS);
      DEBUG ("PAM:get attr_key --> [%s]", attr_key);
      if (!attr_key
	  || amsclient_tmpinfoget ((char *) attr_key,
				   (char **) attr_bufferp) != 0)
	{
	  DEBUG ("PAM:Can not get attributes from tmp database!");
	  goto err;
	}
      else
	{
	  char attr_key2[32];
	  char attr_env[1024];
	  if (amsclient_tmpinfosave (attr_buffer, attr_key2))
	    {
	      DEBUG ("PAM:Can not save attr_buffer to tmp database!");
	      goto err;
	    }
	  sprintf (attr_env, "%s=%s", DATA_ATTRS, attr_key2);
	  if (pam_putenv (pamh, attr_env) != PAM_SUCCESS)
	    {
	      DEBUG ("PAM:Can not set attr_key to ENV variable!");
	      goto err;
	    }
	}
    }
  else
#endif
  if (pam_get_data
	(pamh, DATA_ATTRS, (const void **) &attr_buffer) != PAM_SUCCESS)
    {
      DEBUG ("PAM:Can not fetch attributes from authentication function");
      goto err;
    }

  /* check whether user is in blacklst */
  struct subject_t rule_sub;
  int in_blacklist = 0;

  rule_sub.sub_type = XPLY_ST_ACT;
  strncpy (rule_sub.sub_content_1, dn_buffer, XPLY_MAXLEN);
  strncpy (rule_sub.sub_content_2, attr_buffer, XPLY_MAXLEN);

  if (amsclient_inblacklist (&rule_sub, &in_blacklist) == 0)
    {
      if (in_blacklist)
	{
	  fprintf (stderr, "PAM: login user is in blacklist !!!\n");
	  ret = PAM_AUTH_ERR;
	  goto err;
	}
    }


  if (ctrl & USE_DEBUG)
    /* attrs = /VO=xtreemos/ROLE=other */
    DEBUG ("PAM:Get information,dn->[%s],attrs->[%s]",
	   (char *) dn_buffer, (char *) attr_buffer);

  vop = strstr (attr_buffer, "/VO=");
  if (vop)
    vop += strlen ("/VO=");
  rolep = strstr (attr_buffer, "/ROLE=");
  if (rolep)
    {
      *rolep = '\0';
      rolep += strlen ("/ROLE=");
    }
  if (ctrl & USE_DEBUG)
    DEBUG ("PAM: parse attributes,vo = [%s], role = [%s]", vop, rolep);


  /* do mapping */
  GPASSWD *gpwd = NULL;
  gpwd = gpwd_new ();
  if (amsclient_usermapping (dn_buffer, vop, rolep, NULL, gpwd, NULL))
    {
      fprintf
	(stderr,
	 "PAM:fail in mapping connect !\n"
	 "\t* a)Please check whether AMS daemon is running correctly *\n"
	 "\t* b)Please check whether mapping rules are correct.      *\n"
	 "\t*     If not, try:                                       *\n"
	 "\t*       xos-policy-admin-am  -vo <vo> --force            *\n"
	 "\t*       xos-policy-admin-gm  -vo <vo> --force            *\n"
	 "\t* c)Please check whether setting rule is correct.        *\n"
	 "\t*     If not, try:                                       *\n"
	 "\t*       xos-policy-admin-set -uidmax <num> -uidmin <num> *\n"
	 "\t*                            -gidmax <num> -gidmin <num> *\n");
      goto err;
    }

  if (ctrl & USE_DEBUG)
    DEBUG ("PAM:finish mapping .... ");

  /* pass the mapped uid to other hooks */
  int *user_id = (int *) malloc (sizeof (int));
  *user_id = gpwd->l_idtoken.g_mappeduid;

#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      char uid_key[32];
      char uid_buffer[32];
      char uid_env[1024];
      sprintf (uid_buffer, "%d", *user_id);
      if (amsclient_tmpinfosave (uid_buffer, uid_key))
	{
	  DEBUG ("PAM:Can not save dn_buffer to tmp database!");
	  goto err;
	}
      sprintf (uid_env, "%s=%s", DATA_USER_UID, uid_key);
      if (pam_putenv (pamh, uid_env) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:Can not set uid_key to ENV variable!");
	  goto err;
	}
    }
  else
#endif
  if (pam_set_data
	(pamh, DATA_USER_UID, (void *) user_id, cleanup) != PAM_SUCCESS)
    {
      DEBUG ("PAM:pam_set_data: Can not store data !");
    }

  /* pass the mapped local name to other hooks */
  char *user_name =
    (char *) malloc (strlen (gpwd->l_idtoken.g_mappedname) + 1);
  strcpy (user_name, gpwd->l_idtoken.g_mappedname);

#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      char username_key[32];
      char username_env[1024];
      if (amsclient_tmpinfosave (user_name, username_key))
	{
	  DEBUG ("PAM:Can not save user_name to tmp database!");
	  goto err;
	}
      sprintf (username_env, "%s=%s", DATA_USER_LNAME, username_key);
      if (pam_putenv (pamh, username_env) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:Can not set username_key to ENV variable!");
	  goto err;
	}
    }
  else
#endif
  if (pam_set_data (pamh, DATA_USER_LNAME, user_name, cleanup) != PAM_SUCCESS)
    {
      DEBUG ("PAM:pam_set_data: Can not store data !");
    }
  pam_set_item (pamh, PAM_USER, user_name);

  ret = PAM_SUCCESS;

  //if (gpwd)
  //      free(gpwd);
  // provisional hack: putenv XOS_ENV because getpwnam requires it
  // to allow access to xtreemos info
  {
    struct passwd *p;
    char buff[1024];
    setenv ("XOS_ENV", "", 0);
    p = getpwnam (user_name);
    /* sometimes, the user_name is null, 
     * this may result in segfault.
     * */
    if (p == NULL)
      {
	DEBUG ("%s: getpwnam() fail", __FUNCTION__);
	goto err;
      }
    sprintf (buff, "XOS_ENV=%s", p->pw_gecos);
    putenv (buff);
  }
err:

  DEBUG ("PAM: %s: finish ...", __FUNCTION__);

  return ret;

}

/* SESSION FUNCTIONS */

/* pam_sm_open_session - performs tasks associated with session set-up: add an
 * entry in the utmp and wtmp databases, start an SSH agent, etc. */
PAM_EXTERN int
pam_sm_open_session (pam_handle_t * pamh, int flags,
		     int argc, const char **argv)
{
#if DEBUG_INFO
  char *current_user = NULL;
  if (pam_get_user (pamh, (void *) (&current_user), NULL) != PAM_SUCCESS)
    return PAM_SYSTEM_ERR;
  else
    DEBUG ("PAM:%s: uid = %d, gid = %d, user = [%s]", __FUNCTION__,
	   getuid (), getgid (), current_user);
#endif

  int ctrl;
  int ret = PAM_AUTH_ERR;
  ConfData confdata;
  struct pam_limit_s pl;
  char enforced_rules[8192];


  ctrl = xos_paraparse (argc, argv, &confdata);
  memset (&pl, 0, sizeof (pl));

  ret = xospm_paraparse (pamh, confdata.vo_acconf, &pl);
  if (ret != PAM_SUCCESS)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM: Policy setup fail!\n");
      return PAM_SYSTEM_ERR;
    }


  char *enforced_user = NULL;
  char **enforceduserp = &enforced_user;
  void *uid_buffer = NULL, *dn_buffer = NULL;
  void **dn_bufferp = &dn_buffer;
  int enforced_uid = 0;


#if DEBUG_SHAREBUFFER
  ret = PAM_SYSTEM_ERR;
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      const char *username_key = NULL;
      username_key = pam_getenv (pamh, DATA_USER_LNAME);
      DEBUG ("PAM:get username_key --> [%s]", username_key);
      if (!username_key
	  || amsclient_tmpinfoget ((char *) username_key, enforceduserp) != 0)
	{
	  DEBUG ("PAM:Can not get user_name from tmp database!");
	  goto err;
	}
      else
	{
	  char username_key2[32];
	  char username_env[1024];
	  if (amsclient_tmpinfosave (enforced_user, username_key2))
	    {
	      DEBUG ("PAM:Can not save user_name to tmp database!");
	      goto err;
	    }
	  sprintf (username_env, "%s=%s", DATA_USER_LNAME, username_key2);
	  if (pam_putenv (pamh, username_env) != PAM_SUCCESS)
	    {
	      DEBUG ("PAM:Can not set username_key to ENV variable!");
	      goto err;
	    }
	}

    }
  else
#endif
  if (pam_get_data
	(pamh, DATA_USER_LNAME, (const void **) enforceduserp) != PAM_SUCCESS)
    {
      DEBUG ("PAM:Can not get data from other hook");
      return PAM_USER_UNKNOWN;
    }

#if DEBUG_SHAREBUFFER
  if (ctrl & USE_SHARE_BUFFER)
    {
      /* Use share buffer to exchange data */
      const char *uid_key = NULL;
      char *uid_data = NULL;
      uid_key = pam_getenv (pamh, DATA_USER_UID);
      DEBUG ("PAM:get uid_key --> [%s]", uid_key);
      if (!uid_key || amsclient_tmpinfoget ((char *) uid_key, &uid_data) != 0)
	{
	  DEBUG ("PAM:Can not get user_name from tmp database!");
	  return PAM_USER_UNKNOWN;
	}
      else
	{
	  if (!uid_data)
	    return PAM_USER_UNKNOWN;
	  enforced_uid = strlen (uid_data);
	  char uid_key2[32];
	  char uid_env[1024];
	  if (amsclient_tmpinfosave (uid_data, uid_key2))
	    {
	      DEBUG ("PAM:Can not save dn_buffer to tmp database!");
	      goto err;
	    }
	  sprintf (uid_env, "%s=%s", DATA_USER_UID, uid_key2);
	  if (pam_putenv (pamh, uid_env) != PAM_SUCCESS)
	    {
	      DEBUG ("PAM:Can not set uid_key to ENV variable!");
	      goto err;
	    }
	}

    }
  else
#endif
  if (pam_get_data
	(pamh, DATA_USER_UID, (const void **) &uid_buffer) == PAM_SUCCESS)
    {

      enforced_uid = *(int *) uid_buffer;
    }
  else
    return PAM_USER_UNKNOWN;

  /* check the mount counter of usage */
  int mutex = semget (SEMKEYMOUNT, 1, IFLAGS);
  P (mutex);
  int mount_cnt = amsclient_optmountnum ("get", enforced_uid, -1);
  DEBUG ("PAM: mount counter is: %d", mount_cnt);

  DEBUG ("PAM: setup automount? %s", confdata.use_automount ? "yes" : "no");
  /* Auto-mount user's home volume */
  if (confdata.use_automount && mount_cnt == 0)
    {
      /* get user's GUID from authentication step */
      int isGetUID = 1;
      char mnt_command[128] = { '\0' }, *guidp = NULL;
#if DEBUG_SHAREBUFFER
      if (ctrl & USE_SHARE_BUFFER)
	{
	  /* Use share buffer to exchange data */
	  const char *dn_key = NULL;
	  dn_key = pam_getenv (pamh, DATA_DN);
	  DEBUG ("PAM:get dn_key --> [%s]", dn_key);
	  if (!dn_key
	      || amsclient_tmpinfoget ((char *) dn_key,
				       (char **) dn_bufferp) != 0)
	    {
	      DEBUG ("PAM:Can not get dn from tmp database!");
	      isGetUID = 0;
	    }

	}
      else
#endif
      if (pam_get_data
	    (pamh, DATA_DN, (const void **) &dn_buffer) != PAM_SUCCESS)
	{
	  DEBUG ("PAM:Can not fetch dn from authentication function");
	  isGetUID = 0;
	}
      if (isGetUID == 0)
	DEBUG ("Can not mount user's home directory, continue ...");
      else
	{
	  guidp = (char *) dn_buffer + strlen ("/cn=");
	  DEBUG ("Mount home volume for '%s'", guidp);
	  sprintf (mnt_command, "xtreemfs_mnt mount %s", guidp);
	  DEBUG ("Command: %s", mnt_command);
	  system (mnt_command);
	  /* change ownership */
	  //sprintf(mnt_command, "chown user_u%d:user_u%d /home/%s",enforced_uid,enforced_uid,guidp);
	  //sprintf(mnt_command, "/home/%s",guidp);
	  //DEBUG("change ownership: %s",mnt_command);
	  //system(mnt_command);
	  //chown(mnt_command,(int)enforced_uid,-1);
	  /* chage home directory */
	  sprintf (mnt_command, "/home/%s", guidp);
	  DEBUG ("Change home directory: %s", mnt_command);
	  //chroot(mnt_command);
	  //chdir(mnt_command);
	  if (amsclient_changeuserinfo (enforced_uid, mnt_command))
	    DEBUG
	      ("PAM: fail to call AMS server change home directory, continue...");
	  //chdir(mnt_command);

	}
    }

  mount_cnt = amsclient_optmountnum ("inc", enforced_uid, -1);
  V (mutex);
  /* prepare cgroup support */
  if (confdata.use_cgroups && confdata.cgroup_subsys)
    {
      DEBUG ("PAM: try to mount subsys '%s'", confdata.cgroup_subsys);
      if (hd_cgroup_mountfs (confdata.cgroup_subsys))
	DEBUG ("PAM: fail to mount subsys");
    }

  /* setup policy */
  ret = xospm_plmgtinit (&pl);
  if (ret != PAM_SUCCESS)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM:PAM: quota initiation fail!");
      return PAM_ABORT;
    }

  if (ctrl & USE_DEBUG)
    DEBUG ("PAM:enforced user is: [%s]", (char *) enforced_user);


  /* call the AMS for enfoced info */
  if (amsclient_enforcedrules (enforced_user, enforced_rules))
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("Warning: Can not get enforced info from AMS");
    }
  else
    if (xospm_getconfigure_frombuf
	(pamh, enforced_user, enforced_rules, ctrl, &pl) != PAM_SUCCESS)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("Warning: Can not load rules from buffer");
    }


  ret = xospm_getconfigure (pamh, (char *) enforced_user, ctrl, &pl);
  if (ret == PAM_IGNORE)
    {
      if (ctrl & USE_DEBUG)
	DEBUG
	  ("PAM:the configuration file/info has an applicable '<domain> -' entry");
      return PAM_SUCCESS;
    }
  if (ret != PAM_SUCCESS)
    {
      DEBUG ("PAM:error parsing the configuration file/info ");
      return ret;
    }


  ret = xospm_plysetup (pamh, enforced_user, enforced_uid, ctrl, &pl);
  if (ret & LOGIN_ERR)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM:Too many logins for '%s'.", enforced_user);
      return PAM_SESSION_ERR;
    }
  if (ret != LIMITED_OK)
    {
      if (ctrl & USE_DEBUG)
	DEBUG ("PAM:'%s' has been limited", enforced_user);
      return PAM_PERM_DENIED;
    }

  ret = PAM_SUCCESS;
err:
  return ret;

}
/* pam_sm_close_session - performs tasks associated with session tear-down: add
 * an entry in the utmp and wtmp databases, stop the SSH agent, etc. */
PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh, int flags,
				    int argc, const char **argv)
{

#if DEBUG_INFO
	char *current_user = NULL;
	if (pam_get_user(pamh, (void *) (&current_user), NULL) !=
	    PAM_SUCCESS)
		return PAM_SYSTEM_ERR;
	else
		DEBUG("PAM:%s: uid = %d, user = [%s]", __FUNCTION__,
		      getuid(), current_user);
#endif

	void *attr_buffer = NULL, *dn_buffer = NULL;
	void **attr_bufferp = &attr_buffer, **dn_bufferp = &dn_buffer;
	char *rolep = NULL;
	ConfData confdata;
	int ctrl = 0;
	int ret = PAM_SESSION_ERR;

	ctrl = xos_paraparse(argc, argv, &confdata);


	/* get local uid */
	void *uid_buffer = NULL;
	int enforced_uid = 0;
#if DEBUG_SHAREBUFFER
	if (ctrl & USE_SHARE_BUFFER) {
		/* Use share buffer to exchange data */
		const char *uid_key = NULL;
		char *uid_data = NULL;
		uid_key = pam_getenv(pamh, DATA_USER_UID);
		DEBUG("PAM:get uid_key --> [%s]", uid_key);
		if (!uid_key
		    || amsclient_tmpinfoget((char *) uid_key,
					    &uid_data) != 0) {
			DEBUG
			    ("PAM:Can not get user_name from tmp database!");
			return PAM_USER_UNKNOWN;
		} else {
			if (!uid_data)
				return PAM_USER_UNKNOWN;
			enforced_uid = strlen(uid_data);
			char uid_key2[32];
			char uid_env[1024];
			if (amsclient_tmpinfosave(uid_data, uid_key2)) {
				DEBUG
				    ("PAM:Can not save dn_buffer to tmp database!");
				goto err;
			}
			sprintf(uid_env, "%s=%s", DATA_USER_UID, uid_key2);
			if (pam_putenv(pamh, uid_env) != PAM_SUCCESS) {
				DEBUG
				    ("PAM:Can not set uid_key to ENV variable!");
				goto err;
			}
		}

	} else
#endif
		if (pam_get_data
		    (pamh, DATA_USER_UID, (const void **) &uid_buffer)
		    == PAM_SUCCESS) {

		enforced_uid = *(int *) uid_buffer;
	} else
		return PAM_USER_UNKNOWN;
	
	/* check the mount counter of usage*/

        int mutex = semget (SEMKEYMOUNT, 1, IFLAGS);
	P(mutex);
        int mount_cnt = amsclient_optmountnum("dec",enforced_uid,-1);
	DEBUG("PAM: mount counter is: %d",mount_cnt);


	/* Auto-umount user's home volume */
	if (confdata.use_automount && mount_cnt <= 0) {

		/* get user's GUID from authentication step */
		int isGetUID = 1;
		char mnt_command[128] = { '\0' };
#if DEBUG_SHAREBUFFER
		if (ctrl & USE_SHARE_BUFFER) {
			/* Use share buffer to exchange data */
			const char *dn_key = NULL;
			dn_key = pam_getenv(pamh, DATA_DN);
			DEBUG("PAM:get dn_key --> [%s]", dn_key);
			if (!dn_key
			    || amsclient_tmpinfoget((char *) dn_key,
						    (char **) dn_bufferp)
			    != 0) {
				DEBUG
				    ("PAM:Can not get dn from tmp database!");
				isGetUID = 0;
			}

		} else
#endif
			if (pam_get_data
			    (pamh, DATA_DN, (const void **) &dn_buffer)
			    != PAM_SUCCESS) {
			DEBUG
			    ("PAM:Can not fetch dn from authentication function");
			isGetUID = 0;
		}
		if (isGetUID == 0)
			DEBUG
			    ("Can not umount user's home directory, continue ...");
		else {
			DEBUG("Umount home volume for '%s'",
			      dn_buffer + strlen("/cn="));
			sprintf(mnt_command, "xtreemfs_mnt umount %s",
				(char *) dn_buffer + strlen("/cn="));
			DEBUG("Command: %s", mnt_command);
			system(mnt_command);
		}
		if (!confdata.use_fixedidalloc)
		  if (amsclient_clearuidgid (enforced_uid, 0))
		    DEBUG ("PAM:Can not delete user (%d)", enforced_uid);

	}
	V(mutex);
#if DEBUG_SHAREBUFFER
	if (ctrl & USE_SHARE_BUFFER) {
		/* Use share buffer to exchange data */
		const char *attr_key = NULL;
		//char *dn_data = NULL;
		attr_key = pam_getenv(pamh, DATA_ATTRS);
		DEBUG("PAM:get attr_key --> [%s]", attr_key);
		if (!attr_key
		    || amsclient_tmpinfoget((char *) attr_key,
					    (char **) attr_bufferp) != 0) {
			DEBUG
			    ("PAM:Can not get attributes from tmp database!");
			goto err;
		} else
			/* revoke the environment variable */
		if (pam_putenv(pamh, DATA_ATTRS) != PAM_SUCCESS) {
			DEBUG
			    ("PAM:Can not revoke attr_key from ENV variable!");
			goto err;
		}

	} else
#endif
	if (pam_get_data
		    (pamh, DATA_ATTRS,
			     (const void **) &attr_buffer) !=
		    PAM_SUCCESS) {
		DEBUG
		    ("PAM:Can not fetch attributes from authentication function");
		goto err;
	}

	if (ctrl & USE_DEBUG)
		/* attrs = /VO=xtreemos/ROLE=other */
		DEBUG("PAM:Get attrs->[%s]", (char *) attr_buffer);

	rolep = strstr(attr_buffer, "/ROLE=");
	if (rolep)
		rolep += strlen("/ROLE=");

	if (ctrl & USE_DEBUG)
		DEBUG("PAM:role = [%s]", rolep);

	char *enforced_user = NULL;
	char **enforceduserp = &enforced_user;
#if DEBUG_SHAREBUFFER
	if (ctrl & USE_SHARE_BUFFER) {
		/* Use share buffer to exchange data */
		const char *username_key = NULL;
		username_key = pam_getenv(pamh, DATA_USER_LNAME);
		DEBUG("PAM:get username_key --> [%s]", username_key);
		if (!username_key
		    || amsclient_tmpinfoget((char *) username_key,
					    enforceduserp) != 0) {
			DEBUG
			    ("PAM:Can not get user_name from tmp database!");
			goto err;
		} else {
			/* revoke the environment varibale */
			if (pam_putenv(pamh, DATA_USER_LNAME) !=
			    PAM_SUCCESS) {
				DEBUG
				    ("PAM:Can not set username_key to ENV variable!");
				goto err;
			}
		}

	} else
#endif
	if (pam_get_data
		    (pamh, DATA_USER_LNAME,
			     (const void **) enforceduserp) !=
		    PAM_SUCCESS) {
		DEBUG("PAM:Can not get local username from other hook");
		goto err;
	}

	if (ctrl & USE_DEBUG)
		DEBUG("PAM:finish policy clear ... ");

	/* prepare cgroup support */
	if(confdata.use_cgroups && confdata.cgroup_subsys) {
		DEBUG("PAM: try to umount subsys '%s'",confdata.cgroup_subsys);
		hd_cgroup_umountfs(confdata.cgroup_subsys);
	}

	ret = PAM_SUCCESS;


      err:
	return ret;
}



/* PASSWORD FUNCTIONS */

/* pam_sm_chauthtok - changes the authentication token, optionally verifying 
that it is sufficiently hard to guess, has not been used previously, etc. */
PAM_EXTERN int
pam_sm_chauthtok (pam_handle_t * pamh, int flags, int argc, const char **argv)
{
  _pam_log (LOG_ERR, "%s: password management group is unsupported",
	    __FUNCTION__);
  return PAM_SERVICE_ERR;

}
