/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/* the implementation of strategy 3 *
 * Get rid of the quota.conf
 */

#include <stdio.h>
#include <regex.h>
#include <locale.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include "xos_utils.h"
#include "xos_policy_rule.h"
#include "xos_policy_db.h"
#include "xos_policy_mgmt.h"
#include "xos_dbaux.h"
#include "xos_rquota.h"
#include "xos_reslimit.h"


#define	REGEX_STAR	"([0-9a-zA-Z]+)"
#define	REGEX_QUES	"."
#define	REGEX_MULSPACE	"\s+"




/** internal functions */
int regex_replace(char *regex_old, char **regex_new)
{
	char *p = NULL, *q = NULL, *t = NULL;
	char *tmp, *sub;
	char *des = "";
	int l;

	if (!regex_old)
		return -1;
	p = q = regex_old;

	while (*p) {
		if (*p == '*' || *p == '?') {
			if (*p == '*')
				sub = REGEX_STAR;
			else
				sub = REGEX_QUES;
			tmp =
			    (char *) calloc(sizeof(char),
					    strlen(des) + strlen(sub) + p -
					    q + 1);
			strcpy(tmp, des);
			//*p = '\0';
			//strcat(tmp,q);
			//strncpy(tmp, q, p-q);
			t = q;
			l = strlen(des);
			while (t != p) {
				tmp[l++] = *t;
				t++;
			}
			tmp[l] = '\0';
			strcat(tmp, sub);
			des = strdup(tmp);
			q = p + 1;
			free(tmp);
		}
		p++;
	}

	if (q) {
		tmp =
		    (char *) calloc(sizeof(char),
				    strlen(des) + strlen(q) + 1);
		strcpy(tmp, des);
		strcat(tmp, q);
		des = strdup(tmp);
		free(tmp);
	}
	*regex_new = strdup(des);

	return 0;
}




/** match the specific rule */
int xos_policy_match(enum XPLY_RT rtype,
		     struct subject_t *matched_sub,
		     struct rule_t *matched_rule)
{
	int ret = -1;
	regex_t re;
	char *matched_str = NULL;
	char *rule_1 = NULL, *rule_2 = NULL;

	if (!matched_sub || !matched_rule)
		return ret;

	/* 'rtype != XPLY_RT_NUL' should be noticed,
	   whether it is reational expression 
	 */
	if (matched_rule->r_type != rtype && rtype != XPLY_RT_NUL) {
		DEBUG("rule type unmatch!");
		return ret;
	}

	if (matched_sub->sub_type != matched_rule->r_subject.sub_type) {
		DEBUG("subject type unmatch !");
		return ret;
	}


	/* match the first content of struct subject_t */
	rule_1 = strdup(matched_rule->r_subject.sub_content_1);
	regex_replace(rule_1, &matched_str);
	if (matched_str[0] == '\0')
		return ret;

	if (regcomp(&re, matched_str, REG_EXTENDED) != 0) {
		DEBUG("Can not compile regular expression '%s'",
		      matched_str);
		return ret;
	}

	if (regexec(&re, matched_sub->sub_content_1, 0, NULL, 0) != 0) {
		DEBUG("Warning:unmatch '%s'", matched_sub->sub_content_1);
		return ret;
	}

	/* match the second content of struct subject_t */
	rule_2 = strdup(matched_rule->r_subject.sub_content_2);
	regex_replace(rule_2, &matched_str);
	if (matched_str[0] == '\0')
		return ret;

	if (regcomp(&re, matched_str, REG_EXTENDED) != 0) {
		DEBUG("Can not compile regular expression '%s'",
		      matched_str);
		return ret;
	}

	if (regexec(&re, matched_sub->sub_content_2, 0, NULL, 0) != 0) {
		DEBUG("unmatch '%s'", matched_sub->sub_content_2);
		return ret;
	}

	return 0;
}


/** find the matching rule */
int xos_policy_findrule(enum XPLY_RT rtype,
			struct subject_t *sub, struct rule_t **ret_rule,
			char **ret_handler)
{
	int i, ret;
	struct rule_t *xrule = NULL;
	char *xhandler = NULL;

	ret = -1;
	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("%s: Can not get rule !\n", __FUNCTION__);
			break;
		}
		if (!xrule)
			continue;


		/* match the rule */
		if (xrule->r_type != rtype)
			continue;
		else if (xos_policy_match(rtype, sub, xrule) == 0) {
			ret = 0;
			break;
		}
	}

	if (ret == 0) {
		if (ret_rule)
			*ret_rule = xrule;
		if (ret_handler)
			*ret_handler = xhandler;
	}

	return ret;
}

int xos_policy_bindRU_grprule_pwd(GPASSWD * gpwd, struct rule_t *xrule,
				  char *ru_rules);
int xos_policy_bindRU_actrule_pwd(GPASSWD * gpwd, struct rule_t *xrule,
				  char *ru_rules);
int xos_policy_bindRU_grprule_grp(GGROUPS * ggrp, struct rule_t *xrule,
				  char *ru_rules);



/* stategy 2*/
int xos_policy_pwdbinding(GPASSWD * gpwd, char *ru_rules)
{
	struct subject_t sub;
	struct object_t obj;
	struct rule_t *xrule = NULL;
	int i, ret = -1, fin_ret = -1;
	char *xhandler = NULL;

	if (!gpwd)
		return fin_ret;

	sub.sub_type = XPLY_ST_ACT;
	strcpy(sub.sub_content_1, gpwd->g_idtoken.g_dn);
	strcpy(sub.sub_content_2, gpwd->g_idtoken.g_vo);

	obj.obj_type = XPLY_OT_ACT;
	strcpy(obj.obj_content.objc_object, gpwd->l_idtoken.g_mappedname);


	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("%s: Can not get rule !\n", __FUNCTION__);
			continue;
		}
		if (!xrule)
			continue;


		/* if it doesn't match the rule, search next */
		if (!
		    (xrule->r_type == XPLY_RT_RU
		     && xrule->r_subject.sub_type == XPLY_ST_GRP)) {
			if (xos_policy_match(XPLY_RT_NUL, &sub, xrule) !=
			    0)
				continue;
		}

		switch (xrule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			if (xrule->r_subject.sub_type == XPLY_ST_ACT)
				ret =
				    xos_policy_bindRU_actrule_pwd(gpwd,
								  xrule,
								  ru_rules);
			else
				ret =
				    xos_policy_bindRU_grprule_pwd(gpwd,
								  xrule,
								  ru_rules);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Warning:Can not deal with the rule");
		}



		if (ret) {
			DEBUG
			    ("Warning: Can not apply rule with handler '%s'",
			     xhandler);
		} else if (fin_ret == -1)
			fin_ret = 0;

	}


	return fin_ret;
}



/** local policy binding for GGROUPS */
/* stategy 1*/
int xos_policy_grpbinding(GGROUPS * ggrp, char *ru_rules)
{
	struct subject_t sub;
	struct object_t obj;
	struct rule_t *xrule = NULL;
	int i, ret = -1, fin_ret = -1;
	char *xhandler = NULL;

	if (!ggrp || !ru_rules)
		return fin_ret;

	sub.sub_type = XPLY_ST_GRP;
	strcpy(sub.sub_content_1, ggrp->g_grptoken.g_vo);
	strcpy(sub.sub_content_2, ggrp->g_grptoken.g_subgroup);

	obj.obj_type = XPLY_OT_GRP;
	strcpy(obj.obj_content.objc_object, ggrp->l_grptoken.g_mappedname);

	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("%s: Can not get rule !\n", __FUNCTION__);
			continue;
		}
		if (!xrule)
			continue;


		/* if it doesn't match the rule, search next */
		if (xos_policy_match(XPLY_RT_NUL, &sub, xrule) != 0)
			continue;

		switch (xrule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			ret =
			    xos_policy_bindRU_grprule_grp(ggrp, xrule,
							  ru_rules);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Can not deal with the rule \n");
		}

		if (ret) {
			DEBUG
			    ("Warning: Can not apply rule with handler '%s'\n",
			     xhandler);
		} else if (fin_ret == -1)
			fin_ret = 0;

	}

	return fin_ret;
}

				   /** AC binding *//* stategy 1 */
int xos_policy_bindAC(struct subject_t *sub, struct object_t *obj,
		      struct rule_t *rule)
{
	DEBUG("No support binding AC rules\n");
	return -1;
}

#if 0
				   /** RU binding *//* stategy 1 */
int xos_policy_bindRU(struct subject_t *sub, struct object_t *obj,
		      struct rule_t *rule)
{
	ResQuota rquota;
	int quota_val = 0;
	int ret = -1;

	xos_resquota_init(&rquota);

	if (sub->sub_type == XPLY_ST_ACT && obj->obj_type == XPLY_OT_ACT)
		rquota.f_group = 0;
	else if (sub->sub_type == XPLY_ST_GRP
		 && obj->obj_type == XPLY_OT_GRP)
		rquota.f_group = 1;
	else
		return -1;

	rquota.rname = strdup(obj->obj_content.objc_object);

	quota_val = rule->r_content.rc_quota.quo_val;
	if (rule->r_content.rc_quota.quo_type == XPLY_QT_REL)
		/* Currently, we can not support the relative value,
		   so adjust to absolute value.
		 */
		quota_val *= 100;

	switch (rule->r_object.obj_content.objc_restype) {
	case XPLY_REST_CPU:
		rquota.rcpu = quota_val;
		rquota.f_cpu = 1;
		break;
	case XPLY_REST_MEM:
		rquota.rmem = quota_val;
		rquota.f_mem = 1;
		break;
	case XPLY_REST_DISK:
		rquota.rdisk = quota_val;
		rquota.f_disk = 1;
		break;
	case XPLY_REST_NETWORK:
		DEBUG("%s: No support now !\n", __FUNCTION__);
		goto err;
		break;
	default:
		DEBUG("%s: No support quota item '%d' !\n",
		      __FUNCTION__,
		      rule->r_object.obj_content.objc_restype);
	}

	/* Store in quota.conf. */
	if (xos_resquota_modify(&rquota, 7)) {
		DEBUG("%s: Can not modify quota configuration file\n",
		      __FUNCTION__);
		goto err;
	}

	ret = 0;
      err:
	return ret;

}
#endif


				   /** AP binding *//* stategy 1 */
int xos_policy_bindAP(struct subject_t *sub, struct object_t *obj,
		      struct rule_t *rule)
{
	DEBUG("No support binding AP rules\n");
	return -1;
}


/** auxiliary functions */

/* group_id[0] is stored in the uid for checking 
      * group_id[1] is stored in counter of current number
*//* stategy 2 */

int groups_with_account(GGROUPS * ggrp, int group_id[])
{
	char id_str[XPM_MAXLEN];
	char *p = NULL, *pp = NULL;
	int len = 0;

	if (!ggrp || group_id[0] < 1)
		return -1;
	sprintf(id_str, "%d", group_id[0]);

	len = strlen(id_str);
	if ((p = strstr(ggrp->l_grptoken.g_members, id_str))) {
		pp = ggrp->l_grptoken.g_members;
		if ((pp == p || *(p - 1) == ',')
		    && (*(p + len) == ',' || *(p + len) == '\0')) {
			group_id[group_id[1] + 2] =
			    ggrp->l_grptoken.g_mappedgid;
			if (group_id[1] < XPM_MAXLEN - 2)
				group_id[1] += 1;
		}
	}

	return 0;


}

/** RU binding - auxiliary function */
/* stategy 2*/
int xos_policy_bindRU_grprule_pwd(GPASSWD * gpwd, struct rule_t *xrule,
				  char *ru_rules)
{
	int group_id[XPM_MAXLEN];
	int ret = -1, i, g_id;
	GGROUPS *ggrp;
	struct subject_t sub;
	char *limit_type, *limit_item;
	char limit_value[XPM_MAXLEN];

	if (!gpwd || !xrule || !ru_rules)
		return ret;

	memset(group_id, 0, sizeof(group_id));
	group_id[0] = gpwd->l_idtoken.g_mappeduid;
	ggrp = ggrp_new();
	limit_item = NULL;
	limit_type = strdup("hard");	/* currently, 'hard' setting */

	/* get the group holding the member of current gpwd */
	if (bdb_operatedb_grp
	    (group_id, NULL, (OPTGRPFUNC) groups_with_account)) {
		DEBUG
		    ("Can not get all the group holding specific account\n");
		return ret;
	}

	for (i = 2; (i < group_id[1] + 2) && (i < XPM_MAXLEN); i++) {
		g_id = group_id[i];
		if (bdb_getdbgid_grp(g_id, ggrp)) {
			DEBUG("Can not get ggrp by gid\n");
			continue;
		}

		sub.sub_type = XPLY_ST_GRP;
		strncpy(sub.sub_content_1, ggrp->g_grptoken.g_vo,
			XPLY_MAXLEN);
		strncpy(sub.sub_content_2, ggrp->g_grptoken.g_subgroup,
			XPLY_MAXLEN);


		if (xos_policy_match(XPLY_RT_RU, &sub, xrule) != 0)
			continue;

		switch (xrule->r_object.obj_content.objc_restype) {
		case XPLY_REST_CPU:
			limit_item = strdup("cpu");
			break;
		case XPLY_REST_MEM:
			limit_item = strdup("mem");
			break;
		case XPLY_REST_DISK:
			limit_item = strdup("disk");
			break;
		case XPLY_REST_NPROC:
			limit_item = strdup("nproc");
			break;
		case XPLY_REST_NOFILE:
			limit_item = strdup("nofile");
			break;
		case XPLY_REST_NETWORK:
			DEBUG("%s: No support now !\n", __FUNCTION__);
			break;
		default:
			DEBUG("%s: No support quota item '%d' !\n",
			      __FUNCTION__,
			      xrule->r_object.obj_content.objc_restype);
		}

		/* Currently, we can not support the relative value,
		   so adjust to absolute value.
		 */
		if (xrule->r_content.rc_quota.quo_type == XPLY_QT_REL)
			sprintf(limit_value, "%d",
				(int) (xrule->r_content.rc_quota.quo_val *
				       100));
		else
			sprintf(limit_value, "%d",
				(int) (xrule->r_content.rc_quota.quo_val));

		//xos_reslimit_quota(LIMITS_DEF_GROUP,limit_type,limit_item,limit_value,xlimit);
		/* one is: */
		sprintf(ru_rules, "%s@%s %s %s %s\n", ru_rules,
			ggrp->l_grptoken.g_mappedname, limit_type,
			limit_item, limit_value);
		/* another is:
		   sprintf(ru_rules,"%s%s %s %s %s\n",ru_rules,gpwd->l_idtoken.g_mappedname,
		   limit_type,limit_item,limit_value); 
		 */

		if (limit_item) {
			free(limit_item);
			limit_item = NULL;
		}
	}

	return 0;
}

/* stategy 2*/
int xos_policy_bindRU_actrule_pwd(GPASSWD * gpwd, struct rule_t *xrule,
				  char *ru_rules)
{
	char *limit_type, *limit_item;
	char limit_value[XPM_MAXLEN];
	int ret = -1;

	if (!xrule || !gpwd || !ru_rules)
		return ret;

	limit_item = NULL;
	limit_type = strdup("hard");	/* currently, 'hard' setting */


	switch (xrule->r_object.obj_content.objc_restype) {
	case XPLY_REST_CPU:
		limit_item = strdup("cpu");
		break;
	case XPLY_REST_MEM:
		limit_item = strdup("mem");
		break;
	case XPLY_REST_DISK:
		limit_item = strdup("disk");
		break;
	case XPLY_REST_NPROC:
		limit_item = strdup("nproc");
		break;
	case XPLY_REST_NOFILE:
		limit_item = strdup("nofile");
		break;
	case XPLY_REST_NETWORK:
		DEBUG("No support now !\n");
		break;
	default:
		DEBUG("No support quota item '%d' !\n",
		      xrule->r_object.obj_content.objc_restype);
	}

	/* Currently, we can not support the relative value,
	   so adjust to absolute value.
	 */
	if (xrule->r_content.rc_quota.quo_type == XPLY_QT_REL)
		sprintf(limit_value, "%d",
			(int) (xrule->r_content.rc_quota.quo_val * 100));
	else
		sprintf(limit_value, "%d",
			(int) (xrule->r_content.rc_quota.quo_val));

	sprintf(ru_rules, "%s%s %s %s %s\n", ru_rules,
		gpwd->l_idtoken.g_mappedname, limit_type, limit_item,
		limit_value);

	if (limit_item) {
		free(limit_item);
		limit_item = NULL;
	}

	return 0;
}


/* stategy 2*/
int xos_policy_bindRU_grprule_grp(GGROUPS * ggrp, struct rule_t *xrule,
				  char *ru_rules)
{
	char *limit_type, *limit_item;
	char limit_value[XPM_MAXLEN];
	int ret = -1;		//member_uid = -1;
	//char *p = NULL, *pp = NULL;
	//char *istr = NULL;
	//int tno = 0;

	if (!ggrp || !xrule)
		return ret;
	if (ggrp->l_grptoken.g_members[0] == '\0')
		return 0;

	limit_item = NULL;
	limit_type = strdup("hard");	/* currently, 'hard' setting */


	switch (xrule->r_object.obj_content.objc_restype) {
	case XPLY_REST_CPU:
		limit_item = strdup("cpu");
		break;
	case XPLY_REST_MEM:
		limit_item = strdup("mem");
		break;
	case XPLY_REST_DISK:
		limit_item = strdup("disk");
		break;
	case XPLY_REST_NPROC:
		limit_item = strdup("nproc");
		break;
	case XPLY_REST_NOFILE:
		limit_item = strdup("nofile");
		break;
	case XPLY_REST_NETWORK:
		DEBUG("%s: No support now !\n", __FUNCTION__);
		break;
	default:
		DEBUG("%s: No support quota item '%d' !\n",
		      __FUNCTION__,
		      xrule->r_object.obj_content.objc_restype);
	}

	/* Currently, we can not support the relative value,
	   so adjust to absolute value.
	 */
	if (xrule->r_content.rc_quota.quo_type == XPLY_QT_REL)
		sprintf(limit_value, "%d",
			(int) (xrule->r_content.rc_quota.quo_val * 100));
	else
		sprintf(limit_value, "%d",
			(int) (xrule->r_content.rc_quota.quo_val));

	sprintf(ru_rules, "%s@%s %s %s %s\n", ru_rules,
		ggrp->l_grptoken.g_mappedname, limit_type, limit_item,
		limit_value);

	return 0;
}




/** local policy binding for GPASSWD */
/* stategy 1*/
int xos_policy_pwdunbinding(GPASSWD * gpwd)
{
	struct subject_t sub;
	struct object_t obj;
	struct rule_t *xrule = NULL;
	int i, ret = -1, fin_ret = -1;
	char *xhandler = NULL;

	sub.sub_type = XPLY_ST_ACT;
	strcpy(sub.sub_content_1, gpwd->g_idtoken.g_dn);
	strcpy(sub.sub_content_2, gpwd->g_idtoken.g_vo);

	obj.obj_type = XPLY_OT_ACT;
	strcpy(obj.obj_content.objc_object, gpwd->l_idtoken.g_mappedname);


	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("%s: Can not get rule !\n", __FUNCTION__);
			continue;
		}
		if (!xrule)
			continue;


		/* if it doesn't match the rule, search next */
		if (xos_policy_match(XPLY_RT_NUL, &sub, xrule) != 0)
			continue;

		switch (xrule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			/* Nothing to do in strategy 3 */
			//ret = xos_policy_unbindRU(&sub,&obj,xrule);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Can not deal with the rule \n");
		}

		if (ret) {
			DEBUG
			    ("Warning: Can not apply rule with handler '%s'\n",
			     xhandler);
		} else if (fin_ret == -1)
			fin_ret = 0;

	}

	return fin_ret;
}


/** local policy binding for GGROUPS */
/* stategy 1*/
int xos_policy_grpunbinding(GGROUPS * ggrp)
{
	struct subject_t sub;
	struct object_t obj;
	struct rule_t *xrule = NULL;
	int i, ret = -1, fin_ret = -1;
	char *xhandler = NULL;

	sub.sub_type = XPLY_ST_GRP;
	strcpy(sub.sub_content_1, ggrp->g_grptoken.g_vo);
	strcpy(sub.sub_content_2, ggrp->g_grptoken.g_subgroup);

	obj.obj_type = XPLY_OT_GRP;
	strcpy(obj.obj_content.objc_object, ggrp->l_grptoken.g_mappedname);

	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("%s: Can not get rule !\n", __FUNCTION__);
			continue;
		}
		if (!xrule)
			continue;


		/* if it doesn't match the rule, search next */
		if (xos_policy_match(XPLY_RT_NUL, &sub, xrule) != 0)
			continue;

		switch (xrule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			/* Nothing to do in strategy 3 */
			//ret = xos_policy_unbindRU(&sub,&obj,xrule);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Can not deal with the rule \n");
		}

		if (ret) {
			DEBUG
			    ("Warning: Can not apply rule with handler '%s'\n",
			     xhandler);
		} else if (fin_ret == -1)
			fin_ret = 0;

	}

	return fin_ret;
}


/** AC unbinding */
/* stategy 1*/
int xos_policy_unbindAC(struct subject_t *sub, struct object_t *obj,
			struct rule_t *rule)
{
	DEBUG("No support unbinding AC rules\n");
	return -1;
}

#if 0
/** RU unbinding */
/* stategy 1*/
int xos_policy_unbindRU(struct subject_t *sub, struct object_t *obj,
			struct rule_t *rule)
{
	ResQuota rquota;
	int quota_val = 0;
	int ret = -1;

	xos_resquota_init(&rquota);

	if (sub->sub_type == XPLY_ST_ACT && obj->obj_type == XPLY_OT_ACT)
		rquota.f_group = 0;
	else if (sub->sub_type == XPLY_ST_GRP
		 && obj->obj_type == XPLY_OT_GRP)
		rquota.f_group = 1;
	else
		return -1;

	rquota.rname = strdup(obj->obj_content.objc_object);

	quota_val = rule->r_content.rc_quota.quo_val;
	if (rule->r_content.rc_quota.quo_type == XPLY_QT_REL)
		/* Currently, we can not support the relative value,
		   so adjust to absolute value.
		 */
		quota_val *= 100;

	switch (rule->r_object.obj_content.objc_restype) {
	case XPLY_REST_CPU:
		rquota.rcpu = quota_val;
		rquota.f_cpu = 1;
		break;
	case XPLY_REST_MEM:
		rquota.rmem = quota_val;
		rquota.f_mem = 1;
		break;
	case XPLY_REST_DISK:
		rquota.rdisk = quota_val;
		rquota.f_disk = 1;
		break;
	case XPLY_REST_NETWORK:
		DEBUG("%s: No support now !\n", __FUNCTION__);
		goto err;
		break;
	default:
		DEBUG("%s: No support quota item '%d' !\n",
		      __FUNCTION__,
		      rule->r_object.obj_content.objc_restype);
	}

	/* Store in quota.conf. */
	if (xos_resquota_clear(&rquota)) {
		DEBUG("%s: Can not clear quota\n", __FUNCTION__);
		goto err;
	}

	ret = 0;
      err:
	return ret;

}
#endif


/** AP unbinding */
/* stategy 1*/
int xos_policy_unbindAP(struct subject_t *sub, struct object_t *obj,
			struct rule_t *rule)
{
	DEBUG("No support unbinding AP rule\n");
	return -1;
}

#if 0
/* stategy 2*/
int xos_policy_unbindRU_pwd(GPASSWD * gpwd, XLIMIT * xlimit)
{
	int ret = -1;

	if (!gpwd || !xlimit)
		return ret;

	if (xos_reslimit_setup(gpwd->l_idtoken.g_mappedname,
			       gpwd->l_idtoken.g_mappeduid, xlimit)) {
		DEBUG("fail to setup resource quota\n");
		return -1;
	}

	return 0;
}
#endif

/** Update runtime data in GPASSWD database */
/* stategy 1*/
int xos_policy_pwdupdate(GPASSWD * gpwd, struct rule_t *updated_rule)
{
	struct subject_t sub;
	struct object_t obj;
	int ret = 0;

	if (!gpwd || !updated_rule)
		return -1;

	sub.sub_type = XPLY_ST_ACT;
	strcpy(sub.sub_content_1, gpwd->g_idtoken.g_dn);
	strcpy(sub.sub_content_2, gpwd->g_idtoken.g_vo);

	obj.obj_type = XPLY_OT_ACT;
	strcpy(obj.obj_content.objc_object, gpwd->l_idtoken.g_mappedname);

	if ((updated_rule->r_type == XPLY_RT_RU
	     && updated_rule->r_subject.sub_type == XPLY_ST_GRP)
	    || (xos_policy_match(XPLY_RT_NUL, &sub, updated_rule) == 0)) {
		switch (updated_rule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			//if(xrule->r_subject.sub_type == XPLY_ST_ACT)
			//ret = xos_policy_bindRU_actrule_pwd(gpwd,xrule,ru_rules);
			//else ret = xos_policy_bindRU_grprule_pwd(gpwd,xrule,ru_rules);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Can not deal with the rule \n");
		}
		if (ret) {
			DEBUG("Warning: Can not apply rule to GPASSWD\n");
		}
	}

	return ret;
}

/* stategy 1*/
/** Update runtime data in GGROUPS database */
int xos_policy_grpupdate(GGROUPS * ggrp, struct rule_t *updated_rule)
{
	struct subject_t sub;
	struct object_t obj;
	int ret = 0;

	if (!ggrp || !updated_rule)
		return -1;

	sub.sub_type = XPLY_ST_GRP;
	strcpy(sub.sub_content_1, ggrp->g_grptoken.g_vo);
	strcpy(sub.sub_content_2, ggrp->g_grptoken.g_subgroup);

	obj.obj_type = XPLY_OT_GRP;
	strcpy(obj.obj_content.objc_object, ggrp->l_grptoken.g_mappedname);

	if (xos_policy_match(XPLY_RT_NUL, &sub, updated_rule) == 0) {
		switch (updated_rule->r_type) {
		case XPLY_RT_AC:
			break;
		case XPLY_RT_RU:
			//ret = xos_policy_bindRU(&sub,&obj,updated_rule);
			break;
		case XPLY_RT_AP:
			break;
		default:
			DEBUG("Can not deal with the rule \n");
		}

		if (ret) {
			DEBUG("Warning: Can not apply rule to GGROUPS\n");
		}
	}

	return ret;
}


/** apply rule */
int xos_policy_apply(struct rule_t *updated_rule)
{
	int ret = -1;

	if (!updated_rule)
		return ret;

	switch (updated_rule->r_type) {
	case XPLY_RT_MA:
		/* not support */
		ret = 0;
		break;
	case XPLY_RT_AC:
	case XPLY_RT_RU:
	case XPLY_RT_AP:
		if (updated_rule->r_subject.sub_type == XPLY_ST_ACT)
			ret =
			    bdb_operatedb_pwd(updated_rule, NULL,
					      (OPTPWDFUNC)
					      xos_policy_pwdupdate);
		else
			ret =
			    bdb_operatedb_grp(updated_rule, NULL,
					      (OPTGRPFUNC)
					      xos_policy_grpupdate);
		break;
	default:
		DEBUG("Don't support this type of rule \n");
	}

	return ret;
}

/* revoke the active rule */
int xos_policy_revoke(struct rule_t *revoked_rule)
{
	int ret = -1;

	if (!revoked_rule)
		return ret;

	switch (revoked_rule->r_type) {
	case XPLY_RT_MA:
		/* not support */
		ret = 0;
		break;
	case XPLY_RT_AC:
	case XPLY_RT_RU:
	case XPLY_RT_AP:
		/* since we currently support change rule effect dynamically,
		 * there is nothing to do.
		 */
		ret = 0;
		break;
	default:
		DEBUG("Don't support this type of rule \n");
	}

	return ret;
}


int xos_policy_vosetting(int uid_max, int uid_min, int gid_max,
			 int gid_min)
{
	struct rule_t *xrule = NULL;
	char *handler = NULL;
	int flag = 0;		// ret = -1;
	int uidmax_t, uidmin_t, gidmax_t, gidmin_t;

	if (ruleinfo_db_getbykey(-1, &xrule, &handler)) {
		DEBUG("Can not get VO setting parameters from database\n");
		xrule = (struct rule_t *) malloc(sizeof(struct rule_t));
		if (!xrule)
			return -1;
		memset(xrule, 0, sizeof(struct rule_t));

		if (uid_max < 1 || uid_min < 1 || gid_max < 1
		    || gid_min < 1)
			return -1;

		/* the follow line should be disabled */
		xrule->r_type = XPLY_RT_AP;
		sprintf(xrule->r_subject.sub_content_1, "%d", uid_max);
		sprintf(xrule->r_subject.sub_content_2, "%d", uid_min);
		sprintf(xrule->r_content.rc_driver.driver_name, "%d",
			gid_max);
		sprintf(xrule->r_content.rc_driver.driver_params, "%d",
			gid_min);

		flag = 1;
	} else {
		uidmax_t = atoi(xrule->r_subject.sub_content_1);
		uidmin_t = atoi(xrule->r_subject.sub_content_2);
		gidmax_t = atoi(xrule->r_content.rc_driver.driver_name);
		gidmin_t = atoi(xrule->r_content.rc_driver.driver_params);

		if (uid_max > 0 && uid_max != uidmax_t)
			sprintf(xrule->r_subject.sub_content_1, "%d",
				uid_max);
		if (uid_min > 0 && uid_min != uidmin_t)
			sprintf(xrule->r_subject.sub_content_2, "%d",
				uid_min);
		if (gid_max > 0 && gid_max != gidmax_t)
			sprintf(xrule->r_content.rc_driver.driver_name,
				"%d", gid_max);
		if (gid_min > 0 && gid_min != gidmin_t)
			sprintf(xrule->r_content.rc_driver.driver_params,
				"%d", gid_min);

		if (ruleinfo_db_delete(handler)) {
			DEBUG("Can not delete old parameters\n");
			return -1;
		}

		flag = 1;
	}

	if (flag) {
		if (ruleinfo_db_put_aux(xrule, -1, NULL)) {
			DEBUG("Can not set new parameters\n");
			return -1;
		}
	}
	return 0;
}


/* runtime function
 * add the new rule to databases and enable it to take effect.
 */
int xos_policy_addrule(struct rule_t *new_rule, char **rhandler)
{
	int ret = -1;

	if (!new_rule)
		return ret;

	/* add rule to database */
	if ((ret = ruleinfo_db_put(new_rule, rhandler))) {
		DEBUG("Can not store rule in database\n");
		return ret;
	}

	/* apply the new rule to all current users */
	if ((ret = xos_policy_apply(new_rule))) {
		DEBUG("Can not apply the new rule\n");
		return ret;
	}

	return 0;
}

/* runtime function
 * remove the rule from database and disable its effect.
 */
int xos_policy_removerule(char *rhandler)
{
	struct rule_t *revoked_rule = NULL;
	int ret = -1;

	if (!rhandler)
		return ret;

	if (ruleinfo_db_getbyhdler(rhandler, &revoked_rule)) {
		//DEBUG("No exist rule of handler '%s'\n",rhandler);
		return ret;
	}

	if (xos_policy_revoke(revoked_rule)) {
		DEBUG("Can not disable the rule's function in runtime\n");
		return ret;
	}

	if (ruleinfo_db_delete(rhandler)) {
		DEBUG("Can not delete rule with handler '%s'\n", rhandler);
		return ret;
	}

	ret = 0;

	return ret;

}

/* runtime function
 * remove the rule from database and disable its effect.
 */
int xos_policy_modifyrule(char *rhandler, struct rule_t *new_rule)
{
	int ret = -1;
	char *new_handler = NULL;


	if (!rhandler || !new_rule)
		return ret;

	xos_policy_removerule(rhandler);

	if (xos_policy_addrule(new_rule, &new_handler)) {
		DEBUG("Can not enable modified rule\n");
		return ret;
	}

	if (new_handler)
		snprintf(rhandler, strlen(new_handler) + 1, "%s",
			 new_handler);

	return 0;

}

/* Search rule via rule's subject and rule's object */
int xos_policy_searchrule(enum XPLY_RT rtype, struct subject_t *r_sub,
			  struct object_t *r_obj, struct rule_t **ret_rule,
			  char **ret_handler)
{
	int i, ret;
	struct rule_t *xrule = NULL;
	char *xhandler = NULL;

	ret = -1;
	for (i = 1; i < ruleinfo_db_newhandler(); i++) {

		xrule = NULL;
		if (ruleinfo_db_getbykey(i, &xrule, &xhandler)) {
			DEBUG("Can not get rule !\n");
			break;
		}
		if (!xrule)
			continue;


		/* match the rule */
		if (xrule->r_type != rtype)
			continue;
		else if (!memcmp(r_sub, &(xrule->r_subject), sizeof(r_sub))
			 && !memcmp(r_obj, &(xrule->r_object),
				    sizeof(r_obj))) {
			ret = 0;
			break;
		}
	}

	if (ret == 0) {
		if (ret_rule)
			*ret_rule = xrule;
		if (ret_handler)
			*ret_handler = xhandler;
	}

	return ret;
}

int xos_policy_ACreject(struct subject_t *r_sub, int *in_blacklist)
{
	struct rule_t *matched_rule = NULL;
	int ret = -1;

	if (!r_sub || !in_blacklist)
		return ret;

	if (xos_policy_findrule(XPLY_RT_AC, r_sub, &matched_rule, NULL)) {
		*in_blacklist = 0;
		return 0;
	}

	if (matched_rule) {
		if (matched_rule->r_content.rc_perm == 0)
			*in_blacklist = 1;
		else
			*in_blacklist = 0;

		return 0;
	}

	return ret;
}


int xos_policy_setriority(int from, int to)
{
	return ruleinfo_db_change_location(from, to);
}
