/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "xos_utils.h"
#include "xos_rquota.h"


int xos_resquota_init(ResQuota * rquota)
{
	memset(rquota, 'X', sizeof(*rquota));

	rquota->rname = NULL;
	rquota->f_group = 0;
	rquota->rcpu = -1;
	rquota->f_cpu = 1;
	rquota->rmem = -1;
	rquota->f_mem = 1;
	rquota->rdisk = -1;
	rquota->f_disk = 1;

	return 0;

}

int xos_resquota_setup(ResQuota * rquota)
{
	int ret = -1;
	char *path = NULL;
	char buffer[2048], username[128];
	FILE *fp;

	if (rquota == NULL)
		return -1;
	if (getuid() != 0) {
		DEBUG("Root to setup quota limition");
		return -1;
	}

	if (rquota->rname == NULL) {
		DEBUG("No account to limit quota");
		return -1;
	} else if (rquota->rcpu == -1 && rquota->rmem == -1
		   && rquota->rdisk == -1)
		return 0;	/* No any quota to setup */

	path = (char *) malloc(strlen(QUOTA_DIR) + strlen(QUOTA_FILE) + 2);
	sprintf(path, "%s/%s", QUOTA_DIR, QUOTA_FILE);

	if (rquota->f_group)
		sprintf(username, "@%s", rquota->rname);
	else
		sprintf(username, "%s", rquota->rname);

	if (!(fp = fopen(path, "a"))) {
		DEBUG("Can not open %s", path);
		return -1;
	}

	if (rquota->rcpu != -1) {
		if (rquota->f_cpu)
			sprintf(buffer, "%-16s\thard\tcpu\t\t%ld\n",
				username, rquota->rcpu);
		else
			sprintf(buffer, "%-16s\tsoft\tcpu\t\t%ld\n",
				username, rquota->rcpu);
		fputs(buffer, fp);
	}
	if (rquota->rmem != -1) {
		if (rquota->f_mem)
			sprintf(buffer, "%-16s\thard\tmem\t\t%ld\n",
				username, rquota->rmem);
		else
			sprintf(buffer, "%-16s\tsoft\tmem\t\t%ld\n",
				username, rquota->rmem);
		fputs(buffer, fp);
	}
	if (rquota->rdisk != -1) {
		if (rquota->f_disk)
			sprintf(buffer, "%-16s\thard\tdisk\t\t%ld\n",
				username, rquota->rdisk);
		else
			sprintf(buffer, "%-16s\tsoft\tdisk\t\t%ld\n",
				username, rquota->rdisk);
		fputs(buffer, fp);
	}

	fclose(fp);

	ret = 0;


	return ret;
}

int xos_resquota_clear(ResQuota * rquota)
{
	int ret = -1, rq = 0, clearall = 0;
	char path[1024], bakpath[1024];
	char buffer[1024], buffer2[1024], username[128];
	FILE *fp1 = NULL, *fp2 = NULL;

	if (rquota == NULL)
		return -1;
	if (getuid() != 0) {
		DEBUG("Root to setup quota limition");
		return -1;
	}

	if (rquota->rname == NULL) {
		DEBUG("No account to limit quota");
		return -1;
	} else if (rquota->rcpu != -1)
		rq++;
	else if (rquota->rmem != -1)
		rq++;
	else if (rquota->rdisk != -1)
		rq++;
	else
		clearall = 1;
	sprintf(path, "%s/%s", QUOTA_DIR, QUOTA_FILE);
	sprintf(bakpath, "%s/.%s.bak", QUOTA_DIR, QUOTA_FILE);
	sprintf(buffer, "cp %s %s", path, bakpath);
	system(buffer);

	if (!(fp1 = fopen(bakpath, "r"))) {
		DEBUG("Can not open %s", bakpath);
		return -1;
	}

	if (!(fp2 = fopen(path, "w"))) {
		DEBUG("Can not open %s", path);
		goto error;
	}

	if (rquota->f_group)
		sprintf(username, "@%s", rquota->rname);
	else
		sprintf(username, "%s", rquota->rname);

	while (fgets(buffer, 1023, fp1)) {
		if (!clearall && rq) {
			if (rquota->f_cpu)
				sprintf(buffer2,
					"%-16s\thard\tcpu\t\t%ld\n",
					username, rquota->rcpu);
			else
				sprintf(buffer2,
					"%-16s\tsoft\tcpu\t\t%ld\n",
					username, rquota->rcpu);
			if (!strncmp(buffer, buffer2, strlen(buffer2))) {
				continue;
				rq--;
			} else {
				if (rquota->f_mem)
					sprintf(buffer2,
						"%-16s\thard\tmem\t\t%ld\n",
						username, rquota->rmem);
				else
					sprintf(buffer2,
						"%-16s\tsoft\tmem\t\t%ld\n",
						username, rquota->rmem);
				if (!strncmp
				    (buffer, buffer2, strlen(buffer2))) {
					continue;
					rq--;
				} else {
					if (rquota->f_disk)
						sprintf(buffer2,
							"%-16s\thard\tdisk\t\t%ld\n",
							username,
							rquota->rdisk);
					else
						sprintf(buffer2,
							"%-16s\thard\tdisk\t\t%ld\n",
							username,
							rquota->rdisk);
					if (!strncmp
					    (buffer, buffer2,
					     strlen(buffer2))) {
						continue;
						rq--;
					}
				}
			}
		}

		if (clearall) {
			sprintf(buffer2, "%-16s", username);
			if (!strcmp(buffer2, buffer))
				continue;
		}

		fputs(buffer, fp2);
	}

	ret = 0;
      error:
	if (fp1)
		fclose(fp1);
	if (fp2)
		fclose(fp2);
	return ret;
}

int xos_resquota_modify(ResQuota * newrquota, int flags)
{
	int ret = -1, i;
	char path[1024], bakpath[1024];
	char buffer[1024], username[128];
	char cpu_buf[1024], mem_buf[1024], disk_buf[1024];
	int modified[3];
	FILE *fp1 = NULL, *fp2 = NULL;

	if (newrquota == NULL)
		return -1;
	if (getuid() != 0) {
		DEBUG("Root to setup quota limition");
		return -1;
	}

	if (newrquota->rname == NULL) {
		DEBUG("No account to limit quota");
		return -1;
	}
	sprintf(path, "%s/%s", QUOTA_DIR, QUOTA_FILE);
	sprintf(bakpath, "%s/.%s.bak", QUOTA_DIR, QUOTA_FILE);
	sprintf(buffer, "cp %s %s", path, bakpath);
	system(buffer);

	if (!(fp1 = fopen(bakpath, "r"))) {
		DEBUG("Can not open %s", bakpath);
		return -1;
	}

	if (!(fp2 = fopen(path, "w"))) {
		DEBUG("Can not open %s", path);
		goto error;
	}

	if (newrquota->f_group)
		sprintf(username, "@%s", newrquota->rname);
	else
		sprintf(username, "%s", newrquota->rname);

	if (newrquota->f_cpu)
		sprintf(cpu_buf, "%-16s\thard\tcpu\t\t%ld\n", username,
			newrquota->rcpu);
	else
		sprintf(cpu_buf, "%-16s\tsoft\tcpu\t\t%ld\n", username,
			newrquota->rcpu);

	if (newrquota->f_mem)
		sprintf(mem_buf, "%-16s\thard\tmem\t\t%ld\n", username,
			newrquota->rmem);
	else
		sprintf(mem_buf, "%-16s\tsoft\tmem\t\t%ld\n", username,
			newrquota->rmem);

	if (newrquota->f_disk)
		sprintf(disk_buf, "%-16s\thard\tdisk\t\t%ld\n", username,
			newrquota->rdisk);
	else
		sprintf(disk_buf, "%-16s\thard\tdisk\t\t%ld\n", username,
			newrquota->rdisk);

	for (i = 0; i < 3; i++)
		modified[i] = 0;
	while (fgets(buffer, 1023, fp1)) {

		if ((flags & QUOTA_CHCPU) && strstr(buffer, username)
		    && strstr(buffer, "cpu")) {
			if (newrquota->rcpu != -1) {
				strncpy(buffer, cpu_buf, 1023);
				modified[0] = 1;
			} else
				continue;

		} else if ((flags & QUOTA_CHMEM)
			   && strstr(buffer, username)
			   && strstr(buffer, "mem")) {
			if (newrquota->rmem != -1) {
				strncpy(buffer, mem_buf, 1023);
				modified[1] = 1;
			} else
				continue;
		} else if ((flags & QUOTA_CHDISK)
			   && strstr(buffer, username)
			   && strstr(buffer, "disk")) {
			if (newrquota->rdisk != -1) {
				strncpy(buffer, disk_buf, 1023);
				modified[2] = 1;
			} else
				continue;
		}

		fputs(buffer, fp2);
	}

	if ((flags & QUOTA_CHCPU) && newrquota->rcpu != -1
	    && modified[0] == 0)
		fputs(cpu_buf, fp2);
	if ((flags & QUOTA_CHMEM) && newrquota->rmem != -1
	    && modified[1] == 0)
		fputs(mem_buf, fp2);
	if ((flags & QUOTA_CHDISK) && newrquota->rdisk != -1
	    && modified[2] == 0)
		fputs(disk_buf, fp2);

	ret = 0;
      error:
	if (fp1)
		fclose(fp1);
	if (fp2)
		fclose(fp2);
	return ret;
}

/* Currently, only role can be interpreted 
 * policy rules:
 * role ==>>  CPU=????:MEM=????:DISK=????
 */
int xos_resquota_interpret(char *role, ResQuota * rquota)
{
	FILE *policyfile_fp;
	char buf[1024], *cpup = NULL, *memp = NULL, *diskp = NULL;
	char paras[128], values[128];
	int ret = -1;
	char path[1024];

	sprintf(path, "%s/%s", POLICY_DIR, POLICY_FILE);
	policyfile_fp = fopen(path, "r");
	if (policyfile_fp == NULL) {
		fprintf(stderr, "Error opening file '%s'\n", path);
		goto err;
	}

	if (!rquota) {
		printf("pointer is null\n");
		goto err;
	}

	while (fgets(buf, 1023, policyfile_fp) != NULL && ret) {
		if (*buf == '#')
			continue;
		sscanf(buf, "%s %s[^\n]", paras, values);

		if (!strncmp(paras, role, strlen(role))) {
			cpup = strstr(values, ":CPU=");
			if (cpup)
				cpup += strlen(":CPU=");

			memp = strstr(values, ":MEM=");
			if (memp) {
				*memp = '\0';
				memp += strlen(":MEM=");
			}

			diskp = strstr(memp, ":DISK=");
			if (diskp) {
				*diskp = '\0';
				diskp += strlen(":DISK=");
			}

			if (cpup)
				rquota->rcpu = atoi(cpup);
			if (memp)
				rquota->rmem = atoi(memp);
			if (diskp)
				rquota->rdisk = atoi(diskp);

			ret = 0;
		}
	}

      err:
	return ret;
}
