/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <openssl/asn1.h>
#include <openssl/conf.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/pem.h>
#include <openssl/err.h>
#include <string.h>

#include "xos_utils.h"
#include "xos_cert.h"

int xos_proxy_init_cmd(char *usercert, char *userkey, char *proxyfile)
{
	char cmd_buffer[1024];
	//struct stat file_stat;
	int ret = 0;


	sprintf(cmd_buffer, "createproxy.sh %s %s %s", usercert, userkey,
		proxyfile);
	ret = system(cmd_buffer);

	return ret;
}

int xos_proxy_delegate_cmd(char *current_proxy, char *next_proxy, int days)
{
	char cmd_buffer[1024];
	int ret = 0;
	sprintf(cmd_buffer, "delegateproxy.sh %s %s %d", current_proxy,
		next_proxy, days);
	ret = system(cmd_buffer);

	return ret;
}

int xos_proxy_delegate_exec(char *certfile, char *keyfile, int days,
			    char *attrs, char *proxyfile)
{
	return create_proxy(certfile, keyfile, days, attrs, proxyfile);
}

int xos_cert_attribute_load(char *proxy_file, char *attrs)
{
	char buffer[128];
	char buffer1[128];
	BIO *bufferbio = NULL;
	X509 *cert;
	FILE *inf;
	int i, count, non_found = -1;
	X509_EXTENSION *ext;
	X509V3_add_standard_extensions();
	ERR_load_crypto_strings();

	if (!proxy_file)
		return -1;

	if (!(inf = fopen(proxy_file, "r"))) {
		fprintf(stderr, "Can't open %s\n", proxy_file);
		return -1;
	}
	if (!(cert = PEM_read_X509(inf, NULL, NULL, NULL))) {
		fprintf(stderr, "Can't read certificate %s\n", proxy_file);
		ERR_print_errors_fp(stderr);
		return -1;
	}
	fclose(inf);
	bufferbio = BIO_new(BIO_s_mem());
	count = X509_get_ext_count(cert);

	for (i = 0; i < count && non_found; i++) {
		ext = X509_get_ext(cert, i);
		sprintf(buffer1, "%s",
			OBJ_nid2ln(OBJ_obj2nid(ext->object)));
		if (strcmp(buffer1, "Netscape Comment"))
			continue;
		if (!X509V3_EXT_print(bufferbio, ext, 0, 0))
			ERR_print_errors_fp(stderr);
		BIO_gets(bufferbio, buffer, 128);
		strcpy(attrs, buffer);
		non_found = 0;

	}
	return non_found;
}

int xos_cert_attribute_get(X509 * cert, char *attrs)
{
	char buffer[128];
	char buffer1[128];
	BIO *bufferbio = NULL;
	//X509 *cert;
	//FILE *inf;
	int i, count, non_found = -1;
	X509_EXTENSION *ext;
	X509V3_add_standard_extensions();
	ERR_load_crypto_strings();
#if 0
	if (!proxy_file)
		return -1;

	if (!(inf = fopen(proxy_file, "r"))) {
		fprintf(stderr, "Can't open %s\n", proxy_file);
		return -1;
	}
	if (!(cert = PEM_read_X509(inf, NULL, NULL))) {
		fprintf(stderr, "Can't read certificate %s\n", proxy_file);
		ERR_print_errors_fp(stderr);
		return -1;
	}
	fclose(inf);
#endif
	bufferbio = BIO_new(BIO_s_mem());
	count = X509_get_ext_count(cert);

	for (i = 0; i < count && non_found; i++) {
		ext = X509_get_ext(cert, i);
		sprintf(buffer1, "%s",
			OBJ_nid2ln(OBJ_obj2nid(ext->object)));
		if (strcmp(buffer1, "Netscape Comment"))
			continue;
		if (!X509V3_EXT_print(bufferbio, ext, 0, 0))
			ERR_print_errors_fp(stderr);
		BIO_gets(bufferbio, buffer, 128);
		strcpy(attrs, buffer);
		non_found = 0;

	}
	return non_found;
}


int xos_cert_expirytime_get(X509 * cert, time_t * not_after)
{
	if (!cert)
		return -1;
#if 0
	ASN1_TIME *after_time;
	time_t expire;

	after_time = X509_get_notAfter(cert);
	expire = ASN1_UTCTIME_get(not_after);

	*not_after = expire;
#endif
	printf(" not after %s\n", X509_get_notAfter(cert)->data);

	return 0;
}


int xos_cert_extensions_getattr(char *ext, char *attr_label,
				char *attr_value)
{
	char *p = NULL;
	int i;

	if (!ext || !attr_value || !attr_label)
		return -1;
	p = strstr(ext, attr_label);
	if (!p)
		return -1;

	i = 0;
	p += strlen(attr_label);
	while (*p == '\r' || *p == '\n' || *p == 32
	       || *p == '.' || *p == ':' || *p == '$')  /* get rid of the $ */
		p++;
	while (*(p + i) != '\n' && *(p + i) != '\0') {
		*(attr_value + i) = *(p + i);
		i++;
	}
	*(attr_value + i) = '\0';
	if (i > 0)
		return 0;
	else
		return -1;
}


int cert_extensions_print(BIO * bp, char *title,
			  STACK_OF(X509_EXTENSION) * exts, unsigned index,
			  unsigned long flag, int indent)
{
	int i, j;
	if (sk_X509_EXTENSION_num(exts) <= 0)
		return 1;
	if (title) {
		BIO_printf(bp, "%*s%s:", indent, "", title);
		indent += 4;
	}

	DEBUG("%d extension:", sk_X509_EXTENSION_num(exts));
	if (sk_X509_EXTENSION_num(exts) <= index || index <= 0) {
		for (i = 0; i < sk_X509_EXTENSION_num(exts); i++) {
			ASN1_OBJECT *obj;
			X509_EXTENSION *ex;
			ex = sk_X509_EXTENSION_value(exts, i);
			if (indent
			    && BIO_printf(bp, "%*s", indent, "") <= 0)
				return 0;
			obj = X509_EXTENSION_get_object(ex);
			i2a_ASN1_OBJECT(bp, obj);
			j = X509_EXTENSION_get_critical(ex);
			if (BIO_printf(bp, ": %s\n", j ? "critical" : "")
			    <= 0)
				return 0;
			if (!X509V3_EXT_print(bp, ex, flag, indent + 4)) {
				BIO_printf(bp, "%*s", indent + 4, "");
				M_ASN1_OCTET_STRING_print(bp, ex->value);
			}
			if (BIO_write(bp, "\n", 1) <= 0)
				return 0;
		}
	} else {
		indent = 0;
		ASN1_OBJECT *obj;
		X509_EXTENSION *ex;
		ex = sk_X509_EXTENSION_value(exts, index);
		if (indent && BIO_printf(bp, "%*s", indent, "") <= 0)
			return 0;
		obj = X509_EXTENSION_get_object(ex);
		i2a_ASN1_OBJECT(bp, obj);
		j = X509_EXTENSION_get_critical(ex);
		if (BIO_printf(bp, ": %s", j ? "critical" : "") <= 0)
			return 0;
		if (!X509V3_EXT_print(bp, ex, flag, indent)) {
			//BIO_printf(bp, "%*s", indent + 4, "");
			M_ASN1_OCTET_STRING_print(bp, ex->value);
		}
		if (BIO_write(bp, "", 1) <= 0)
			return 0;
	}
	return 1;
}

int xos_cert_attribute_get_ax(X509 * cert, char *ext_buf)
{
	BIO *mem = BIO_new(BIO_s_mem());
	char *ptr = NULL;
	int length = 0;
	//char ext_buf[512]={'\0'};
	//char attr_buf[512] = {'\0'};

	if (!cert || !ext_buf)
		return -1;

	//BIO *STDout=BIO_new_fp(stdout,BIO_NOCLOSE);
	STACK_OF(X509_EXTENSION) * extensions =
	    cert->cert_info->extensions;
	cert_extensions_print(mem, NULL, extensions, 0, 0, 0);

	length = BIO_get_mem_data(mem, &ptr);
	memcpy(ext_buf, ptr, length);
	//printf("length=%d,extension=:\n%s\n",length,ext_buf);
	//getValueFromStr(ext_buf,"CertType",certType);
	//printf("CertType=:%s\n",certType);


	//if(cert_extensions_getattr(ext_buf, "1.34.5.0.14.2", attr_buf))
	//return -1;

	//printf("VO = '%s'\n",attr_buf);
	//*attrs = strdup(attr_buf);
	//strcpy(attrs_buf,attr_buf);
	return 0;
}
