/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/x509_vfy.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/rand.h>
#include <time.h>

#include "xos_utils.h"
#include "xos_sslutils.h"

#if 0
void
handle_error (const char *file, int lineno, const char *msg)
{
  fprintf (stderr, "** %s:%i %s\n", file, lineno, msg);
  ERR_print_errors_fp (stderr);
  exit (-1);
}

#define int_error(msg) handle_error(__FILE__, __LINE__, msg)
`

/* these are defintions to make the example simpler */
#define CA_FILE "xtreemos-ca.crt"
#define CA_DIR "../testcerts/xosca/"
#define CRL_FILE "CRLfile.pem"
#define CLIENT_CERT "../testcerts/xtreemos-anqin-unlimited.pem"
#endif

void _seed_prng()
{
	time_t rawtime;
	struct tm* timeinfo;
	
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	RAND_seed(asctime(timeinfo), 4);
}

int _verify_callback (int ok, X509_STORE_CTX * stor)
{
	if (!ok)
	fprintf (stderr, "Error: %s\n",
		X509_verify_cert_error_string (stor->error));
	return ok;
}

int xosssl_cdacert_verify(X509 *cert, char *cda_dir, char *cda_file)
{

	X509_STORE *store;
	X509_LOOKUP *lookup;
	X509_STORE_CTX *verify_ctx;
	char *CDA_dir, *CDA_file;

	if(!cert || !cda_dir || !cda_file)
	return SSL_ERROR;
	
	OpenSSL_add_all_algorithms ();
	ERR_load_crypto_strings ();
	_seed_prng ();
	
	CDA_dir = CDA_file = NULL;
	
	CDA_dir = cda_dir;
	CDA_file = (char*)malloc(strlen(cda_dir) + strlen(cda_file)+2);
	if(CDA_file == NULL) return SSL_ERROR;
	sprintf(CDA_file,"%s/%s",cda_dir,cda_file);
	

	/* create the cert store and set the verify callback */
	if (!(store = X509_STORE_new ())) {
		fprintf (stderr, "Error creating X509_STORE_CTX object\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
	
	X509_STORE_set_verify_cb_func (store, _verify_callback);
	
	/* load the CA certificates and CRLs */
	if (X509_STORE_load_locations (store, CDA_file, CDA_dir) != 1) {
		fprintf (stderr, "Error loading the CA file or directory\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
			
	if (X509_STORE_set_default_paths (store) != 1) {
		fprintf (stderr, "Error loading the system-wide CA certificates\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
		
	if (!(lookup = X509_STORE_add_lookup (store, X509_LOOKUP_file ()))) {
		fprintf (stderr, "Error creating X509_LOOKUP object\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
	
	//if (X509_load_crl_file (lookup, CRL_FILE, X509_FILETYPE_PEM) != 1) {
	//	fprintf (stderr, Error reading the CRL file\n");
	//	ERR_print_errors_fp (stderr);
	//	return -1;
	// }
	
	/* enabling verification against CRLs is not possible
	in prior versions */
#if (OPENSSL_VERSION_NUMBER > 0x00907000L)
	/* set the flags of the store so that CRLs are consulted */
	//X509_STORE_set_flags (store, X509_V_FLAG_CRL_CHECK |
	//			X509_V_FLAG_CRL_CHECK_ALL);
#endif
	
	/* create a verification context and initialize it */
	if (!(verify_ctx = X509_STORE_CTX_new ())) {
		fprintf(stderr, "Error creating X509_STORE_CTX object\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
	/* X509_STORE_CTX_init did not return an error condition
	in prior versions */
#if (OPENSSL_VERSION_NUMBER > 0x00907000L)
	if (X509_STORE_CTX_init (verify_ctx, store, cert, NULL) != 1) {
		fprintf (stderr, "Error initializing verification context\n");
		ERR_print_errors_fp (stderr);
		return SSL_ERROR;
	}
#else
	X509_STORE_CTX_init (verify_ctx, store, cert, NULL);
#endif
	
	/* verify the certificate */
	if (X509_verify_cert (verify_ctx) != 1) {
		fprintf (stderr, "Error verifying the certificate\n");
		return SSL_ERROR;
	}
	
	return SSL_SUCCESS;

	
}



