/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#include "xos_key.h"
#include "xos_sslutils.h"
#include "xos_rquota.h"
#include "xos_cert.h"
#include "xos_interfaces.h"


/* Certificate-related interfaces */
int xos_proxy_init(char *usercert, char *userkey, char *proxy_file)
{
	int ret;

	ret = xos_proxy_init_cmd(usercert, userkey, proxy_file);

	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}


int xos_proxy_delegate(char *current_proxy, char *next_proxy, int days)
{
	int ret;

	ret = xos_proxy_delegate_cmd(current_proxy, next_proxy, days);

	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}



int xos_proxy_destroy(char *proxyfile)
{
	int ret;

	ret = xosssl_proxydestroy(proxyfile);

	return ret == SSL_SUCCESS ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cdaproxy_verify(char *cadir, char *proxyfile, int loglevel)
{
	unsigned long ret;

	ret = xosssl_cdaproxy_verify(cadir, proxyfile, loglevel);


	if (ret == X509_V_OK)
		return XOS_SUCCESS;
	else
		return XOS_ERROR;
}

int xos_cdacert_verify(unsigned char *cert_buf, char *cda_dir, char *cda_file)
{
	XOS_CREDS *creds = xos_creds_new();
	int ret;

	if(!creds) return XOS_ERROR;

	if(xos_cert_loadfrombuffer(cert_buf,strlen((char*)cert_buf),creds))
		return XOS_ERROR;

	ret = xosssl_cdacert_verify(creds->certificate,cda_dir,cda_file);
	
	return ret == SSL_SUCCESS?XOS_SUCCESS:XOS_ERROR;
}

int xos_cert_attributes_set(char *proxy_file, char *attrs)
{
	int ret;
	char cmd_buffer[128];
	char *tmp_proxyname = "/tmp/proxyname.setattrs";

	ret =
	    xos_proxy_delegate_exec(proxy_file, proxy_file, 10, attrs,
				    tmp_proxyname);

	if (ret == 0) {
		sprintf(cmd_buffer, "cp -f %s %s", tmp_proxyname,
			proxy_file);
		system(cmd_buffer);
		sprintf(cmd_buffer, "rm -f %s", tmp_proxyname);
		system(cmd_buffer);
	}

	return ret;
}



int xos_cert_attributes_load(char *proxy_file, char *attrs)
{
	int ret;

	ret = xos_cert_attribute_load(proxy_file, attrs);
	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_attributes_fetch(X509 * cert, char *attrs)
{
	int ret;

	ret = xos_cert_attribute_get(cert, attrs);
	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_attributes_fetch_ext(X509 * cert, char *exts)
{
	int ret = XOS_ERROR;
	ret = xos_cert_attribute_get_ax(cert, exts);
	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}


int xos_cert_attributes_fetch_attr(char *ext, char *attr_label,
				   char *attr_value)
{
	int ret = XOS_ERROR;
	ret = xos_cert_extensions_getattr(ext, attr_label, attr_value);
	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_lifetime_set(char *proxy_buffer, long seconds)
{
	int ret = XOS_ERROR;

	return ret;
}

int xos_cert_lifetime_get(X509 * proxy_cert, time_t * not_after)
{
	int ret;

	ret = xos_cert_expirytime_get(proxy_cert, not_after);

	return ret == 0 ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_subject_set()
{
	int ret = XOS_ERROR;

	return ret;
}

int xos_cert_subject_get(X509 * cert, char *buffer, int buflen)
{
	int ret = SSL_ERROR;

	ret = xosssl_getsubjectfromcert(cert, buffer, buflen);

	if (ret == SSL_SUCCESS)
		return XOS_SUCCESS;
	else
		return XOS_ERROR;

}

/* key-related interfaces */

XOS_ASYMKEY *xos_asymkey_gen(int type, u_int bits)
{
	return (XOS_ASYMKEY *) key_generate(type, bits);
}

XOS_ASYMKEY *xos_asymkey_getpubkey(const XOS_ASYMKEY * key)
{
	return key_from_private(key);
}

int xos_asymkey_keytype(char *name)
{
	return key_type_from_name(name);
}

int xos_asymkey_sign(const XOS_ASYMKEY * key, u_char ** sigp, u_int * lenp,
		     const u_char * data, u_int datalen)
{
	int ret;

	ret = xos_key_sign(key, sigp, lenp, data, datalen);

	return (ret == -1) ? XOS_ERROR : XOS_SUCCESS;
}

int xos_asymkey_verify(const XOS_ASYMKEY * key, const u_char * signature,
		       u_int signaturelen, const u_char * data,
		       u_int datalen)
{
	int ret;

	ret = xos_key_verify(key, signature, signaturelen, data, datalen);


	return (ret == 1) ? XOS_SUCCESS : XOS_ERROR;
}


int xos_symkey_gen(XOS_SYMKEY * k)
{
	xos_symkey_generate(k);

	return XOS_SUCCESS;
}


int xos_symkey_encrypt(unsigned char *dataToEncrypt, int dataToEncryptLen,
		       unsigned char *encryptedData, int *encryptedDataLen,
		       XOS_SYMKEY * k)
{
	int ret;

	ret = xosssl_encryptWithSymKey(dataToEncrypt, dataToEncryptLen,
				       encryptedData, encryptedDataLen, k);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_symkey_decrypt(unsigned char *dataToDecrypt, int dataToDecryptLen,
		       unsigned char *decryptedData, int *decryptedDataLen,
		       XOS_SYMKEY * k)
{
	int ret;

	ret = xosssl_decryptWithSymKey(dataToDecrypt, dataToDecryptLen,
				       decryptedData, decryptedDataLen, k);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_symkey_prt(XOS_SYMKEY * k)
{

	xos_symkey_print(k);

	return XOS_SUCCESS;
}


/* I/O-related interfaces */
XOS_CREDS *xos_creds_new()
{
	return ssl_credentials_new();
}

void xos_creds_free(XOS_CREDS * creds)
{
	xosssl_credsfreecontents(creds);
}


int xos_cert_loadfromfile(XOS_CREDS * creds, const char *path)
{
	int ret;

	ret = xosssl_loadcertfromfile(creds, path);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_savetofile(XOS_CREDS * creds,
			const char *path, const char *pass_phrase)
{
	int ret;

	ret = xosssl_proxytofile(creds, path, pass_phrase);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_prvkeyfromfile(XOS_CREDS * creds,
			    const char *path,
			    const char *pass_phrase,
			    const char *pass_phrase_prompt)
{
	int ret;

	ret =
	    xosssl_getprivatekeyfromfile(creds, path, pass_phrase,
					 pass_phrase_prompt);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_proxy_loadfromfile(XOS_CREDS * creds, const char *path,
			   const char *pass_phrase)
{
	int ret;

	ret = xosssl_loadproxyfromfile(creds, path, pass_phrase);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_proxy_loadfrombuffer(XOS_CREDS * creds,
			     const unsigned char *buffer, int buffer_len,
			     const char *pass_phrase)
{
	int ret;

	ret = xosssl_proxyfrompem(creds, buffer, buffer_len, pass_phrase);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_loadfrombuffer(unsigned char *buffer, int buffer_length,
			    XOS_CREDS * creds)
{
	int ret;

	ret = xosssl_certfrombuffer(buffer, buffer_length, creds);

	return (ret == SSL_SUCCESS) ? XOS_SUCCESS : XOS_ERROR;
}

int xos_cert_buffromfile(const char *path, unsigned char **pbuffer,
			 int *pbuffer_len)
{
	int ret;

	ret = xosssl_bufferfromfile(path, pbuffer, pbuffer_len);

	return (ret == 0) ? XOS_SUCCESS : XOS_ERROR;
}


/* Service-related interfaces */
int xos_service_identity_get()
{
	int ret = XOS_ERROR;

	return ret;
}

int xos_service_attr_get()
{
	int ret = XOS_ERROR;

	return ret;
}

int xos_service_prolicy_get()
{
	int ret = XOS_ERROR;

	return ret;
}

#if 0
/* Mapping-related interfaces */

int xos_ams_mapping(char *mapped_dn, char *mapped_vo, char *mapped_role,
		    char *mapped_subgroup, XOS_GPASSWD * gpwd,
		    XOS_GGROUPS * primarygrp)
{
	int ret = XOS_ERROR;

#if XOS_STD_CODE
	return ret;
#else
	ret = xos_mapping2(mapped_dn, mapped_vo, mapped_role,
			   mapped_subgroup, gpwd, primarygrp);
	return (ret == 0) ? XOS_SUCCESS : XOS_ERROR;
#endif


}

int xos_ams_imapping();
#endif

/* Quota-related interfaces */
int xos_rquota_init(XOS_RQUOTA * rquota)
{
	int ret;

	ret = xos_resquota_init(rquota);

	return ret < 0 ? XOS_ERROR : XOS_SUCCESS;
}

int xos_rquota_setup(XOS_RQUOTA * rquota)
{
	int ret;

	ret = xos_resquota_setup(rquota);

	return ret < 0 ? XOS_ERROR : XOS_SUCCESS;
}

int xos_rquota_modify(XOS_RQUOTA * rquota, int flags)
{
	int ret;

	ret = xos_resquota_modify(rquota, flags);

	return ret < 0 ? XOS_ERROR : XOS_SUCCESS;
}

int xos_rquota_clear(XOS_RQUOTA * rquota)
{
	int ret;

	ret = xos_resquota_clear(rquota);

	return ret < 0 ? XOS_ERROR : XOS_SUCCESS;
}

int xos_rquota_interpret(char *user_role, XOS_RQUOTA * rquota)
{
	int ret;

	ret = xos_resquota_interpret(user_role, rquota);

	return ret < 0 ? XOS_ERROR : XOS_SUCCESS;
}
